@@ -1,6 +1,6 @@
 
 /* Compiler implementation of the D programming language
- * Copyright (C) 1999-2020 by The D Language Foundation, All Rights Reserved
+ * Copyright (C) 1999-2021 by The D Language Foundation, All Rights Reserved
  * written by Walter Bright
  * http://www.digitalmars.com
  * Distributed under the Boost Software License, Version 1.0.
@@ -27,7 +27,6 @@ class VarDeclaration;
 class FuncDeclaration;
 class FuncLiteralDeclaration;
 class CtorDeclaration;
-class NewDeclaration;
 class Dsymbol;
 class ScopeDsymbol;
 class Expression;
@@ -58,7 +57,7 @@ Expression *defaultInit(Type *mt, const Loc &loc);
 
 void expandTuples(Expressions *exps);
 bool isTrivialExp(Expression *e);
-bool hasSideEffect(Expression *e);
+bool hasSideEffect(Expression *e, bool assumeImpureCalls = false);
 bool canThrow(Expression *e, FuncDeclaration *func, bool mustNotThrow);
 
 typedef unsigned char OwnedBy;
@@ -74,6 +73,19 @@ enum
 #define WANTexpand 1    // expand const/immutable variables if possible
 #endif
 
+/**
+ * Specifies how the checkModify deals with certain situations
+ */
+enum class ModifyFlags
+{
+    /// Issue error messages on invalid modifications of the variable
+    none,
+    /// No errors are emitted for invalid modifications
+    noError = 0x1,
+    /// The modification occurs for a subfield of the current variable
+    fieldAssign = 0x2,
+};
+
 class Expression : public ASTNode
 {
 public:
@@ -112,8 +124,7 @@ public:
     virtual bool checkType();
     virtual bool checkValue();
     bool checkDeprecated(Scope *sc, Dsymbol *s);
-    virtual int checkModifiable(Scope *sc, int flag = 0);
-    virtual Expression *toBoolean(Scope *sc);
+    virtual int checkModifiable(Scope *sc, ModifyFlags flag = ModifyFlags::none);
     virtual Expression *addDtorHook(Scope *sc);
     Expression *addressOf();
     Expression *deref();
@@ -161,7 +172,7 @@ public:
     TraitsExp* isTraitsExp();
     HaltExp* isHaltExp();
     IsExp* isExp();
-    CompileExp* isCompileExp();
+    MixinExp* isMixinExp();
     ImportExp* isImportExp();
     AssertExp* isAssertExp();
     DotIdExp* isDotIdExp();
@@ -227,6 +238,7 @@ public:
     EqualExp* isEqualExp();
     IdentityExp* isIdentityExp();
     CondExp* isCondExp();
+    GenericExp* isGenericExp();
     DefaultInitExp* isDefaultInitExp();
     FileInitExp* isFileInitExp();
     LineInitExp* isLineInitExp();
@@ -234,6 +246,7 @@ public:
     FuncInitExp* isFuncInitExp();
     PrettyFuncInitExp* isPrettyFuncInitExp();
     ClassReferenceExp* isClassReferenceExp();
+    virtual BinAssignExp* isBinAssignExp();
 
     void accept(Visitor *v) { v->visit(this); }
 };
@@ -325,7 +338,7 @@ public:
     Dsymbol *s;
     bool hasOverloads;
 
-    Expression *syntaxCopy();
+    DsymbolExp *syntaxCopy();
     bool isLvalue();
     Expression *toLvalue(Scope *sc, Expression *e);
     void accept(Visitor *v) { v->visit(this); }
@@ -336,7 +349,7 @@ class ThisExp : public Expression
 public:
     VarDeclaration *var;
 
-    Expression *syntaxCopy();
+    ThisExp *syntaxCopy();
     bool isBool(bool result);
     bool isLvalue();
     Expression *toLvalue(Scope *sc, Expression *e);
@@ -416,7 +429,7 @@ public:
 
     static TupleExp *create(Loc loc, Expressions *exps);
     TupleExp *toTupleExp();
-    Expression *syntaxCopy();
+    TupleExp *syntaxCopy();
     bool equals(const RootObject *o) const;
 
     void accept(Visitor *v) { v->visit(this); }
@@ -431,7 +444,7 @@ public:
 
     static ArrayLiteralExp *create(Loc loc, Expressions *elements);
     static void emplace(UnionExp *pue, Loc loc, Expressions *elements);
-    Expression *syntaxCopy();
+    ArrayLiteralExp *syntaxCopy();
     bool equals(const RootObject *o) const;
     Expression *getElement(d_size_t i); // use opIndex instead
     Expression *opIndex(d_size_t i);
@@ -449,7 +462,7 @@ public:
     OwnedBy ownedByCtfe;
 
     bool equals(const RootObject *o) const;
-    Expression *syntaxCopy();
+    AssocArrayLiteralExp *syntaxCopy();
     bool isBool(bool result);
 
     void accept(Visitor *v) { v->visit(this); }
@@ -468,7 +481,7 @@ public:
     // to the memory used to build the literal for resolving such references.
     llvm::Value *inProgressMemory;
 #else
-    Symbol *sym;        // back end symbol to initialize with literal
+    Symbol *sym;                // back end symbol to initialize with literal
 #endif
 
     /** pointer to the origin instance of the expression.
@@ -494,10 +507,11 @@ public:
 
     static StructLiteralExp *create(Loc loc, StructDeclaration *sd, void *elements, Type *stype = NULL);
     bool equals(const RootObject *o) const;
-    Expression *syntaxCopy();
+    StructLiteralExp *syntaxCopy();
     Expression *getField(Type *type, unsigned offset);
     int getFieldIndex(Type *type, unsigned offset);
     Expression *addDtorHook(Scope *sc);
+    Expression *toLvalue(Scope *sc, Expression *e);
 
     void accept(Visitor *v) { v->visit(this); }
 };
@@ -505,7 +519,7 @@ public:
 class TypeExp : public Expression
 {
 public:
-    Expression *syntaxCopy();
+    TypeExp *syntaxCopy();
     bool checkType();
     bool checkValue();
     void accept(Visitor *v) { v->visit(this); }
@@ -516,7 +530,7 @@ class ScopeExp : public Expression
 public:
     ScopeDsymbol *sds;
 
-    Expression *syntaxCopy();
+    ScopeExp *syntaxCopy();
     bool checkType();
     bool checkValue();
     void accept(Visitor *v) { v->visit(this); }
@@ -548,12 +562,11 @@ public:
     Expression *argprefix;      // expression to be evaluated just before arguments[]
 
     CtorDeclaration *member;    // constructor function
-    NewDeclaration *allocator;  // allocator function
     bool onstack;               // allocate on stack
     bool thrownew;              // this NewExp is the expression of a ThrowStatement
 
     static NewExp *create(Loc loc, Expression *thisexp, Expressions *newargs, Type *newtype, Expressions *arguments);
-    Expression *syntaxCopy();
+    NewExp *syntaxCopy();
 
     void accept(Visitor *v) { v->visit(this); }
 };
@@ -568,7 +581,7 @@ public:
     ClassDeclaration *cd;       // class being instantiated
     Expressions *arguments;     // Array of Expression's to call class constructor
 
-    Expression *syntaxCopy();
+    NewAnonClassExp *syntaxCopy();
     void accept(Visitor *v) { v->visit(this); }
 };
 
@@ -602,7 +615,7 @@ public:
     bool delegateWasExtracted;
     static VarExp *create(Loc loc, Declaration *var, bool hasOverloads = true);
     bool equals(const RootObject *o) const;
-    int checkModifiable(Scope *sc, int flag);
+    int checkModifiable(Scope *sc, ModifyFlags flag);
     bool isLvalue();
     Expression *toLvalue(Scope *sc, Expression *e);
     Expression *modifiableLvalue(Scope *sc, Expression *e);
@@ -632,7 +645,7 @@ public:
     TOK tok;
 
     bool equals(const RootObject *o) const;
-    Expression *syntaxCopy();
+    FuncExp *syntaxCopy();
     const char *toChars() const;
     bool checkType();
     bool checkValue();
@@ -650,7 +663,7 @@ class DeclarationExp : public Expression
 public:
     Dsymbol *declaration;
 
-    Expression *syntaxCopy();
+    DeclarationExp *syntaxCopy();
 
     bool hasCode();
 
@@ -662,7 +675,7 @@ class TypeidExp : public Expression
 public:
     RootObject *obj;
 
-    Expression *syntaxCopy();
+    TypeidExp *syntaxCopy();
     void accept(Visitor *v) { v->visit(this); }
 };
 
@@ -672,7 +685,7 @@ public:
     Identifier *ident;
     Objects *args;
 
-    Expression *syntaxCopy();
+    TraitsExp *syntaxCopy();
     void accept(Visitor *v) { v->visit(this); }
 };
 
@@ -695,7 +708,7 @@ public:
     TOK tok;       // ':' or '=='
     TOK tok2;      // 'struct', 'union', etc.
 
-    Expression *syntaxCopy();
+    IsExp *syntaxCopy();
     void accept(Visitor *v) { v->visit(this); }
 };
 
@@ -707,7 +720,7 @@ public:
     Expression *e1;
     Type *att1; // Save alias this type to detect recursion
 
-    Expression *syntaxCopy();
+    UnaExp *syntaxCopy();
     Expression *incompatibleTypes();
     Expression *resolveLoc(const Loc &loc, Scope *sc);
 
@@ -723,7 +736,7 @@ public:
     Type *att1; // Save alias this type to detect recursion
     Type *att2; // Save alias this type to detect recursion
 
-    Expression *syntaxCopy();
+    BinExp *syntaxCopy();
     Expression *incompatibleTypes();
 
     Expression *reorderSettingAAElem(Scope *sc);
@@ -737,12 +750,13 @@ public:
     bool isLvalue();
     Expression *toLvalue(Scope *sc, Expression *ex);
     Expression *modifiableLvalue(Scope *sc, Expression *e);
+    BinAssignExp* isBinAssignExp();
     void accept(Visitor *v) { v->visit(this); }
 };
 
 /****************************************************************/
 
-class CompileExp : public UnaExp
+class MixinExp : public UnaExp
 {
 public:
     void accept(Visitor *v) { v->visit(this); }
@@ -759,7 +773,7 @@ class AssertExp : public UnaExp
 public:
     Expression *msg;
 
-    Expression *syntaxCopy();
+    AssertExp *syntaxCopy();
 
     void accept(Visitor *v) { v->visit(this); }
 };
@@ -770,6 +784,7 @@ public:
     Identifier *ident;
     bool noderef;       // true if the result of the expression will never be dereferenced
     bool wantsym;       // do not replace Symbol with its initializer during semantic()
+    bool arrow;         // ImportC: if -> instead of .
 
     static DotIdExp *create(Loc loc, Expression *e, Identifier *ident);
     void accept(Visitor *v) { v->visit(this); }
@@ -780,6 +795,8 @@ class DotTemplateExp : public UnaExp
 public:
     TemplateDeclaration *td;
 
+    bool checkType();
+    bool checkValue();
     void accept(Visitor *v) { v->visit(this); }
 };
 
@@ -789,7 +806,7 @@ public:
     Declaration *var;
     bool hasOverloads;
 
-    int checkModifiable(Scope *sc, int flag);
+    int checkModifiable(Scope *sc, ModifyFlags flag);
     bool isLvalue();
     Expression *toLvalue(Scope *sc, Expression *e);
     Expression *modifiableLvalue(Scope *sc, Expression *e);
@@ -801,7 +818,7 @@ class DotTemplateInstanceExp : public UnaExp
 public:
     TemplateInstance *ti;
 
-    Expression *syntaxCopy();
+    DotTemplateInstanceExp *syntaxCopy();
     bool findTempDecl(Scope *sc);
     void accept(Visitor *v) { v->visit(this); }
 };
@@ -832,6 +849,7 @@ public:
     FuncDeclaration *f;         // symbol to call
     bool directcall;            // true if a virtual call is devirtualized
     bool inDebugStatement;      // true if this was in a debug statement
+    bool ignoreAttributes;      // don't enforce attributes (e.g. call @gc function in @nogc code)
     VarDeclaration *vthis2;     // container for multi-context
 
     static CallExp *create(Loc loc, Expression *e, Expressions *exps);
@@ -839,7 +857,7 @@ public:
     static CallExp *create(Loc loc, Expression *e, Expression *earg1);
     static CallExp *create(Loc loc, FuncDeclaration *fd, Expression *earg1);
 
-    Expression *syntaxCopy();
+    CallExp *syntaxCopy();
     bool isLvalue();
     Expression *toLvalue(Scope *sc, Expression *e);
     Expression *addDtorHook(Scope *sc);
@@ -856,7 +874,7 @@ public:
 class PtrExp : public UnaExp
 {
 public:
-    int checkModifiable(Scope *sc, int flag);
+    int checkModifiable(Scope *sc, ModifyFlags flag);
     bool isLvalue();
     Expression *toLvalue(Scope *sc, Expression *e);
     Expression *modifiableLvalue(Scope *sc, Expression *e);
@@ -892,7 +910,6 @@ class DeleteExp : public UnaExp
 {
 public:
     bool isRAII;
-    Expression *toBoolean(Scope *sc);
     void accept(Visitor *v) { v->visit(this); }
 };
 
@@ -903,7 +920,9 @@ public:
     Type *to;                   // type to cast to
     unsigned char mod;          // MODxxxxx
 
-    Expression *syntaxCopy();
+    CastExp *syntaxCopy();
+    bool isLvalue();
+    Expression *toLvalue(Scope *sc, Expression *e);
 
     void accept(Visitor *v) { v->visit(this); }
 };
@@ -917,7 +936,7 @@ public:
 
     static VectorExp *create(Loc loc, Expression *e, Type *t);
     static void emplace(UnionExp *pue, Loc loc, Expression *e, Type *t);
-    Expression *syntaxCopy();
+    VectorExp *syntaxCopy();
     void accept(Visitor *v) { v->visit(this); }
 };
 
@@ -939,8 +958,8 @@ public:
     bool lowerIsLessThanUpper;  // true if lwr <= upr
     bool arrayop;               // an array operation, rather than a slice
 
-    Expression *syntaxCopy();
-    int checkModifiable(Scope *sc, int flag);
+    SliceExp *syntaxCopy();
+    int checkModifiable(Scope *sc, ModifyFlags flag);
     bool isLvalue();
     Expression *toLvalue(Scope *sc, Expression *e);
     Expression *modifiableLvalue(Scope *sc, Expression *e);
@@ -961,7 +980,7 @@ public:
     Expression *lwr;
     Expression *upr;
 
-    Expression *syntaxCopy();
+    IntervalExp *syntaxCopy();
     void accept(Visitor *v) { v->visit(this); }
 };
 
@@ -992,7 +1011,7 @@ public:
     size_t currentDimension;            // for opDollar
     VarDeclaration *lengthVar;
 
-    Expression *syntaxCopy();
+    ArrayExp *syntaxCopy();
     bool isLvalue();
     Expression *toLvalue(Scope *sc, Expression *e);
 
@@ -1012,14 +1031,33 @@ class CommaExp : public BinExp
 public:
     bool isGenerated;
     bool allowCommaExp;
-    int checkModifiable(Scope *sc, int flag);
+    int checkModifiable(Scope *sc, ModifyFlags flag);
     bool isLvalue();
     Expression *toLvalue(Scope *sc, Expression *e);
     Expression *modifiableLvalue(Scope *sc, Expression *e);
     bool isBool(bool result);
-    Expression *toBoolean(Scope *sc);
     Expression *addDtorHook(Scope *sc);
     void accept(Visitor *v) { v->visit(this); }
+
+#if IN_LLVM
+    // Returns the head of this CommaExp, descending recursively.
+    //    `(a, b), c` => a
+    Expression *getHead() {
+      auto l = e1;
+      while (auto ce = l->isCommaExp())
+        l = ce->e1;
+      return l;
+    }
+
+    // Returns the tail of this CommaExp, descending recursively.
+    //    `a, (b, c)` => c
+    Expression *getTail() {
+      auto r = e2;
+      while (auto ce = r->isCommaExp())
+        r = ce->e2;
+      return r;
+    }
+#endif
 };
 
 class IndexExp : public BinExp
@@ -1029,8 +1067,8 @@ public:
     bool modifiable;
     bool indexIsInBounds;       // true if 0 <= e2 && e2 <= e1.length - 1
 
-    Expression *syntaxCopy();
-    int checkModifiable(Scope *sc, int flag);
+    IndexExp *syntaxCopy();
+    int checkModifiable(Scope *sc, ModifyFlags flag);
     bool isLvalue();
     Expression *toLvalue(Scope *sc, Expression *e);
     Expression *modifiableLvalue(Scope *sc, Expression *e);
@@ -1068,7 +1106,6 @@ public:
 
     bool isLvalue();
     Expression *toLvalue(Scope *sc, Expression *ex);
-    Expression *toBoolean(Scope *sc);
 
     void accept(Visitor *v) { v->visit(this); }
 };
@@ -1244,7 +1281,6 @@ public:
 class LogicalExp : public BinExp
 {
 public:
-    Expression *toBoolean(Scope *sc);
     void accept(Visitor *v) { v->visit(this); }
 };
 
@@ -1289,17 +1325,27 @@ class CondExp : public BinExp
 public:
     Expression *econd;
 
-    Expression *syntaxCopy();
-    int checkModifiable(Scope *sc, int flag);
+    CondExp *syntaxCopy();
+    int checkModifiable(Scope *sc, ModifyFlags flag);
     bool isLvalue();
     Expression *toLvalue(Scope *sc, Expression *e);
     Expression *modifiableLvalue(Scope *sc, Expression *e);
-    Expression *toBoolean(Scope *sc);
     void hookDtors(Scope *sc);
 
     void accept(Visitor *v) { v->visit(this); }
 };
 
+class GenericExp : Expression
+{
+    Expression *cntlExp;
+    Types *types;
+    Expressions *exps;
+
+    GenericExp *syntaxCopy();
+
+    void accept(Visitor *v) { v->visit(this); }
+};
+
 /****************************************************************/
 
 class DefaultInitExp : public Expression
