@@ -1,6 +1,6 @@
 
 /* Compiler implementation of the D programming language
- * Copyright (C) 2017-2020 by The D Language Foundation, All Rights Reserved
+ * Copyright (C) 2017-2021 by The D Language Foundation, All Rights Reserved
  * written by Walter Bright
  * http://www.digitalmars.com
  * Distributed under the Boost Software License, Version 1.0.
@@ -32,6 +32,9 @@ struct Id
     static Identifier *__c_longlong;
     static Identifier *__c_ulonglong;
     static Identifier *__c_long_double;
+    static Identifier *__c_complex_float;
+    static Identifier *__c_complex_double;
+    static Identifier *__c_complex_real;
     static Identifier *__switch;
     static Identifier *crt_constructor;
     static Identifier *crt_destructor;
@@ -41,6 +44,9 @@ struct Id
     static Identifier *dcompute;
     static Identifier *dcPointer;
     static Identifier *object;
+    static Identifier *core;
+    static Identifier *etc;
+    static Identifier *std;
     static Identifier *ensure;
     static Identifier *require;
     static Identifier *xopEquals;
@@ -71,6 +77,7 @@ struct Id
     static Identifier *LDC_extern_weak;
     static Identifier *LDC_profile_instr;
     static Identifier *dcReflect;
+    static Identifier *opencl;
     static Identifier *criticalenter;
     static Identifier *criticalexit;
     static Identifier *attributes;
@@ -87,5 +94,6 @@ struct Id
     static Identifier *udaDynamicCompile;
     static Identifier *udaDynamicCompileConst;
     static Identifier *udaDynamicCompileEmit;
+    static Identifier *io;
 #endif
 };
