@@ -1,6 +1,6 @@
 
 /* Compiler implementation of the D programming language
- * Copyright (C) 1999-2020 by The D Language Foundation, All Rights Reserved
+ * Copyright (C) 1999-2021 by The D Language Foundation, All Rights Reserved
  * written by Walter Bright
  * http://www.digitalmars.com
  * Distributed under the Boost Software License, Version 1.0.
@@ -17,10 +17,6 @@
 #include "globals.h"
 #include "visitor.h"
 
-#if IN_LLVM
-#include <cstdlib>
-#endif
-
 struct Scope;
 class AggregateDeclaration;
 class Identifier;
@@ -50,7 +46,7 @@ void semanticTypeInfo(Scope *sc, Type *t);
 Type *typeSemantic(Type *t, const Loc &loc, Scope *sc);
 Type *merge(Type *type);
 
-enum ENUMTY
+enum class TY : uint8_t
 {
     Tarray,             // slice array, aka T[]
     Tsarray,            // static array, aka T[dimension]
@@ -101,9 +97,10 @@ enum ENUMTY
     Tint128,
     Tuns128,
     Ttraits,
+    Tmixin,
+    Tnoreturn,
     TMAX
 };
-typedef unsigned char TY;       // ENUMTY
 
 #define SIZE_INVALID (~(d_uns64)0)   // error return from size() functions
 
@@ -186,6 +183,7 @@ public:
     static Type *tdstring;              // immutable(dchar)[]
     static Type *terror;                // for error recovery
     static Type *tnull;                 // for null type
+    static Type *tnoreturn;             // for bottom type typeof(*null)
 
     static Type *tsize_t;               // matches size_t alias
     static Type *tptrdiff_t;            // matches ptrdiff_t alias
@@ -214,7 +212,7 @@ public:
     static TemplateDeclaration *rtinfoImpl;
 #endif
 
-    static Type *basic[TMAX];
+    static Type *basic[(int)TY::TMAX];
 
     virtual const char *kind();
     Type *copy() const;
@@ -308,6 +306,7 @@ public:
     virtual int hasWild() const;
     virtual bool hasPointers();
     virtual bool hasVoidInitPointers();
+    virtual bool hasInvariant();
     virtual Type *nextOf();
     Type *baseElemOf();
     uinteger_t sizemask();
@@ -319,6 +318,8 @@ public:
 
     // For eliminating dynamic_cast
     virtual TypeBasic *isTypeBasic();
+    TypeFunction *isPtrToFunction();
+    TypeFunction *isFunction_Delegate_PtrToFunction();
     TypeError *isTypeError();
     TypeVector *isTypeVector();
     TypeSArray *isTypeSArray();
@@ -340,6 +341,8 @@ public:
     TypeNull *isTypeNull();
     TypeMixin *isTypeMixin();
     TypeTraits *isTypeTraits();
+    TypeNoreturn *isTypeNoreturn();
+    TypeTag *isTypeTag();
 
     void accept(Visitor *v) { v->visit(this); }
 };
@@ -348,7 +351,7 @@ class TypeError : public Type
 {
 public:
     const char *kind();
-    Type *syntaxCopy();
+    TypeError *syntaxCopy();
 
     d_uns64 size(const Loc &loc);
     Expression *defaultInitLiteral(const Loc &loc);
@@ -385,7 +388,7 @@ public:
     unsigned flags;
 
     const char *kind();
-    Type *syntaxCopy();
+    TypeBasic *syntaxCopy();
     d_uns64 size(const Loc &loc) /*const*/;
     unsigned alignsize();
     bool isintegral();
@@ -410,7 +413,7 @@ public:
 
     static TypeVector *create(Type *basetype);
     const char *kind();
-    Type *syntaxCopy();
+    TypeVector *syntaxCopy();
     d_uns64 size(const Loc &loc);
     unsigned alignsize();
     bool isintegral();
@@ -439,7 +442,8 @@ public:
     Expression *dim;
 
     const char *kind();
-    Type *syntaxCopy();
+    TypeSArray *syntaxCopy();
+    bool isIncomplete();
     d_uns64 size(const Loc &loc);
     unsigned alignsize();
     bool isString();
@@ -449,6 +453,7 @@ public:
     MATCH implicitConvTo(Type *to);
     Expression *defaultInitLiteral(const Loc &loc);
     bool hasPointers();
+    bool hasInvariant();
     bool needsDestruction();
     bool needsCopyOrPostblit();
     bool needsNested();
@@ -461,7 +466,7 @@ class TypeDArray : public TypeArray
 {
 public:
     const char *kind();
-    Type *syntaxCopy();
+    TypeDArray *syntaxCopy();
     d_uns64 size(const Loc &loc) /*const*/;
     unsigned alignsize() /*const*/;
     bool isString();
@@ -481,7 +486,7 @@ public:
 
     static TypeAArray *create(Type *t, Type *index);
     const char *kind();
-    Type *syntaxCopy();
+    TypeAArray *syntaxCopy();
     d_uns64 size(const Loc &loc);
     bool isZeroInit(const Loc &loc) /*const*/;
     bool isBoolean() /*const*/;
@@ -497,7 +502,7 @@ class TypePointer : public TypeNext
 public:
     static TypePointer *create(Type *t);
     const char *kind();
-    Type *syntaxCopy();
+    TypePointer *syntaxCopy();
     d_uns64 size(const Loc &loc) /*const*/;
     MATCH implicitConvTo(Type *to);
     MATCH constConv(Type *to);
@@ -512,7 +517,7 @@ class TypeReference : public TypeNext
 {
 public:
     const char *kind();
-    Type *syntaxCopy();
+    TypeReference *syntaxCopy();
     d_uns64 size(const Loc &loc) /*const*/;
     bool isZeroInit(const Loc &loc) /*const*/;
     void accept(Visitor *v) { v->visit(this); }
@@ -567,7 +572,7 @@ public:
     static size_t dim(Parameters *parameters);
     static Parameter *getNth(Parameters *parameters, d_size_t nth);
     const char *toChars() const;
-    bool isCovariant(bool returnByRef, const Parameter *p) const;
+    bool isCovariant(bool returnByRef, const Parameter *p, bool previewIn) const;
 };
 
 struct ParameterList
@@ -575,6 +580,7 @@ struct ParameterList
     Parameters* parameters;
     StorageClass stc;
     VarArg varargs;
+    bool hasIdentifierList; // true if C identifier-list style
 
     size_t length();
     Parameter *operator[](size_t i) { return Parameter::getNth(parameters, i); }
@@ -595,7 +601,7 @@ public:
 
     static TypeFunction *create(Parameters *parameters, Type *treturn, VarArg varargs, LINK linkage, StorageClass stc = 0);
     const char *kind();
-    Type *syntaxCopy();
+    TypeFunction *syntaxCopy();
     void purityLevel();
     bool hasLazyParameters();
     bool isDstyleVariadic() const;
@@ -604,6 +610,7 @@ public:
     Type *addStorageClass(StorageClass stc);
 
     Type *substWildTo(unsigned mod);
+    MATCH constConv(Type *to);
 
     bool isnothrow() const;
     void isnothrow(bool v);
@@ -639,9 +646,9 @@ class TypeDelegate : public TypeNext
 public:
     // .next is a TypeFunction
 
-    static TypeDelegate *create(Type *t);
+    static TypeDelegate *create(TypeFunction *t);
     const char *kind();
-    Type *syntaxCopy();
+    TypeDelegate *syntaxCopy();
     Type *addStorageClass(StorageClass stc);
     d_uns64 size(const Loc &loc) /*const*/;
     unsigned alignsize() /*const*/;
@@ -662,7 +669,7 @@ class TypeTraits : public Type
     Dsymbol *sym;
 
     const char *kind();
-    Type *syntaxCopy();
+    TypeTraits *syntaxCopy();
     d_uns64 size(const Loc &loc);
     Dsymbol *toDsymbol(Scope *sc);
     void accept(Visitor *v) { v->visit(this); }
@@ -672,9 +679,10 @@ class TypeMixin : public Type
 {
     Loc loc;
     Expressions *exps;
+    RootObject *obj;
 
     const char *kind();
-    Type *syntaxCopy();
+    TypeMixin *syntaxCopy();
     Dsymbol *toDsymbol(Scope *sc);
     void accept(Visitor *v) { v->visit(this); }
 };
@@ -702,8 +710,11 @@ public:
     Identifier *ident;
     Dsymbol *originalSymbol; // The symbol representing this identifier, before alias resolution
 
+#if IN_LLVM
+    static TypeIdentifier *create(const Loc &loc, Identifier *ident);
+#endif
     const char *kind();
-    Type *syntaxCopy();
+    TypeIdentifier *syntaxCopy();
     Dsymbol *toDsymbol(Scope *sc);
     void accept(Visitor *v) { v->visit(this); }
 };
@@ -716,7 +727,7 @@ public:
     TemplateInstance *tempinst;
 
     const char *kind();
-    Type *syntaxCopy();
+    TypeInstance *syntaxCopy();
     Dsymbol *toDsymbol(Scope *sc);
     void accept(Visitor *v) { v->visit(this); }
 };
@@ -728,7 +739,7 @@ public:
     int inuse;
 
     const char *kind();
-    Type *syntaxCopy();
+    TypeTypeof *syntaxCopy();
     Dsymbol *toDsymbol(Scope *sc);
     d_uns64 size(const Loc &loc);
     void accept(Visitor *v) { v->visit(this); }
@@ -738,7 +749,7 @@ class TypeReturn : public TypeQualified
 {
 public:
     const char *kind();
-    Type *syntaxCopy();
+    TypeReturn *syntaxCopy();
     Dsymbol *toDsymbol(Scope *sc);
     void accept(Visitor *v) { v->visit(this); }
 };
@@ -766,7 +777,7 @@ public:
     const char *kind();
     d_uns64 size(const Loc &loc);
     unsigned alignsize();
-    Type *syntaxCopy();
+    TypeStruct *syntaxCopy();
     Dsymbol *toDsymbol(Scope *sc);
     structalign_t alignment();
     Expression *defaultInitLiteral(const Loc &loc);
@@ -778,6 +789,7 @@ public:
     bool needsNested();
     bool hasPointers();
     bool hasVoidInitPointers();
+    bool hasInvariant();
     MATCH implicitConvTo(Type *to);
     MATCH constConv(Type *to);
     unsigned char deduceWild(Type *t, bool isRef);
@@ -792,7 +804,7 @@ public:
     EnumDeclaration *sym;
 
     const char *kind();
-    Type *syntaxCopy();
+    TypeEnum *syntaxCopy();
     d_uns64 size(const Loc &loc);
     unsigned alignsize();
     Type *memType(const Loc &loc = Loc());
@@ -815,6 +827,7 @@ public:
     bool isZeroInit(const Loc &loc);
     bool hasPointers();
     bool hasVoidInitPointers();
+    bool hasInvariant();
     Type *nextOf();
 
     void accept(Visitor *v) { v->visit(this); }
@@ -829,7 +842,7 @@ public:
 
     const char *kind();
     d_uns64 size(const Loc &loc) /*const*/;
-    Type *syntaxCopy();
+    TypeClass *syntaxCopy();
     Dsymbol *toDsymbol(Scope *sc);
     ClassDeclaration *isClassHandle();
     bool isBaseOf(Type *t, int *poffset);
@@ -858,7 +871,7 @@ public:
     static TypeTuple *create(Type *t1);
     static TypeTuple *create(Type *t1, Type *t2);
     const char *kind();
-    Type *syntaxCopy();
+    TypeTuple *syntaxCopy();
     bool equals(const RootObject *o) const;
     void accept(Visitor *v) { v->visit(this); }
 };
@@ -870,7 +883,7 @@ public:
     Expression *upr;
 
     const char *kind();
-    Type *syntaxCopy();
+    TypeSlice *syntaxCopy();
     void accept(Visitor *v) { v->visit(this); }
 };
 
@@ -879,7 +892,7 @@ class TypeNull : public Type
 public:
     const char *kind();
 
-    Type *syntaxCopy();
+    TypeNull *syntaxCopy();
     MATCH implicitConvTo(Type *to);
     bool isBoolean() /*const*/;
 
@@ -887,6 +900,28 @@ public:
     void accept(Visitor *v) { v->visit(this); }
 };
 
+class TypeNoreturn final : public Type
+{
+public:
+    const char *kind();
+    TypeNoreturn *syntaxCopy();
+    MATCH implicitConvTo(Type* to);
+    MATCH constConv(Type* to);
+    bool isBoolean() /* const */;
+    d_uns64 size(const Loc& loc) /* const */;
+    unsigned alignsize();
+
+    void accept(Visitor *v) { v->visit(this); }
+};
+
+class TypeTag final : public Type
+{
+public:
+    TypeTag *syntaxCopy();
+
+    void accept(Visitor *v) { v->visit(this); }
+};
+
 /**************************************************************/
 
 bool arrayTypeCompatibleWithoutCasting(Type *t1, Type *t2);
