@@ -1,4 +1,4 @@
-/* Copyright (C) 2011-2020 by The D Language Foundation, All Rights Reserved
+/* Copyright (C) 2011-2021 by The D Language Foundation, All Rights Reserved
  * written by Walter Bright
  * http://www.digitalmars.com
  * Distributed under the Boost Software License, Version 1.0.
@@ -34,9 +34,9 @@ struct Array
   public:
     Array()
     {
-        data.ptr = SMALLARRAYCAP ? &smallarray[0] : NULL;
+        data.ptr = NULL;
         length = 0;
-        data.length = SMALLARRAYCAP;
+        data.length = 0;
     }
 
     ~Array()
