@@ -1,5 +1,5 @@
 
-/* Copyright (C) 1999-2020 by The D Language Foundation, All Rights Reserved
+/* Copyright (C) 1999-2021 by The D Language Foundation, All Rights Reserved
  * written by Walter Bright
  * http://www.digitalmars.com
  * Distributed under the Boost Software License, Version 1.0.
@@ -22,14 +22,12 @@ struct Mem
     static void *xrealloc(void *p, d_size_t size);
     static void error();
 
-#if 1 // version (GC)
     static bool _isGCEnabled;
 
     static bool isGCEnabled();
     static void disableGC();
     static void addRange(const void *p, d_size_t size);
     static void removeRange(const void *p);
-#endif
 };
 
 extern Mem mem;
