@@ -1,6 +1,6 @@
 
 /* Compiler implementation of the D programming language
- * Copyright (C) 1999-2020 by The D Language Foundation, All Rights Reserved
+ * Copyright (C) 1999-2021 by The D Language Foundation, All Rights Reserved
  * written by Walter Bright
  * http://www.digitalmars.com
  * Distributed under the Boost Software License, Version 1.0.
@@ -43,8 +43,7 @@ class StaticForeach;
 // Back end
 #if IN_LLVM
 namespace llvm { class Value; }
-struct AsmCode;
-using code = AsmCode;
+using code = struct AsmCode;
 #else
 struct code;
 #endif
@@ -128,8 +127,6 @@ public:
     bool usesEH();
     bool comeFrom();
     bool hasCode();
-    virtual Statement *scopeCode(Scope *sc, Statement **sentry, Statement **sexit, Statement **sfinally);
-    virtual Statements *flatten(Scope *sc);
     virtual Statement *last();
 
     virtual ReturnStatement *endsWithReturnStatement() { return NULL; }
@@ -148,6 +145,8 @@ public:
 #endif
     ReturnStatement      *isReturnStatement()      { return stmt == STMTreturn      ? (ReturnStatement*)this      : NULL; }
     IfStatement          *isIfStatement()          { return stmt == STMTif          ? (IfStatement*)this          : NULL; }
+    ConditionalStatement *isConditionalStatement() { return stmt == STMTconditional ? (ConditionalStatement*)this : NULL; }
+    StaticForeachStatement *isStaticForeachStatement() { return stmt == STMTstaticForeach ? (StaticForeachStatement*)this : NULL; }
     CaseStatement        *isCaseStatement()        { return stmt == STMTcase        ? (CaseStatement*)this        : NULL; }
     DefaultStatement     *isDefaultStatement()     { return stmt == STMTdefault     ? (DefaultStatement*)this     : NULL; }
     LabelStatement       *isLabelStatement()       { return stmt == STMTlabel       ? (LabelStatement*)this       : NULL; }
@@ -155,6 +154,7 @@ public:
     GotoCaseStatement    *isGotoCaseStatement()    { return stmt == STMTgotoCase    ? (GotoCaseStatement*)this    : NULL; }
     BreakStatement       *isBreakStatement()       { return stmt == STMTbreak       ? (BreakStatement*)this       : NULL; }
     DtorExpStatement     *isDtorExpStatement()     { return stmt == STMTdtorExp     ? (DtorExpStatement*)this     : NULL; }
+    CompileStatement     *isCompileStatement()     { return stmt == STMTcompile     ? (CompileStatement*)this     : NULL; }
     ForwardingStatement  *isForwardingStatement()  { return stmt == STMTforwarding  ? (ForwardingStatement*)this  : NULL; }
     DoStatement          *isDoStatement()          { return stmt == STMTdo          ? (DoStatement*)this          : NULL; }
     ForStatement         *isForStatement()         { return stmt == STMTfor         ? (ForStatement*)this         : NULL; }
@@ -164,7 +164,9 @@ public:
     WithStatement        *isWithStatement()        { return stmt == STMTwith        ? (WithStatement*)this        : NULL; }
     TryCatchStatement    *isTryCatchStatement()    { return stmt == STMTtryCatch    ? (TryCatchStatement*)this    : NULL; }
     ThrowStatement       *isThrowStatement()       { return stmt == STMTthrow       ? (ThrowStatement*)this       : NULL; }
+    DebugStatement       *isDebugStatement()       { return stmt == STMTdebug       ? (DebugStatement*)this       : NULL; }
     TryFinallyStatement  *isTryFinallyStatement()  { return stmt == STMTtryFinally  ? (TryFinallyStatement*)this  : NULL; }
+    ScopeGuardStatement  *isScopeGuardStatement()  { return stmt == STMTscopeGuard  ? (ScopeGuardStatement*)this  : NULL; }
     SwitchErrorStatement  *isSwitchErrorStatement()  { return stmt == STMTswitchError  ? (SwitchErrorStatement*)this  : NULL; }
     UnrolledLoopStatement *isUnrolledLoopStatement() { return stmt == STMTunrolledLoop ? (UnrolledLoopStatement*)this : NULL; }
     ForeachRangeStatement *isForeachRangeStatement() { return stmt == STMTforeachRange ? (ForeachRangeStatement*)this : NULL; }
@@ -179,7 +181,7 @@ public:
 class ErrorStatement : public Statement
 {
 public:
-    Statement *syntaxCopy();
+    ErrorStatement *syntaxCopy();
 
     void accept(Visitor *v) { v->visit(this); }
 };
@@ -198,9 +200,7 @@ public:
     Expression *exp;
 
     static ExpStatement *create(Loc loc, Expression *exp);
-    Statement *syntaxCopy();
-    Statement *scopeCode(Scope *sc, Statement **sentry, Statement **sexit, Statement **sfinally);
-    Statements *flatten(Scope *sc);
+    ExpStatement *syntaxCopy();
 
     void accept(Visitor *v) { v->visit(this); }
 };
@@ -213,7 +213,7 @@ public:
 
     VarDeclaration *var;
 
-    Statement *syntaxCopy();
+    DtorExpStatement *syntaxCopy();
     void accept(Visitor *v) { v->visit(this); }
 };
 
@@ -222,8 +222,7 @@ class CompileStatement : public Statement
 public:
     Expressions *exps;
 
-    Statement *syntaxCopy();
-    Statements *flatten(Scope *sc);
+    CompileStatement *syntaxCopy();
     void accept(Visitor *v) { v->visit(this); }
 };
 
@@ -233,8 +232,7 @@ public:
     Statements *statements;
 
     static CompoundStatement *create(Loc loc, Statement *s1, Statement *s2);
-    Statement *syntaxCopy();
-    Statements *flatten(Scope *sc);
+    CompoundStatement *syntaxCopy();
     ReturnStatement *endsWithReturnStatement();
     Statement *last();
 
@@ -248,7 +246,7 @@ public:
 class CompoundDeclarationStatement : public CompoundStatement
 {
 public:
-    Statement *syntaxCopy();
+    CompoundDeclarationStatement *syntaxCopy();
     void accept(Visitor *v) { v->visit(this); }
 };
 
@@ -260,7 +258,7 @@ class UnrolledLoopStatement : public Statement
 public:
     Statements *statements;
 
-    Statement *syntaxCopy();
+    UnrolledLoopStatement *syntaxCopy();
     bool hasBreak() const;
     bool hasContinue() const;
 
@@ -273,7 +271,7 @@ public:
     Statement *statement;
     Loc endloc;                 // location of closing curly bracket
 
-    Statement *syntaxCopy();
+    ScopeStatement *syntaxCopy();
     ReturnStatement *endsWithReturnStatement();
     bool hasBreak() const;
     bool hasContinue() const;
@@ -287,19 +285,19 @@ public:
     ForwardingScopeDsymbol *sym;
     Statement *statement;
 
-    Statement *syntaxCopy();
-    Statements *flatten(Scope *sc);
+    ForwardingStatement *syntaxCopy();
     void accept(Visitor *v) { v->visit(this); }
 };
 
 class WhileStatement : public Statement
 {
 public:
+    Parameter *param;
     Expression *condition;
     Statement *_body;
     Loc endloc;                 // location of closing curly bracket
 
-    Statement *syntaxCopy();
+    WhileStatement *syntaxCopy();
     bool hasBreak() const;
     bool hasContinue() const;
 
@@ -313,7 +311,7 @@ public:
     Expression *condition;
     Loc endloc;                 // location of ';' after while
 
-    Statement *syntaxCopy();
+    DoStatement *syntaxCopy();
     bool hasBreak() const;
     bool hasContinue() const;
 
@@ -334,8 +332,7 @@ public:
     // treat that label as referring to this loop.
     Statement *relatedLabeled;
 
-    Statement *syntaxCopy();
-    Statement *scopeCode(Scope *sc, Statement **sentry, Statement **sexit, Statement **sfinally);
+    ForStatement *syntaxCopy();
     Statement *getRelatedLabeled() { return relatedLabeled ? relatedLabeled : this; }
     bool hasBreak() const;
     bool hasContinue() const;
@@ -360,7 +357,7 @@ public:
     Statements *cases;          // put breaks, continues, gotos and returns here
     ScopeStatements *gotos;     // forward referenced goto's go here
 
-    Statement *syntaxCopy();
+    ForeachStatement *syntaxCopy();
     bool hasBreak() const;
     bool hasContinue() const;
 
@@ -379,7 +376,7 @@ public:
 
     VarDeclaration *key;
 
-    Statement *syntaxCopy();
+    ForeachRangeStatement *syntaxCopy();
     bool hasBreak() const;
     bool hasContinue() const;
 
@@ -396,7 +393,7 @@ public:
     VarDeclaration *match;      // for MatchExpression results
     Loc endloc;                 // location of closing curly bracket
 
-    Statement *syntaxCopy();
+    IfStatement *syntaxCopy();
 
     void accept(Visitor *v) { v->visit(this); }
 };
@@ -408,8 +405,7 @@ public:
     Statement *ifbody;
     Statement *elsebody;
 
-    Statement *syntaxCopy();
-    Statements *flatten(Scope *sc);
+    ConditionalStatement *syntaxCopy();
 
     void accept(Visitor *v) { v->visit(this); }
 };
@@ -419,8 +415,7 @@ class StaticForeachStatement : public Statement
 public:
     StaticForeach *sfe;
 
-    Statement *syntaxCopy();
-    Statements *flatten(Scope *sc);
+    StaticForeachStatement *syntaxCopy();
 
     void accept(Visitor *v) { v->visit(this); }
 };
@@ -432,7 +427,7 @@ public:
     Expressions *args;          // array of Expression's
     Statement *_body;
 
-    Statement *syntaxCopy();
+    PragmaStatement *syntaxCopy();
 
     void accept(Visitor *v) { v->visit(this); }
 };
@@ -442,7 +437,7 @@ class StaticAssertStatement : public Statement
 public:
     StaticAssert *sa;
 
-    Statement *syntaxCopy();
+    StaticAssertStatement *syntaxCopy();
 
     void accept(Visitor *v) { v->visit(this); }
 };
@@ -467,7 +462,7 @@ public:
     bool hasGotoDefault;        // true iff there is a `goto default` statement for this switch
 #endif
 
-    Statement *syntaxCopy();
+    SwitchStatement *syntaxCopy();
     bool hasBreak() const;
 
     void accept(Visitor *v) { v->visit(this); }
@@ -487,7 +482,7 @@ public:
     bool gototarget; // true iff this is the target of a 'goto case'
 #endif
 
-    Statement *syntaxCopy();
+    CaseStatement *syntaxCopy();
 
     void accept(Visitor *v) { v->visit(this); }
 };
@@ -500,7 +495,7 @@ public:
     Expression *last;
     Statement *statement;
 
-    Statement *syntaxCopy();
+    CaseRangeStatement *syntaxCopy();
     void accept(Visitor *v) { v->visit(this); }
 };
 
@@ -515,7 +510,7 @@ public:
     bool gototarget; // true iff this is the target of a 'goto default'
 #endif
 
-    Statement *syntaxCopy();
+    DefaultStatement *syntaxCopy();
 
     void accept(Visitor *v) { v->visit(this); }
 };
@@ -525,7 +520,7 @@ class GotoDefaultStatement : public Statement
 public:
     SwitchStatement *sw;
 
-    Statement *syntaxCopy();
+    GotoDefaultStatement *syntaxCopy();
 
     void accept(Visitor *v) { v->visit(this); }
 };
@@ -540,7 +535,7 @@ public:
     SwitchStatement *sw;
 #endif
 
-    Statement *syntaxCopy();
+    GotoCaseStatement *syntaxCopy();
 
     void accept(Visitor *v) { v->visit(this); }
 };
@@ -559,7 +554,7 @@ public:
     Expression *exp;
     size_t caseDim;
 
-    Statement *syntaxCopy();
+    ReturnStatement *syntaxCopy();
 
     ReturnStatement *endsWithReturnStatement() { return this; }
     void accept(Visitor *v) { v->visit(this); }
@@ -575,7 +570,7 @@ public:
     LabelStatement *target;
 #endif
 
-    Statement *syntaxCopy();
+    BreakStatement *syntaxCopy();
 
     void accept(Visitor *v) { v->visit(this); }
 };
@@ -590,7 +585,7 @@ public:
     LabelStatement *target;
 #endif
 
-    Statement *syntaxCopy();
+    ContinueStatement *syntaxCopy();
 
     void accept(Visitor *v) { v->visit(this); }
 };
@@ -601,7 +596,7 @@ public:
     Expression *exp;
     Statement *_body;
 
-    Statement *syntaxCopy();
+    SynchronizedStatement *syntaxCopy();
     bool hasBreak() const;
     bool hasContinue() const;
 
@@ -616,7 +611,7 @@ public:
     VarDeclaration *wthis;
     Loc endloc;
 
-    Statement *syntaxCopy();
+    WithStatement *syntaxCopy();
 
     void accept(Visitor *v) { v->visit(this); }
 };
@@ -629,7 +624,7 @@ public:
 
     Statement *tryBody;   /// set to enclosing TryCatchStatement or TryFinallyStatement if in _body portion
 
-    Statement *syntaxCopy();
+    TryCatchStatement *syntaxCopy();
     bool hasBreak() const;
 
     void accept(Visitor *v) { v->visit(this); }
@@ -664,7 +659,7 @@ public:
     bool bodyFallsThru;   // true if _body falls through to finally
 
     static TryFinallyStatement *create(Loc loc, Statement *body, Statement *finalbody);
-    Statement *syntaxCopy();
+    TryFinallyStatement *syntaxCopy();
     bool hasBreak() const;
     bool hasContinue() const;
 
@@ -677,8 +672,7 @@ public:
     TOK tok;
     Statement *statement;
 
-    Statement *syntaxCopy();
-    Statement *scopeCode(Scope *sc, Statement **sentry, Statement **sexit, Statement **sfinally);
+    ScopeGuardStatement *syntaxCopy();
 
     void accept(Visitor *v) { v->visit(this); }
 };
@@ -691,7 +685,7 @@ public:
     // wasn't present in source code
     bool internalThrow;
 
-    Statement *syntaxCopy();
+    ThrowStatement *syntaxCopy();
 
     void accept(Visitor *v) { v->visit(this); }
 };
@@ -701,8 +695,7 @@ class DebugStatement : public Statement
 public:
     Statement *statement;
 
-    Statement *syntaxCopy();
-    Statements *flatten(Scope *sc);
+    DebugStatement *syntaxCopy();
     void accept(Visitor *v) { v->visit(this); }
 };
 
@@ -716,7 +709,7 @@ public:
     ScopeGuardStatement *os;
     VarDeclaration *lastVar;
 
-    Statement *syntaxCopy();
+    GotoStatement *syntaxCopy();
 
     void accept(Visitor *v) { v->visit(this); }
 };
@@ -734,9 +727,7 @@ public:
     void* extra;                // used by Statement_toIR()
     bool breaks;                // someone did a 'break ident'
 
-    Statement *syntaxCopy();
-    Statements *flatten(Scope *sc);
-    Statement *scopeCode(Scope *sc, Statement **sentry, Statement **sexit, Statement **sfinally);
+    LabelStatement *syntaxCopy();
 
     void accept(Visitor *v) { v->visit(this); }
 };
@@ -761,7 +752,7 @@ class AsmStatement : public Statement
 public:
     Token *tokens;
 
-    Statement *syntaxCopy();
+    AsmStatement *syntaxCopy();
     void accept(Visitor *v) { v->visit(this); }
 };
 
@@ -779,7 +770,7 @@ public:
     LabelDsymbol *isBranchToLabel;
 #endif
 
-    Statement *syntaxCopy();
+    InlineAsmStatement *syntaxCopy();
     void accept(Visitor *v) { v->visit(this); }
 };
 
@@ -797,7 +788,7 @@ public:
     Identifiers *labels;        // list of goto labels
     GotoStatements *gotos;      // of the goto labels, the equivalent statements they represent
 
-    Statement *syntaxCopy();
+    GccAsmStatement *syntaxCopy();
     void accept(Visitor *v) { v->visit(this); }
 };
 
@@ -812,7 +803,6 @@ public:
 #endif
 
     CompoundAsmStatement *syntaxCopy();
-    Statements *flatten(Scope *sc);
 
     void accept(Visitor *v) { v->visit(this); }
 
@@ -826,7 +816,7 @@ class ImportStatement : public Statement
 public:
     Dsymbols *imports;          // Array of Import's
 
-    Statement *syntaxCopy();
+    ImportStatement *syntaxCopy();
 
     void accept(Visitor *v) { v->visit(this); }
 };
