@@ -9,7 +9,9 @@
 
 #include "dmd/errors.h"
 #include "dmd/globals.h"
+#include "dmd/target.h"
 #include "driver/cl_options.h"
+#include "driver/timetrace.h"
 #include "driver/tool.h"
 #include "gen/logger.h"
 #include "llvm/ADT/Triple.h"
@@ -231,6 +233,34 @@ int internalLib(ArrayRef<const char *> args) {
   return libDriverMain(args);
 }
 
+std::string getOutputPath() {
+  std::string libName;
+
+  if (global.params.libname.length) { // explicit
+    // DMD adds the default extension if there is none
+    libName = opts::invokedByLDMD
+                  ? FileName::defaultExt(global.params.libname.ptr,
+                                         target.lib_ext.ptr)
+                  : global.params.libname.ptr;
+  } else { // infer from first object file
+    libName =
+        global.params.objfiles.length
+            ? FileName::removeExt(FileName::name(global.params.objfiles[0]))
+            : "a.out";
+    libName += '.';
+    libName += target.lib_ext.ptr;
+  }
+
+  // DMD creates static libraries in the objects directory (unless using an
+  // absolute output path via `-of`).
+  if (opts::invokedByLDMD && global.params.objdir.length &&
+      !FileName::absolute(libName.c_str())) {
+    libName = FileName::combine(global.params.objdir.ptr, libName.c_str());
+  }
+
+  return libName;
+}
+
 } // anonymous namespace
 
 ////////////////////////////////////////////////////////////////////////////////
@@ -238,8 +268,12 @@ int internalLib(ArrayRef<const char *> args) {
 static llvm::cl::opt<std::string> ar("ar", llvm::cl::desc("Archiver"),
                                      llvm::cl::Hidden, llvm::cl::ZeroOrMore);
 
+// path to the produced static library
+static std::string gStaticLibPath;
+
 int createStaticLibrary() {
   Logger::println("*** Creating static library ***");
+  ::TimeTraceScope timeScope("Create static library");
 
   const bool isTargetMSVC =
       global.params.targetTriple->isWindowsMSVCEnvironment();
@@ -263,6 +297,11 @@ int createStaticLibrary() {
     tool = getProgram(isTargetMSVC ? "lib.exe" : "ar", &ar);
   }
 
+  // remember output path for later
+  gStaticLibPath = getOutputPath();
+
+  createDirectoryForFileOrFail(gStaticLibPath);
+
   // build arguments
   std::vector<std::string> args;
 
@@ -276,34 +315,10 @@ int createStaticLibrary() {
     args.push_back("/NOLOGO");
   }
 
-  // output filename
-  std::string libName;
-  if (global.params.libname.length) { // explicit
-    // DMD adds the default extension if there is none
-    libName = opts::invokedByLDMD
-                  ? FileName::defaultExt(global.params.libname.ptr,
-                                         global.lib_ext.ptr)
-                  : global.params.libname.ptr;
-  } else { // infer from first object file
-    libName =
-        global.params.objfiles.length
-            ? FileName::removeExt(FileName::name(global.params.objfiles[0]))
-            : "a.out";
-    libName += '.';
-    libName += global.lib_ext.ptr;
-  }
-
-  // DMD creates static libraries in the objects directory (unless using an
-  // absolute output path via `-of`).
-  if (opts::invokedByLDMD && global.params.objdir.length &&
-      !FileName::absolute(libName.c_str())) {
-    libName = FileName::combine(global.params.objdir.ptr, libName.c_str());
-  }
-
   if (isTargetMSVC) {
-    args.push_back("/OUT:" + libName);
+    args.push_back("/OUT:" + gStaticLibPath);
   } else {
-    args.push_back(libName);
+    args.push_back(gStaticLibPath);
   }
 
   // object files
@@ -324,9 +339,6 @@ int createStaticLibrary() {
       args.push_back(std::string("/DEF:") + global.params.deffile.ptr);
   }
 
-  // create path to the library
-  createDirectoryForFileOrFail(libName);
-
   if (useInternalArchiver) {
     const auto fullArgs =
         getFullArgs(tool.c_str(), args, global.params.verbose);
@@ -342,3 +354,8 @@ int createStaticLibrary() {
   // invoke external archiver
   return executeToolAndWait(tool, args, global.params.verbose);
 }
+
+const char *getPathToProducedStaticLibrary() {
+  assert(!gStaticLibPath.empty());
+  return gStaticLibPath.c_str();
+}
