@@ -30,6 +30,7 @@
 #include "driver/cache.h"
 
 #include "dmd/errors.h"
+#include "dmd/target.h"
 #include "driver/cache_pruning.h"
 #include "driver/cl_options.h"
 #include "driver/cl_options_sanitizers.h"
@@ -217,7 +218,7 @@ void storeCacheFileName(llvm::StringRef cacheObjectHash,
   filePath = opts::cacheDir;
   llvm::sys::path::append(
       filePath, llvm::Twine("ircache_") + cacheObjectHash + "." +
-                    llvm::StringRef(global.obj_ext.ptr, global.obj_ext.length));
+                    llvm::StringRef(target.obj_ext.ptr, target.obj_ext.length));
 }
 
 // Output to `hash_os` all commandline flags, and try to skip the ones that have
@@ -321,7 +322,7 @@ void outputIR2ObjRelevantCmdlineArgs(llvm::raw_ostream &hash_os) {
 #if LDC_LLVM_VER >= 800
   const auto framePointerUsage = opts::framePointerUsage();
   if (framePointerUsage.hasValue())
-    hash_os << framePointerUsage.getValue();
+    hash_os << static_cast<int>(framePointerUsage.getValue());
 #else
   hash_os << opts::disableFPElim();
 #endif
@@ -480,7 +481,12 @@ void recoverObjectFile(llvm::StringRef cacheObjectHash,
 #if LDC_LLVM_VER >= 700
                                         llvm::sys::fs::CD_OpenExisting,
 #endif
-                                        llvm::sys::fs::F_Append)) {
+#if LDC_LLVM_VER >= 900
+                                        llvm::sys::fs::OF_Append
+#else
+                                        llvm::sys::fs::F_Append
+#endif
+                                        )) {
       error(Loc(), "Failed to open the cached file for writing: %s",
             cacheFile.c_str());
       fatal();
