@@ -16,6 +16,7 @@
 
 namespace llvm {
 class Function;
+class Triple;
 }
 
 namespace opts {
@@ -23,7 +24,9 @@ namespace opts {
 std::string getArchStr();
 llvm::Optional<llvm::Reloc::Model> getRelocModel();
 llvm::Optional<llvm::CodeModel::Model> getCodeModel();
-#if LDC_LLVM_VER >= 800
+#if LDC_LLVM_VER >= 1300
+llvm::Optional<llvm::FramePointerKind> framePointerUsage();
+#elif LDC_LLVM_VER >= 800
 llvm::Optional<llvm::FramePointer::FP> framePointerUsage();
 #else
 llvm::cl::boolOrDefault disableFPElim();
@@ -32,7 +35,9 @@ llvm::cl::boolOrDefault disableFPElim();
 bool disableRedZone();
 bool printTargetFeaturesHelp();
 
-llvm::TargetOptions InitTargetOptionsFromCodeGenFlags();
+llvm::TargetOptions
+InitTargetOptionsFromCodeGenFlags(const llvm::Triple &triple);
+
 std::string getCPUStr();
 std::string getFeaturesStr();
 #if LDC_LLVM_VER >= 1000
