@@ -60,12 +60,30 @@ static cl::opt<bool, true>
     createSharedLib("shared", cl::desc("Create shared library (DLL)"),
                     cl::ZeroOrMore, cl::location(global.params.dll));
 
-cl::opt<unsigned char> defaultToHiddenVisibility(
-    "fvisibility", cl::ZeroOrMore,
-    cl::desc("Default visibility of symbols (not relevant for Windows)"),
-    cl::values(clEnumValN(0, "default", "Export all symbols"),
-               clEnumValN(1, "hidden",
-                          "Only export symbols marked with 'export'")));
+cl::opt<SymbolVisibility> symbolVisibility(
+    "fvisibility", cl::ZeroOrMore, cl::desc("Default visibility of symbols"),
+    cl::init(SymbolVisibility::default_),
+    cl::values(
+        clEnumValN(
+            SymbolVisibility::default_, "default",
+            "Hidden for Windows targets without -shared, otherwise public"),
+        clEnumValN(SymbolVisibility::hidden, "hidden",
+                   "Only export symbols marked with 'export'"),
+        clEnumValN(SymbolVisibility::public_, "public", "Export all symbols")));
+
+cl::opt<DLLImport, true> dllimport(
+    "dllimport", cl::ZeroOrMore, cl::location(global.params.dllimport),
+    cl::desc("Windows only: which extern(D) global variables to dllimport "
+             "implicitly if not defined in a root module"),
+    cl::values(
+        clEnumValN(DLLImport::none, "none",
+                   "None (default with -link-defaultlib-shared=false)"),
+        clEnumValN(DLLImport::defaultLibsOnly, "defaultLibsOnly",
+                   "Only druntime/Phobos symbols (default with "
+                   "-link-defaultlib-shared and -fvisibility=hidden)."),
+        clEnumValN(DLLImport::all, "all",
+                   "All (default with -link-defaultlib-shared and "
+                   "-fvisibility=public)")));
 
 static cl::opt<bool, true> verbose("v", cl::desc("Verbose"), cl::ZeroOrMore,
                                    cl::location(global.params.verbose));
@@ -170,12 +188,15 @@ static cl::opt<CppStdRevision, true> cplusplus(
     cl::values(
         clEnumValN(CppStdRevisionCpp98, "c++98",
                    "Sets `__traits(getTargetInfo, \"cppStd\")` to `199711`"),
-        clEnumValN(CppStdRevisionCpp11, "c++11",
-                   "Sets `__traits(getTargetInfo, \"cppStd\")` to `201103`"),
+        clEnumValN(
+            CppStdRevisionCpp11, "c++11",
+            "Sets `__traits(getTargetInfo, \"cppStd\")` to `201103` (default)"),
         clEnumValN(CppStdRevisionCpp14, "c++14",
                    "Sets `__traits(getTargetInfo, \"cppStd\")` to `201402`"),
         clEnumValN(CppStdRevisionCpp17, "c++17",
-                   "Sets `__traits(getTargetInfo, \"cppStd\")` to `201703`")));
+                   "Sets `__traits(getTargetInfo, \"cppStd\")` to `201703`"),
+        clEnumValN(CppStdRevisionCpp20, "c++20",
+                   "Sets `__traits(getTargetInfo, \"cppStd\")` to `202002`")));
 
 static cl::opt<unsigned char, true> debugInfo(
     cl::desc("Generating debug information:"), cl::ZeroOrMore,
@@ -386,10 +407,16 @@ cl::list<std::string>
 cl::opt<std::string>
     moduleDeps("deps", cl::ValueOptional, cl::ZeroOrMore,
                cl::value_desc("filename"),
-               cl::desc("Write module dependencies to filename (only imports). "
+               cl::desc("Write module dependencies to <filename> (only imports). "
                         "'-deps' alone prints module dependencies "
                         "(imports/file/version/debug/lib)"));
 
+cl::opt<std::string>
+    makeDeps("makedeps", cl::ValueOptional, cl::ZeroOrMore,
+             cl::value_desc("filename"),
+             cl::desc("Write module dependencies in Makefile compatible format "
+                      "to <filename>/stdout (only imports)"));
+
 cl::opt<bool> m32bits("m32", cl::desc("32 bit target"), cl::ZeroOrMore);
 
 cl::opt<bool> m64bits("m64", cl::desc("64 bit target"), cl::ZeroOrMore);
@@ -487,8 +514,9 @@ cl::opt<uint32_t, true> hashThreshold(
     "hash-threshold", cl::ZeroOrMore, cl::location(global.params.hashThreshold),
     cl::desc("Hash symbol names longer than this threshold (experimental)"));
 
-cl::opt<bool> linkonceTemplates(
+static cl::opt<bool, true> linkonceTemplates(
     "linkonce-templates", cl::ZeroOrMore,
+    cl::location(global.params.linkonceTemplates),
     cl::desc(
         "Use linkonce_odr linkage for template symbols instead of weak_odr"));
 
@@ -521,13 +549,10 @@ cl::opt<unsigned, true> nestedTemplateDepth(
     cl::desc("Set maximum number of nested template instantiations"));
 
 // legacy options superseded by `-preview=dip<N>`
-static cl::opt<bool, true>
-    useDIP25("dip25", cl::ZeroOrMore, cl::location(global.params.useDIP25),
-             cl::desc("Implement DIP25 (sealed references)"), cl::ReallyHidden);
-static cl::opt<bool, true>
-    useDIP1000("dip1000", cl::ZeroOrMore, cl::location(global.params.vsafe),
-               cl::desc("Implement DIP1000 (scoped pointers)"),
-               cl::ReallyHidden);
+cl::opt<bool> useDIP25("dip25", cl::ZeroOrMore, cl::ReallyHidden,
+                       cl::desc("Implement DIP25 (sealed references)"));
+cl::opt<bool> useDIP1000("dip1000", cl::ZeroOrMore, cl::ReallyHidden,
+                         cl::desc("Implement DIP1000 (scoped pointers)"));
 static cl::opt<bool, true>
     useDIP1008("dip1008", cl::ZeroOrMore, cl::location(global.params.ehnogc),
                cl::desc("Implement DIP1008 (@nogc Throwable)"),
@@ -575,6 +600,32 @@ static cl::opt<DummyDataType, false, CoverageParser> coverageAnalysis(
              "Use -cov=<n> for n% minimum required coverage\n"
              "Use -cov=ctfe to include code executed during CTFE"));
 
+cl::opt<CoverageIncrement> coverageIncrement(
+    "cov-increment", cl::ZeroOrMore,
+    cl::desc("Set the type of coverage line count increment instruction"),
+    cl::init(CoverageIncrement::_default),
+    cl::values(clEnumValN(CoverageIncrement::_default, "default",
+                          "Use the default (atomic)"),
+               clEnumValN(CoverageIncrement::atomic, "atomic", "Atomic increment"),
+               clEnumValN(CoverageIncrement::nonatomic, "non-atomic",
+                          "Non-atomic increment (not thread safe)"),
+               clEnumValN(CoverageIncrement::boolean, "boolean",
+                          "Don't read, just set counter to 1")));
+
+// Compilation time tracing options
+cl::opt<bool> fTimeTrace(
+    "ftime-trace", cl::ZeroOrMore,
+    cl::desc("Turn on time profiler. Generates JSON file "
+             "based on the output filename (also see --ftime-trace-file)."));
+cl::opt<unsigned> fTimeTraceGranularity(
+    "ftime-trace-granularity", cl::ZeroOrMore, cl::init(500),
+    cl::desc(
+        "Minimum time granularity (in microseconds) traced by time profiler"));
+cl::opt<std::string>
+fTimeTraceFile("ftime-trace-file",
+               cl::desc("Specify time trace file destination"),
+               cl::value_desc("filename"));
+
 cl::opt<LTOKind> ltoMode(
     "flto", cl::ZeroOrMore, cl::desc("Set LTO mode, requires linker support"),
     cl::init(LTO_None),
@@ -678,18 +729,33 @@ void createClashingOptions() {
 /// to be useful for end users from the -help output.
 void hideLLVMOptions() {
   static const char *const hiddenOptions[] = {
-      "aarch64-neon-syntax", "addrsig", "arm-add-build-attributes",
+      "aarch64-neon-syntax", "aarch64-use-aa",
+      "abort-on-max-devirt-iterations-reached",
+      "addrsig", "allow-ginsert-as-artifact",
+      "amdgpu-bypass-slow-div", "amdgpu-disable-loop-alignment",
+      "amdgpu-disable-power-sched", "amdgpu-dpp-combine",
+      "amdgpu-dump-hsa-metadata", "amdgpu-enable-flat-scratch",
+      "amdgpu-enable-global-sgpr-addr", "amdgpu-enable-merge-m0",
+      "amdgpu-promote-alloca-to-vector-limit",
+      "amdgpu-reserve-vgpr-for-sgpr-spill", "amdgpu-sdwa-peephole",
+      "amdgpu-use-aa-in-codegen", "amdgpu-verify-hsa-metadata",
+      "amdgpu-vgpr-index-mode", "arm-add-build-attributes",
       "arm-implicit-it", "asm-instrumentation", "asm-show-inst",
       "atomic-counter-update-promoted", "atomic-first-counter",
+      "basic-block-sections",
       "basicblock-sections", "bounds-checking-single-trap",
+      "cfg-hide-cold-paths",
       "cfg-hide-deoptimize-paths", "cfg-hide-unreachable-paths",
       "code-model", "cost-kind", "cppfname", "cppfor", "cppgen",
       "cvp-dont-add-nowrap-flags",
       "cvp-dont-process-adds", "debug-counter", "debug-entry-values",
       "debugger-tune", "debugify-level", "debugify-quiet",
       "denormal-fp-math", "denormal-fp-math-f32", "disable-debug-info-verifier",
-      "disable-objc-arc-checkforcfghazards", "disable-spill-fusing",
-      "do-counter-promotion", "dwarf64", "emit-call-site-info",
+      "disable-i2p-p2i-opt",
+      "disable-objc-arc-checkforcfghazards", "disable-promote-alloca-to-lds",
+      "disable-promote-alloca-to-vector", "disable-slp-vectorization",
+      "disable-spill-fusing",
+      "do-counter-promotion", "dot-cfg-mssa", "dwarf64", "emit-call-site-info",
       "emscripten-cxx-exceptions-allowed",
       "emscripten-cxx-exceptions-whitelist",
       "emulated-tls", "enable-correct-eh-support",
@@ -703,17 +769,20 @@ void hideLLVMOptions() {
       "enable-no-nans-fp-math", "enable-no-signed-zeros-fp-math",
       "enable-no-trapping-fp-math", "enable-objc-arc-annotations",
       "enable-objc-arc-opts", "enable-pie", "enable-scoped-noalias",
+      "enable-split-backedge-in-load-pre",
       "enable-tbaa", "enable-unsafe-fp-math", "exception-model",
       "exhaustive-register-search", "expensive-combines",
+      "experimental-debug-variable-locations",
       "fatal-assembler-warnings", "filter-print-funcs",
-      "force-dwarf-frame-section", "gpsize", "hash-based-counter-split",
-      "hot-cold-split",
+      "force-dwarf-frame-section", "force-opaque-pointers",
+      "gpsize", "hash-based-counter-split",
+      "hot-cold-split", "ignore-xcoff-visibility",
       "imp-null-check-page-size", "imp-null-max-insts-to-consider",
       "import-all-index", "incremental-linker-compatible",
       "instcombine-code-sinking", "instcombine-guard-widening-window",
       "instcombine-max-iterations", "instcombine-max-num-phis",
       "instcombine-maxarray-size", "instcombine-negator-enabled",
-      "instcombine-negator-max-depth",
+      "instcombine-negator-max-depth", "instcombine-unsafe-select-transform",
       "instrprof-atomic-counter-update-all", "internalize-public-api-file",
       "internalize-public-api-list", "iterative-counter-promotion",
       "join-liveintervals", "jump-table-type", "limit-float-precision",
@@ -721,12 +790,12 @@ void hideLLVMOptions() {
       "max-counter-promotions", "max-counter-promotions-per-loop",
       "mc-relax-all", "mc-x86-disable-arith-relaxation", "meabi",
       "memop-size-large", "memop-size-range", "merror-missing-parenthesis",
-      "merror-noncontigious-register", "mfuture-regs", "mips-compact-branches",
-      "mips16-constant-islands", "mips16-hard-float", "mir-strip-debugify-only",
-      "mlsm", "mno-compound",
+      "merror-noncontigious-register", "mfuture-regs", "mhvx",
+      "mips-compact-branches", "mips16-constant-islands", "mips16-hard-float",
+      "mir-strip-debugify-only", "mlsm", "mno-compound",
       "mno-fixup", "mno-ldc1-sdc1", "mno-pairing", "mwarn-missing-parenthesis",
       "mwarn-noncontigious-register", "mwarn-sign-mismatch",
-      "no-discriminators", "no-xray-index",
+      "no-discriminators", "no-type-check", "no-xray-index",
       "nozero-initialized-in-bss", "nvptx-sched4reg",
       "objc-arc-annotation-target-identifier", "pie-copy-relocations",
       "poison-checking-function-local",
@@ -735,6 +804,7 @@ void hideLLVMOptions() {
       "print-before-all", "print-machineinstrs", "print-module-scope",
       "profile-estimator-loop-weight", "profile-estimator-loop-weight",
       "profile-file", "profile-info-file", "profile-verifier-noassert",
+      "pseudo-probe-for-profiling",
       "r600-ir-structurize", "rdf-dump", "rdf-limit", "recip", "regalloc",
       "relax-elf-relocations", "remarks-section", "rewrite-map-file", "rng-seed",
       "runtime-counter-relocation", "safepoint-ir-verifier-print-only",
@@ -742,20 +812,26 @@ void hideLLVMOptions() {
       "sample-profile-check-sample-coverage",
       "sample-profile-inline-hot-threshold",
       "sample-profile-max-propagate-iterations", "shrink-wrap", "simplify-mir",
+      "skip-ret-exit-block",
       "speculative-counter-promotion-max-exiting",
       "speculative-counter-promotion-to-loop", "spiller", "spirv-debug",
       "spirv-erase-cl-md", "spirv-lower-const-expr", "spirv-mem2reg",
-      "spirv-no-deref-attr", "spirv-text",
-      "spv-lower-saddwithoverflow-validate", "spvbool-validate",
-      "spvmemmove-validate",
-      "stack-alignment", "stack-size-section", "stack-symbol-ordering",
+      "spirv-no-deref-attr", "spirv-text", "spirv-verify-regularize-passes",
+      "split-machine-functions", "spv-lower-saddwithoverflow-validate",
+      "spvbool-validate", "spvmemmove-validate",
+      "stack-alignment", "stack-protector-guard",
+      "stack-protector-guard-offset", "stack-protector-guard-reg",
+      "stack-size-section", "stack-symbol-ordering",
       "stackmap-version", "static-func-full-module-prefix",
-      "static-func-strip-dirname-prefix", "stats", "stats-json", "strip-debug",
-      "struct-path-tbaa", "summary-file", "tail-predication", "tailcallopt",
+      "static-func-strip-dirname-prefix", "stats", "stats-json", "strict-dwarf",
+      "strip-debug", "struct-path-tbaa", "summary-file", "tail-predication",
+      "tailcallopt", "thinlto-assume-merged",
       "thread-model", "time-passes", "time-trace-granularity", "tls-size",
-      "unfold-element-atomic-memcpy-max-elements", "unique-bb-section-names",
+      "unfold-element-atomic-memcpy-max-elements",
+      "unique-basic-block-section-names", "unique-bb-section-names",
       "unique-section-names", "unit-at-a-time", "use-ctors",
-      "verify-debug-info", "verify-dom-info", "verify-loop-info",
+      "vec-extabi", "verify-debug-info", "verify-dom-info",
+      "verify-legalizer-debug-locs", "verify-loop-info",
       "verify-loop-lcssa", "verify-machine-dom-info", "verify-regalloc",
       "verify-region-info", "verify-scev", "verify-scev-maps",
       "vp-counters-per-site", "vp-static-alloc",
@@ -763,6 +839,7 @@ void hideLLVMOptions() {
       "x86-branches-within-32B-boundaries", "x86-early-ifcvt",
       "x86-pad-max-prefix-size",
       "x86-recip-refinement-steps", "x86-use-vzeroupper",
+      "xcoff-traceback-table",
 
       // We enable -fdata-sections/-ffunction-sections by default where it makes
       // sense for reducing code size, so hide them to avoid confusion.
