@@ -44,10 +44,6 @@
 #include <sys/stat.h>
 #endif
 
-#ifdef HAVE_SC_ARG_MAX
-#include <unistd.h>
-#endif
-
 namespace ls = llvm::sys;
 
 // We reuse DMD's response file parsing routine for maximum compatibility - it
@@ -182,6 +178,7 @@ Where:\n\
 #endif
 "  -fPIC             generate position independent code\n\
   -g                add symbolic debug info\n\
+  -gdwarf=<version> add DWARF symbolic debug info\n\
   -gf               emit debug info for all referenced types\n\
   -gs               always emit stack frame\n"
 #if 0
@@ -209,6 +206,9 @@ Where:\n\
 #endif
 "  -m64              generate 64 bit code\n\
   -main             add default main() (e.g. for unittesting)\n\
+  -makedeps         print module dependencies in Makefile compatible format to stdout\n\
+  -makedeps=<filename>\n\
+                    write module dependencies in Makefile compatible format to filename (only imports)\n\
   -man              open web browser on manual page\n"
 #if 0
 "  -map              generate linker .map file\n"
@@ -239,6 +239,7 @@ Where:\n\
                     list all revertable language changes\n\
   -run <srcfile>    compile, link, and run the program srcfile\n\
   -shared           generate shared library (DLL)\n\
+  -target=<triple>  use <triple> as <arch>-[<vendor>-]<os>[-<cenv>[-<cppenv]]\n\
   -transition=<name>\n\
                     help with language change identified by 'name'\n\
   -transition=[h|help|?]\n\
@@ -401,38 +402,55 @@ void translateArgs(const llvm::SmallVectorImpl<const char *> &ldmdArgs,
         // Parse:
         //      -check=[assert|bounds|in|invariant|out|switch][=[on|off]]
         const char *arg = p + 7;
-        const auto argLength = strlen(arg);
-        bool enabled = false;
-        size_t kindLength = 0;
-        if (argLength > 3 && memcmp(arg + argLength - 3, "=on", 3) == 0) {
-          enabled = true;
-          kindLength = argLength - 3;
-        } else if (argLength > 4 &&
-                   memcmp(arg + argLength - 4, "=off", 4) == 0) {
-          enabled = false;
-          kindLength = argLength - 4;
+        if (strcmp(arg, "on") == 0) {
+          ldcArgs.push_back("-boundscheck=on");
+          ldcArgs.push_back("-enable-asserts");
+          ldcArgs.push_back("-enable-preconditions");
+          ldcArgs.push_back("-enable-invariants");
+          ldcArgs.push_back("-enable-postconditions");
+          ldcArgs.push_back("-enable-switch-errors");
+        } else if (strcmp(arg, "off") == 0) {
+          ldcArgs.push_back("-boundscheck=off");
+          ldcArgs.push_back("-disable-asserts");
+          ldcArgs.push_back("-disable-preconditions");
+          ldcArgs.push_back("-disable-invariants");
+          ldcArgs.push_back("-disable-postconditions");
+          ldcArgs.push_back("-disable-switch-errors");
         } else {
-          enabled = true;
-          kindLength = argLength;
-        }
-
-        const auto check = [&](size_t dmdLength, const char *dmd,
-                               const char *ldc) {
-          if (kindLength == dmdLength && memcmp(arg, dmd, dmdLength) == 0) {
-            ldcArgs.push_back(concat(enabled ? "-enable-" : "-disable-", ldc));
-            return true;
+          const auto argLength = strlen(arg);
+          bool enabled = false;
+          size_t kindLength = 0;
+          if (argLength > 3 && memcmp(arg + argLength - 3, "=on", 3) == 0) {
+            enabled = true;
+            kindLength = argLength - 3;
+          } else if (argLength > 4 &&
+                     memcmp(arg + argLength - 4, "=off", 4) == 0) {
+            enabled = false;
+            kindLength = argLength - 4;
+          } else {
+            enabled = true;
+            kindLength = argLength;
           }
-          return false;
-        };
 
-        if (kindLength == 6 && memcmp(arg, "bounds", 6) == 0) {
-          ldcArgs.push_back(enabled ? "-boundscheck=on" : "-boundscheck=off");
-        } else if (!(check(6, "assert", "asserts") ||
-                     check(2, "in", "preconditions") ||
-                     check(9, "invariant", "invariants") ||
-                     check(3, "out", "postconditions") ||
-                     check(6, "switch", "switch-errors"))) {
-          goto Lerror;
+          const auto check = [&](size_t dmdLength, const char *dmd,
+                                 const char *ldc) {
+            if (kindLength == dmdLength && memcmp(arg, dmd, dmdLength) == 0) {
+              ldcArgs.push_back(
+                  concat(enabled ? "-enable-" : "-disable-", ldc));
+              return true;
+            }
+            return false;
+          };
+
+          if (kindLength == 6 && memcmp(arg, "bounds", 6) == 0) {
+            ldcArgs.push_back(enabled ? "-boundscheck=on" : "-boundscheck=off");
+          } else if (!(check(6, "assert", "asserts") ||
+                       check(2, "in", "preconditions") ||
+                       check(9, "invariant", "invariants") ||
+                       check(3, "out", "postconditions") ||
+                       check(6, "switch", "switch-errors"))) {
+            goto Lerror;
+          }
         }
       }
       /* -checkaction
@@ -476,7 +494,11 @@ void translateArgs(const llvm::SmallVectorImpl<const char *> &ldmdArgs,
       /* -g
        * -gc
        */
-      else if (strcmp(p + 1, "gf") == 0) {
+      else if (startsWith(p + 1, "gdwarf=")) {
+        ldcArgs.push_back("-gdwarf"); // implies -g and enforces DWARF for MSVC
+        ldcArgs.push_back("-dwarf-version");
+        ldcArgs.push_back(p + 8);
+      } else if (strcmp(p + 1, "gf") == 0) {
         ldcArgs.push_back("-g");
       } else if (strcmp(p + 1, "gs") == 0) {
 #if LDC_LLVM_VER >= 1100
@@ -529,7 +551,9 @@ void translateArgs(const llvm::SmallVectorImpl<const char *> &ldmdArgs,
       }
       /* -verror-style
        */
-      else if (startsWith(p + 1, "mcpu=")) {
+      else if (startsWith(p + 1, "target=")) {
+        ldcArgs.push_back(concat("-mtriple=", p + 8));
+      } else if (startsWith(p + 1, "mcpu=")) {
         const char *c = p + 6;
         if (strcmp(c, "?") == 0 || strcmp(c, "h") == 0 ||
             strcmp(c, "help") == 0) {
@@ -691,7 +715,13 @@ void translateArgs(const llvm::SmallVectorImpl<const char *> &ldmdArgs,
       }
     } else {
       const auto ext = ls::path::extension(p);
-      if (ext.equals_lower(".exe")) {
+      if (
+#if LDC_LLVM_VER >= 1300
+        ext.equals_insensitive(".exe")
+#else
+        ext.equals_lower(".exe")
+#endif
+          ) {
         // should be for Windows targets only
         ldcArgs.push_back(concat("-of=", p));
         continue;
