@@ -36,6 +36,10 @@ void addMscrtLibs(bool useInternalToolchain, std::vector<std::string> &args) {
   // We need the vcruntime lib for druntime's exception handling (ldc.eh_msvc).
   // Pick one of the 4 variants matching the selected main UCRT lib.
 
+#if LDC_LLVM_VER >= 1300
+#define contains_lower contains_insensitive
+#define endswith_lower endswith_insensitive
+#endif
   if (useInternalToolchain) {
     assert(mscrtlibName.contains_lower("vcruntime"));
     return;
@@ -109,7 +113,7 @@ int linkObjToBinaryMSVC(llvm::StringRef outputPath,
 
   // specify that the image will contain a table of safe exception handlers
   // and can handle addresses >2GB (32bit only)
-  if (!global.params.is64bit) {
+  if (global.params.targetTriple->isArch32Bit()) {
     args.push_back("/SAFESEH");
     args.push_back("/LARGEADDRESSAWARE");
   }
@@ -144,6 +148,12 @@ int linkObjToBinaryMSVC(llvm::StringRef outputPath,
     args.push_back(objfile);
   }
 
+  // add precompiled rt.dso_windows object file (in lib directory) when linking
+  // against shared druntime
+  if (!defaultLibNames.empty() && linkAgainstSharedDefaultLibs()) {
+    args.push_back("dso_windows.obj");
+  }
+
   // .res/.def files
   if (global.params.resfile.length)
     args.push_back(global.params.resfile.ptr);
@@ -240,7 +250,7 @@ int linkObjToBinaryMSVC(llvm::StringRef outputPath,
 
 #if LDC_WITH_LLD
   if (useInternalLLDForLinking() ||
-      (useInternalToolchain && opts::linker.empty() && !opts::isUsingLTO())) {
+      (useInternalToolchain && opts::linker.empty())) {
     const auto fullArgs = getFullArgs("lld-link", args, global.params.verbose);
 
     const bool success = lld::coff::link(fullArgs,
