@@ -10,7 +10,9 @@
 #include "driver/linker.h"
 
 #include "dmd/errors.h"
+#include "dmd/target.h"
 #include "driver/cl_options.h"
+#include "driver/timetrace.h"
 #include "driver/tool.h"
 #include "gen/llvm.h"
 #include "gen/logger.h"
@@ -92,13 +94,13 @@ int linkObjToBinaryMSVC(llvm::StringRef outputPath,
 
 //////////////////////////////////////////////////////////////////////////////
 
-static std::string getOutputName() {
+static std::string getOutputPath() {
   const auto &triple = *global.params.targetTriple;
   const bool sharedLib = global.params.dll;
 
   const char *extension = nullptr;
   if (sharedLib) {
-    extension = global.dll_ext.ptr;
+    extension = target.dll_ext.ptr;
   } else if (triple.isOSWindows()) {
     extension = "exe";
   } else if (triple.getArch() == llvm::Triple::wasm32 ||
@@ -202,10 +204,11 @@ bool useInternalLLDForLinking() {
   return linkInternally
 #if LDC_WITH_LLD
          ||
-         // DWARF debuginfos for MSVC require LLD
-         (opts::emitDwarfDebugInfo && linkInternally.getNumOccurrences() == 0 &&
-          opts::linker.empty() && !opts::isUsingLTO() &&
-          global.params.targetTriple->isWindowsMSVCEnvironment())
+         // MSVC: DWARF debuginfos and LTO require LLD
+         (linkInternally.getNumOccurrences() == 0 && // not explicitly disabled
+          opts::linker.empty() && // no explicitly selected linker
+          global.params.targetTriple->isWindowsMSVCEnvironment() &&
+          (opts::emitDwarfDebugInfo || opts::isUsingLTO()))
 #endif
       ;
 }
@@ -244,8 +247,7 @@ llvm::StringRef getMscrtLibName(const bool *useInternalToolchain) {
   if (useInternal) {
     return "vcruntime140";
   } else {
-    // default to static release variant
-    return linkFullyStatic() != llvm::cl::BOU_FALSE ? "libcmt" : "msvcrt";
+    return linkAgainstSharedDefaultLibs() ? "msvcrt" : "libcmt";
   }
 }
 
@@ -283,9 +285,10 @@ static std::string gExePath;
 
 int linkObjToBinary() {
   Logger::println("*** Linking executable ***");
+  TimeTraceScope timeScope("Linking executable");
 
   // remember output path for later
-  gExePath = getOutputName();
+  gExePath = getOutputPath();
 
   createDirectoryForFileOrFail(gExePath);
 
@@ -298,6 +301,11 @@ int linkObjToBinary() {
   return linkObjToBinaryGcc(gExePath, defaultLibNames);
 }
 
+const char *getPathToProducedBinary() {
+  assert(!gExePath.empty());
+  return gExePath.c_str();
+}
+
 //////////////////////////////////////////////////////////////////////////////
 
 void deleteExeFile() {
@@ -309,6 +317,8 @@ void deleteExeFile() {
 //////////////////////////////////////////////////////////////////////////////
 
 int runProgram() {
+  TimeTraceScope timeScope("Run user program");
+
   assert(!gExePath.empty());
 
   // Run executable
