@@ -35,6 +35,7 @@
 #include "driver/linker.h"
 #include "driver/plugins.h"
 #include "driver/targetmachine.h"
+#include "driver/timetrace.h"
 #include "gen/abi.h"
 #include "gen/cl_helpers.h"
 #include "gen/irstate.h"
@@ -63,6 +64,9 @@
 #include "llvm/Support/TargetRegistry.h"
 #include "llvm/Support/TargetSelect.h"
 #include "llvm/Target/TargetMachine.h"
+#if LDC_MLIR_ENABLED
+#include "mlir/IR/MLIRContext.h"
+#endif
 #include <assert.h>
 #include <limits.h>
 #include <stdio.h>
@@ -338,7 +342,7 @@ void parseCommandLine(Strings &sourceFiles) {
       global.params.docdir.length || global.params.docname.length;
 
   global.params.jsonfilename = opts::fromPathString(jsonFile);
-  if (global.params.jsonfilename.length) {
+  if (global.params.jsonfilename.length || !jsonFields.empty()) {
     global.params.doJsonGeneration = true;
   }
 
@@ -357,11 +361,17 @@ void parseCommandLine(Strings &sourceFiles) {
   global.params.mixinFile = opts::fromPathString(mixinFile).ptr;
 
   if (moduleDeps.getNumOccurrences() != 0) {
-    global.params.moduleDeps = new OutBuffer;
+    global.params.moduleDeps = createOutBuffer();
     if (!moduleDeps.empty())
       global.params.moduleDepsFile = opts::dupPathString(moduleDeps);
   }
 
+  if (makeDeps.getNumOccurrences() != 0) {
+    global.params.emitMakeDeps = true;
+    if (!makeDeps.empty())
+      global.params.makeDepsFile = opts::dupPathString(makeDeps);
+  }
+
 #if _WIN32
   const auto toWinPaths = [](Strings *paths) {
     if (!paths)
@@ -412,11 +422,22 @@ void parseCommandLine(Strings &sourceFiles) {
     parseRevertOption(global.params, id.c_str());
 
   if (global.params.useDIP1021) // DIP1021 implies DIP1000
-    global.params.vsafe = true;
-  if (global.params.vsafe) // DIP1000 implies DIP25
-    global.params.useDIP25 = true;
-  if (global.params.noDIP25)
-    global.params.useDIP25 = false;
+    global.params.useDIP1000 = FeatureState::enabled;
+  // legacy -dip1000 option
+  if (global.params.useDIP1000 == FeatureState::default_ &&
+      opts::useDIP1000.getNumOccurrences()) {
+    global.params.useDIP1000 =
+        opts::useDIP1000 ? FeatureState::enabled : FeatureState::disabled;
+  }
+
+  if (global.params.useDIP1000 == FeatureState::enabled) // DIP1000 implies DIP25
+    global.params.useDIP25 = FeatureState::enabled;
+  // legacy -dip25 option
+  if (global.params.useDIP25 == FeatureState::default_ &&
+      opts::useDIP25.getNumOccurrences()) {
+    global.params.useDIP25 =
+        opts::useDIP25 ? FeatureState::enabled : FeatureState::disabled;
+  }
 
   global.params.output_o =
       (opts::output_o == cl::BOU_UNSET &&
@@ -429,8 +450,9 @@ void parseCommandLine(Strings &sourceFiles) {
   global.params.output_mlir = opts::output_mlir ? OUTPUTFLAGset : OUTPUTFLAGno;
   global.params.output_s = opts::output_s ? OUTPUTFLAGset : OUTPUTFLAGno;
 
-  templateLinkage = opts::linkonceTemplates ? LLGlobalValue::LinkOnceODRLinkage
-                                            : LLGlobalValue::WeakODRLinkage;
+  templateLinkage = global.params.linkonceTemplates
+                        ? LLGlobalValue::LinkOnceODRLinkage
+                        : LLGlobalValue::WeakODRLinkage;
 
   if (global.params.run || !runargs.empty()) {
     // FIXME: how to properly detect the presence of a PositionalEatsArgs
@@ -458,6 +480,12 @@ void parseCommandLine(Strings &sourceFiles) {
       global.params.run = false;
       error(Loc(), "Expected at least one argument to '-run'\n");
     }
+
+    // -run: enforce -oq and -cleanup-obj for non-conflicting temporary objects
+    if (global.params.run) {
+      global.params.fullyQualifiedObjectFiles = true;
+      global.params.cleanupObjectFiles = true;
+    }
   }
 
   sourceFiles.reserve(fileList.size());
@@ -479,19 +507,19 @@ void parseCommandLine(Strings &sourceFiles) {
     if (!ext) {
       // keep things as they are
     } else if (opts::output_ll.getNumOccurrences() == 0 &&
-               strcmp(ext, global.ll_ext.ptr) == 0) {
+               strcmp(ext, ll_ext.ptr) == 0) {
       global.params.output_ll = OUTPUTFLAGset;
       global.params.output_o = OUTPUTFLAGno;
     } else if (opts::output_bc.getNumOccurrences() == 0 &&
-               strcmp(ext, global.bc_ext.ptr) == 0) {
+               strcmp(ext, bc_ext.ptr) == 0) {
       global.params.output_bc = OUTPUTFLAGset;
       global.params.output_o = OUTPUTFLAGno;
     } else if (opts::output_s.getNumOccurrences() == 0 &&
-               strcmp(ext, global.s_ext.ptr) == 0) {
+               strcmp(ext, s_ext.ptr) == 0) {
       global.params.output_s = OUTPUTFLAGset;
       global.params.output_o = OUTPUTFLAGno;
     } else if (opts::output_mlir.getNumOccurrences() == 0 &&
-               strcmp(ext, global.mlir_ext.ptr) == 0) {
+               strcmp(ext, mlir_ext.ptr) == 0) {
       global.params.output_mlir = OUTPUTFLAGset;
       global.params.output_o = OUTPUTFLAGno;
     }
@@ -705,8 +733,10 @@ void registerPredefinedTargetVersions() {
   }
 
   // Set versions for arch bitwidth
-  if (global.params.isLP64) {
+  if (gDataLayout->getPointerSizeInBits() == 64) {
     VersionCondition::addPredefinedGlobalIdent("D_LP64");
+  } else if (triple.isArch64Bit()) {
+    VersionCondition::addPredefinedGlobalIdent("D_X32");
   } else if (triple.isArch16Bit()) {
     VersionCondition::addPredefinedGlobalIdent("D_P16");
   }
@@ -732,8 +762,8 @@ void registerPredefinedTargetVersions() {
   switch (triple.getOS()) {
   case llvm::Triple::Win32:
     VersionCondition::addPredefinedGlobalIdent("Windows");
-    VersionCondition::addPredefinedGlobalIdent(global.params.is64bit ? "Win64"
-                                                                     : "Win32");
+    VersionCondition::addPredefinedGlobalIdent(triple.isArch64Bit() ? "Win64"
+                                                                    : "Win32");
     if (triple.isWindowsMSVCEnvironment()) {
       VersionCondition::addPredefinedGlobalIdent("CRuntime_Microsoft");
       VersionCondition::addPredefinedGlobalIdent("CppRuntime_Microsoft");
@@ -757,6 +787,9 @@ void registerPredefinedTargetVersions() {
       VersionCondition::addPredefinedGlobalIdent("CRuntime_Bionic");
     } else if (triple.isMusl()) {
       VersionCondition::addPredefinedGlobalIdent("CRuntime_Musl");
+      VersionCondition::addPredefinedGlobalIdent("CppRuntime_Gcc");
+      // use libunwind for backtraces
+      VersionCondition::addPredefinedGlobalIdent("DRuntime_Use_Libunwind");
     } else if (global.params.isUClibcEnvironment) {
       VersionCondition::addPredefinedGlobalIdent("CRuntime_UClibc");
     } else {
@@ -778,6 +811,12 @@ void registerPredefinedTargetVersions() {
     break;
   case llvm::Triple::FreeBSD:
     VersionCondition::addPredefinedGlobalIdent("FreeBSD");
+    if (unsigned major = triple.getOSMajorVersion()) {
+      const auto withMajor = "FreeBSD_" + std::to_string(major);
+      VersionCondition::addPredefinedGlobalIdent(withMajor.c_str());
+    } else {
+      warning(Loc(), "FreeBSD major version not specified in target triple");
+    }
     VersionCondition::addPredefinedGlobalIdent("Posix");
     VersionCondition::addPredefinedGlobalIdent("CppRuntime_Clang");
     break;
@@ -830,7 +869,9 @@ void registerPredefinedTargetVersions() {
       VersionCondition::addPredefinedGlobalIdent("Android");
     } else {
       llvm::StringRef osName = triple.getOSName();
-      if (!osName.empty() && osName != "unknown" && osName != "none") {
+      if (osName.empty() || osName == "unknown" || osName == "none") {
+        VersionCondition::addPredefinedGlobalIdent("FreeStanding");
+      } else {
         warning(Loc(), "unknown target OS: %s", osName.str().c_str());
       }
     }
@@ -911,6 +952,25 @@ void registerPredefinedVersions() {
 #undef STR
 }
 
+static llvm::SmallString<32> tempObjectsDir;
+
+// With -cleanup-obj, potentially invoked (once) by Module.setOutfilename() to
+// create a unique temporary directory for generated object files (if -od hasn't
+// been specified), e.g., `/tmp/objtmp-ldc-688445`.
+const char *createTempObjectsDir() {
+  assert(tempObjectsDir.empty());
+
+  auto ec = llvm::sys::fs::createUniqueDirectory("objtmp-ldc", tempObjectsDir);
+  if (ec) {
+    error(Loc(),
+          "failed to create temporary directory for object files: %s\n%s",
+          tempObjectsDir.c_str(), ec.message().c_str());
+    fatal();
+  }
+
+  return tempObjectsDir.c_str();
+}
+
 /// LDC's entry point, C main.
 /// Without `-lowmem`, we need to switch to the bump-pointer allocation scheme
 /// right from the start, before any module ctors are run, so we need this hook
@@ -986,6 +1046,10 @@ int cppmain() {
     fatal();
   }
 
+  if (opts::fTimeTrace) {
+    initializeTimeTrace(opts::fTimeTraceGranularity, 0, opts::allArguments[0]);
+  }
+
   // Set up the TargetMachine.
   const auto arch = getArchStr();
   if ((m32bits || m64bits) && (!arch.empty() || !mTargetTriple.empty())) {
@@ -1022,63 +1086,65 @@ int cppmain() {
   static llvm::DataLayout DL = gTargetMachine->createDataLayout();
   gDataLayout = &DL;
 
-  {
-    llvm::Triple *triple = new llvm::Triple(gTargetMachine->getTargetTriple());
-    global.params.targetTriple = triple;
-    global.params.isLinux = triple->isOSLinux();
-    global.params.isOSX = triple->isOSDarwin();
-    global.params.isWindows = triple->isOSWindows();
-    global.params.isFreeBSD = triple->isOSFreeBSD();
-    global.params.isOpenBSD = triple->isOSOpenBSD();
-    global.params.isDragonFlyBSD = triple->isOSDragonFly();
-    global.params.isSolaris = triple->isOSSolaris();
-    global.params.isLP64 = gDataLayout->getPointerSizeInBits() == 64;
-    global.params.is64bit = triple->isArch64Bit();
-    global.params.hasObjectiveC = objc_isSupported(*triple);
-    global.params.dwarfVersion = gTargetMachine->Options.MCOptions.DwarfVersion;
-    // mscoff enables slightly different handling of interface functions
-    // in the front end
-    global.params.mscoff = triple->isKnownWindowsMSVCEnvironment();
-    if (global.params.mscoff)
-      global.obj_ext = {3, "obj"};
-  }
+  llvm::Triple *triple = new llvm::Triple(gTargetMachine->getTargetTriple());
+  global.params.targetTriple = triple;
+
+  global.params.dwarfVersion = gTargetMachine->Options.MCOptions.DwarfVersion;
 
   // -gdwarf implies -g if not specified explicitly
   if (opts::emitDwarfDebugInfo && global.params.symdebug == 0) {
     global.params.symdebug = 1;
   }
 
+  if (triple->isOSWindows()) {
+    const auto v = opts::symbolVisibility.getValue();
+    global.params.dllexport =
+        v == opts::SymbolVisibility::public_ ||
+        // default with -shared
+        (v == opts::SymbolVisibility::default_ && global.params.dll);
+    if (opts::dllimport.getNumOccurrences() == 0) {
+      global.params.dllimport =
+          !linkAgainstSharedDefaultLibs() ? DLLImport::none
+          : global.params.dllexport       ? DLLImport::all
+                                          : DLLImport::defaultLibsOnly;
+    }
+  } else {
+    global.params.dllexport = false;
+    global.params.dllimport = DLLImport::none;
+  }
+
   // allocate the target abi
   gABI = TargetABI::getTarget();
 
-  if (global.params.targetTriple->isOSWindows()) {
-    global.dll_ext = {3, "dll"};
-    if (global.params.mscoff) {
-      global.lib_ext = {3, "lib"};
-    } else {
-      global.lib_ext = {1, "a"};
-    }
-  } else {
-    if (global.params.targetTriple->isOSDarwin()) {
-      global.dll_ext = {5, "dylib"};
-    } else {
-      global.dll_ext = {2, "so"};
-    }
-    global.lib_ext = {1, "a"};
-  }
-
-  opts::initializeInstrumentationOptionsFromCmdline(
-      *global.params.targetTriple);
+  opts::initializeInstrumentationOptionsFromCmdline(*triple);
 
   loadAllPlugins();
 
-  Strings libmodules;
-  return mars_mainBody(global.params, files, libmodules);
+  int status;
+  {
+    TimeTraceScope timeScope("ExecuteCompiler");
+    Strings libmodules;
+    status = mars_mainBody(global.params, files, libmodules);
+  }
+
+  // try to remove the temp objects dir if created for -cleanup-obj
+  if (!tempObjectsDir.empty())
+    llvm::sys::fs::remove(tempObjectsDir);
+
+  std::string fTimeTraceFile = opts::fTimeTraceFile;
+  writeTimeTraceProfile(fTimeTraceFile.empty() ? "" : fTimeTraceFile.c_str());
+  deinitializeTimeTrace();
+
+  llvm::llvm_shutdown();
+
+  return status;
 }
 
 void codegenModules(Modules &modules) {
   // Generate one or more object/IR/bitcode files/dcompute kernels.
   if (global.params.obj && !modules.empty()) {
+    TimeTraceScope timeScope("Codegen all modules");
+
 #if LDC_MLIR_ENABLED
     mlir::MLIRContext mlircontext;
     ldc::CodeGenerator cg(getGlobalContext(), mlircontext,
@@ -1108,6 +1174,11 @@ void codegenModules(Modules &modules) {
       const auto atCompute = hasComputeAttr(m);
       if (atCompute == DComputeCompileFor::hostOnly ||
           atCompute == DComputeCompileFor::hostAndDevice) {
+        TimeTraceScope timeScope(
+            ("Codegen module " + llvm::SmallString<20>(m->toChars()))
+                .str()
+                .c_str(),
+            m->loc);
 #if LDC_MLIR_ENABLED
         if (global.params.output_mlir == OUTPUTFLAGset)
           cg.emitMLIR(m);
@@ -1133,8 +1204,15 @@ void codegenModules(Modules &modules) {
     }
 
     if (!computeModules.empty()) {
-      for (auto &mod : computeModules)
+      TimeTraceScope timeScope("Codegen DCompute device modules");
+      for (auto &mod : computeModules) {
+        TimeTraceScope timeScope(("Codegen DCompute device module " +
+                                  llvm::SmallString<20>(mod->toChars()))
+                                     .str()
+                                     .c_str(),
+                                 mod->loc);
         dccg.emit(mod);
+      }
     }
     dccg.writeModules();
 
@@ -1143,8 +1221,10 @@ void codegenModules(Modules &modules) {
       global.params.link = false;
   }
 
-  cache::pruneCache();
+  {
+    TimeTraceScope timeScope("Prune object file cache");
+    cache::pruneCache();
+  }
 
   freeRuntime();
-  llvm::llvm_shutdown();
 }
