@@ -25,17 +25,17 @@
 #include "ir/irmodule.h"
 
 // returns the keytype typeinfo
-static LLConstant *to_keyti(DValue *aa, LLType *targetType) {
+static LLConstant *to_keyti(const Loc &loc, DValue *aa, LLType *targetType) {
   // keyti param
-  assert(aa->type->toBasetype()->ty == Taarray);
+  assert(aa->type->toBasetype()->ty == TY::Taarray);
   TypeAArray *aatype = static_cast<TypeAArray *>(aa->type->toBasetype());
-  LLConstant *ti = DtoTypeInfoOf(aatype->index, /*base=*/false);
+  LLConstant *ti = DtoTypeInfoOf(loc, aatype->index, /*base=*/false);
   return DtoBitCast(ti, targetType);
 }
 
 ////////////////////////////////////////////////////////////////////////////////
 
-DLValue *DtoAAIndex(Loc &loc, Type *type, DValue *aa, DValue *key,
+DLValue *DtoAAIndex(const Loc &loc, Type *type, DValue *aa, DValue *key,
                     bool lvalue) {
   // D2:
   // call:
@@ -61,13 +61,13 @@ DLValue *DtoAAIndex(Loc &loc, Type *type, DValue *aa, DValue *key,
   LLValue *ret;
   if (lvalue) {
     LLValue *rawAATI =
-        DtoTypeInfoOf(aa->type->unSharedOf()->mutableOf(), /*base=*/false);
+        DtoTypeInfoOf(loc, aa->type->unSharedOf()->mutableOf(), /*base=*/false);
     LLValue *castedAATI = DtoBitCast(rawAATI, funcTy->getParamType(1));
     LLValue *valsize = DtoConstSize_t(getTypeAllocSize(DtoType(type)));
     ret = gIR->CreateCallOrInvoke(func, aaval, castedAATI, valsize, pkey,
                                   "aa.index");
   } else {
-    LLValue *keyti = to_keyti(aa, funcTy->getParamType(1));
+    LLValue *keyti = to_keyti(loc, aa, funcTy->getParamType(1));
     ret = gIR->CreateCallOrInvoke(func, aaval, keyti, pkey, "aa.index");
   }
 
@@ -88,10 +88,8 @@ DLValue *DtoAAIndex(Loc &loc, Type *type, DValue *aa, DValue *key,
     gIR->ir->CreateCondBr(cond, okbb, failbb);
 
     // set up failbb to call the array bounds error runtime function
-
     gIR->ir->SetInsertPoint(failbb);
-
-    DtoBoundsCheckFailCall(gIR, loc);
+    emitRangeError(gIR, loc);
 
     // if ok, proceed in okbb
     gIR->ir->SetInsertPoint(okbb);
@@ -101,7 +99,7 @@ DLValue *DtoAAIndex(Loc &loc, Type *type, DValue *aa, DValue *key,
 
 ////////////////////////////////////////////////////////////////////////////////
 
-DValue *DtoAAIn(Loc &loc, Type *type, DValue *aa, DValue *key) {
+DValue *DtoAAIn(const Loc &loc, Type *type, DValue *aa, DValue *key) {
   // D1:
   // call:
   // extern(C) void* _aaIn(AA aa*, TypeInfo keyti, void* pkey)
@@ -125,7 +123,7 @@ DValue *DtoAAIn(Loc &loc, Type *type, DValue *aa, DValue *key) {
   aaval = DtoBitCast(aaval, funcTy->getParamType(0));
 
   // keyti param
-  LLValue *keyti = to_keyti(aa, funcTy->getParamType(1));
+  LLValue *keyti = to_keyti(loc, aa, funcTy->getParamType(1));
 
   // pkey param
   LLValue *pkey = makeLValue(loc, key);
@@ -145,7 +143,7 @@ DValue *DtoAAIn(Loc &loc, Type *type, DValue *aa, DValue *key) {
 
 ////////////////////////////////////////////////////////////////////////////////
 
-DValue *DtoAARemove(Loc &loc, DValue *aa, DValue *key) {
+DValue *DtoAARemove(const Loc &loc, DValue *aa, DValue *key) {
   // D1:
   // call:
   // extern(C) void _aaDel(AA aa, TypeInfo keyti, void* pkey)
@@ -169,7 +167,7 @@ DValue *DtoAARemove(Loc &loc, DValue *aa, DValue *key) {
   aaval = DtoBitCast(aaval, funcTy->getParamType(0));
 
   // keyti param
-  LLValue *keyti = to_keyti(aa, funcTy->getParamType(1));
+  LLValue *keyti = to_keyti(loc, aa, funcTy->getParamType(1));
 
   // pkey param
   LLValue *pkey = makeLValue(loc, key);
@@ -183,7 +181,7 @@ DValue *DtoAARemove(Loc &loc, DValue *aa, DValue *key) {
 
 ////////////////////////////////////////////////////////////////////////////////
 
-LLValue *DtoAAEquals(Loc &loc, TOK op, DValue *l, DValue *r) {
+LLValue *DtoAAEquals(const Loc &loc, TOK op, DValue *l, DValue *r) {
   Type *t = l->type->toBasetype();
   assert(t == r->type->toBasetype() &&
          "aa equality is only defined for aas of same type");
@@ -192,7 +190,7 @@ LLValue *DtoAAEquals(Loc &loc, TOK op, DValue *l, DValue *r) {
 
   LLValue *aaval = DtoBitCast(DtoRVal(l), funcTy->getParamType(1));
   LLValue *abval = DtoBitCast(DtoRVal(r), funcTy->getParamType(2));
-  LLValue *aaTypeInfo = DtoTypeInfoOf(t);
+  LLValue *aaTypeInfo = DtoTypeInfoOf(loc, t);
   LLValue *res =
       gIR->CreateCallOrInvoke(func, aaTypeInfo, aaval, abval, "aaEqRes");
 
