@@ -26,7 +26,7 @@ struct SPIRVTargetABI : TargetABI {
   }
   bool passByVal(TypeFunction *, Type *t) override {
     t = t->toBasetype();
-    return ((t->ty == Tsarray || t->ty == Tstruct) && t->size() > 64);
+    return ((t->ty == TY::Tsarray || t->ty == TY::Tstruct) && t->size() > 64);
   }
   bool reverseExplicitParams(TypeFunction *) override { return false; }
   void rewriteFunctionType(IrFuncTy &fty) override {
@@ -34,14 +34,24 @@ struct SPIRVTargetABI : TargetABI {
       if (!arg->byref)
         rewriteArgument(fty, *arg);
     }
+    if (!skipReturnValueRewrite(fty))
+      rewriteArgument(fty, *fty.ret);
   }
   bool returnInArg(TypeFunction *tf, bool) override {
-    return !tf->isref() && DtoIsInMemoryOnly(tf->next);
+    if (tf->isref())
+      return false;
+    Type *retty = tf->next->toBasetype();
+    if (retty->ty == TY::Tsarray)
+      return true;
+    else if (auto st = retty->isTypeStruct())
+      return !toDcomputePointer(st->sym);
+    else
+      return false;
   }
   void rewriteArgument(IrFuncTy &fty, IrFuncTyArg &arg) override {
     Type *ty = arg.type->toBasetype();
     llvm::Optional<DcomputePointer> ptr;
-    if (ty->ty == Tstruct &&
+    if (ty->ty == TY::Tstruct &&
         (ptr = toDcomputePointer(static_cast<TypeStruct *>(ty)->sym))) {
       pointerRewite.applyTo(arg);
     }
