@@ -7,6 +7,8 @@
 //
 //===----------------------------------------------------------------------===//
 
+#include "dmd/enum.h"
+#include "dmd/id.h"
 #include "gen/abi-generic.h"
 #include "gen/abi.h"
 #include "gen/dvalue.h"
@@ -40,17 +42,16 @@ struct X86TargetABI : TargetABI {
       return llvm::CallingConv::C;
 
     switch (l) {
-    case LINKc:
-    case LINKobjc:
+    case LINK::c:
+    case LINK::objc:
       return llvm::CallingConv::C;
-    case LINKcpp:
+    case LINK::cpp:
       return isMSVC && fdecl && fdecl->needThis()
                  ? llvm::CallingConv::X86_ThisCall
                  : llvm::CallingConv::C;
-    case LINKd:
-    case LINKdefault:
-    case LINKpascal:
-    case LINKwindows:
+    case LINK::d:
+    case LINK::default_:
+    case LINK::windows:
       return llvm::CallingConv::X86_StdCall;
     default:
       llvm_unreachable("Unhandled D linkage type.");
@@ -59,11 +60,11 @@ struct X86TargetABI : TargetABI {
 
   std::string mangleFunctionForLLVM(std::string name, LINK l) override {
     if (global.params.targetTriple->isOSWindows()) {
-      if (l == LINKd || l == LINKdefault) {
+      if (l == LINK::d || l == LINK::default_) {
         // Prepend a 0x1 byte to prevent LLVM from applying MS stdcall mangling:
         // _D… => __D…@<paramssize>, and add extra underscore manually.
         name.insert(0, "\1_");
-      } else if (l == LINKcpp && name[0] == '?') {
+      } else if (l == LINK::cpp && name[0] == '?') {
         // Prepend a 0x1 byte to prevent LLVM from prepending the C underscore
         // for MSVC++ symbols (starting with '?').
         name.insert(0, "\1");
@@ -73,7 +74,7 @@ struct X86TargetABI : TargetABI {
   }
 
   std::string mangleVariableForLLVM(std::string name, LINK l) override {
-    if (global.params.targetTriple->isOSWindows() && l == LINKcpp &&
+    if (global.params.targetTriple->isOSWindows() && l == LINK::cpp &&
         name[0] == '?') {
       // Prepend a 0x1 byte to prevent LLVM from prepending the C underscore for
       // MSVC++ symbols (starting with '?').
@@ -82,11 +83,27 @@ struct X86TargetABI : TargetABI {
     return name;
   }
 
+  // Helper folding the magic __c_complex_{float,double,real} enums to the basic
+  // complex type.
+  static Type *getExtraLoweredReturnType(TypeFunction *tf) {
+    Type *rt = tf->next;
+    if (auto te = rt->isTypeEnum()) {
+      auto id = te->sym->ident;
+      if (id == Id::__c_complex_float)
+        return Type::tcomplex32;
+      if (id == Id::__c_complex_double)
+        return Type::tcomplex64;
+      if (id == Id::__c_complex_real)
+        return Type::tcomplex80;
+    }
+    return rt->toBasetype();
+  }
+
   bool returnInArg(TypeFunction *tf, bool needsThis) override {
     if (tf->isref())
       return false;
 
-    Type *rt = tf->next->toBasetype();
+    Type *rt = getExtraLoweredReturnType(tf);
     const bool externD = isExternD(tf);
 
     // non-aggregates are returned directly
@@ -101,17 +118,17 @@ struct X86TargetABI : TargetABI {
       // extern(C) and all others:
       // * cfloat will be rewritten as 64-bit integer and returned in registers
       // * sret for cdouble and creal
-      return rt->ty != Tcomplex32;
+      return rt->ty != TY::Tcomplex32;
     }
 
     // non-extern(D): some OSs don't return structs in registers at all
     if (!externD && !returnStructsInRegs)
       return true;
 
-    const bool isMSVCpp = isMSVC && tf->linkage == LINKcpp;
+    const bool isMSVCpp = isMSVC && tf->linkage == LINK::cpp;
 
     // for non-static member functions, MSVC++ enforces sret for all structs
-    if (isMSVCpp && needsThis && rt->ty == Tstruct) {
+    if (isMSVCpp && needsThis && rt->ty == TY::Tstruct) {
       return true;
     }
 
@@ -126,8 +143,9 @@ struct X86TargetABI : TargetABI {
   }
 
   bool passByVal(TypeFunction *tf, Type *t) override {
-    // indirectly by-value for non-POD args on Posix
-    if (!isMSVC && !isPOD(t))
+    // indirectly by-value for non-POD args (except for MSVC++)
+    const bool isMSVCpp = isMSVC && tf->linkage == LINK::cpp;
+    if (!isMSVCpp && !isPOD(t))
       return false;
 
     // pass all structs and static arrays with the LLVM byval attribute
@@ -138,17 +156,18 @@ struct X86TargetABI : TargetABI {
     const bool externD = isExternD(fty.type);
 
     // return value:
-    if (!fty.ret->byref) {
-      Type *rt = fty.ret->type->toBasetype(); // for sret, rt == void
+    if (!skipReturnValueRewrite(fty)) {
+      Type *rt = getExtraLoweredReturnType(fty.type);
       if (isAggregate(rt) && canRewriteAsInt(rt) &&
           // don't rewrite cfloat for extern(D)
-          !(externD && rt->ty == Tcomplex32)) {
+          !(externD && rt->ty == TY::Tcomplex32)) {
         integerRewrite.applyToIfNotObsolete(*fty.ret);
       }
     }
 
-    // Posix: non-POD args are passed indirectly by-value
-    if (!isMSVC) {
+    // non-POD args are passed indirectly by-value (except for MSVC++)
+    const bool isMSVCpp = isMSVC && fty.type->linkage == LINK::cpp;
+    if (!isMSVCpp) {
       for (auto arg : fty.args) {
         if (!arg->byref && !isPOD(arg->type))
           indirectByvalRewrite.applyTo(*arg);
@@ -182,22 +201,23 @@ struct X86TargetABI : TargetABI {
         //   * It is not a floating point type.
 
         IrFuncTyArg *last = fty.args.back();
-        Type *lastTy = last->type->toBasetype();
-        unsigned sz = lastTy->size();
-
         if (last->rewrite == &indirectByvalRewrite ||
             (last->byref && !last->isByVal())) {
           Logger::println("Putting last (byref) parameter in register");
           last->attrs.addAttribute(LLAttribute::InReg);
-        } else if (!lastTy->isfloating() && (sz == 1 || sz == 2 || sz == 4)) {
-          // rewrite aggregates as integers to make inreg work
-          if (lastTy->ty == Tstruct || lastTy->ty == Tsarray) {
-            integerRewrite.applyTo(*last);
-            // undo byval semantics applied via passByVal() returning true
-            last->byref = false;
-            last->attrs.clear();
+        } else {
+          Type *lastTy = last->type->toBasetype();
+          auto sz = lastTy->size();
+          if (!lastTy->isfloating() && (sz == 1 || sz == 2 || sz == 4)) {
+            // rewrite aggregates as integers to make inreg work
+            if (lastTy->ty == TY::Tstruct || lastTy->ty == TY::Tsarray) {
+              integerRewrite.applyTo(*last);
+              // undo byval semantics applied via passByVal() returning true
+              last->byref = false;
+              last->attrs.clear();
+            }
+            last->attrs.addAttribute(LLAttribute::InReg);
           }
-          last->attrs.addAttribute(LLAttribute::InReg);
         }
       }
     }
@@ -213,7 +233,7 @@ struct X86TargetABI : TargetABI {
     size_t i = 0;
     while (i < fty.args.size()) {
       Type *type = fty.args[i]->type->toBasetype();
-      if (type->ty == Tstruct) {
+      if (type->ty == TY::Tstruct) {
         // Do not pass empty structs at all for C++ ABI compatibility.
         // Tests with clang reveal that more complex "empty" types, for
         // example a struct containing an empty struct, are not
