@@ -68,13 +68,24 @@ bool TargetABI::isHFVA(Type *t, int maxNumElements, LLType **hfvaType) {
   return false;
 }
 
+bool TargetABI::isHVA(Type *t, int maxNumElements, LLType **hvaType) {
+  Type *rewriteType = nullptr;
+  if (::isHFVA(t, maxNumElements, &rewriteType) &&
+      rewriteType->nextOf()->ty == TY::Tvector) {
+    if (hvaType)
+      *hvaType = DtoType(rewriteType);
+    return true;
+  }
+  return false;
+}
+
 //////////////////////////////////////////////////////////////////////////////
 
 TypeTuple *TargetABI::getArgTypes(Type *t) {
   // try to reuse cached argTypes of StructDeclarations
   if (auto ts = t->toBasetype()->isTypeStruct()) {
     auto sd = ts->sym;
-    if (sd->sizeok == SIZEOKdone)
+    if (sd->sizeok == Sizeok::done)
       return sd->argTypes;
   }
 
@@ -107,8 +118,8 @@ bool TargetABI::isAggregate(Type *t) {
   // FIXME: dynamic arrays can currently not be rewritten as they are used
   //        by runtime functions, for which we don't apply the rewrites yet
   //        when calling them
-  return ty == Tstruct || ty == Tsarray ||
-         /*ty == Tarray ||*/ ty == Tdelegate || t->iscomplex();
+  return ty == TY::Tstruct || ty == TY::Tsarray ||
+         /*ty == TY::Tarray ||*/ ty == TY::Tdelegate || t->iscomplex();
 }
 
 namespace {
@@ -128,7 +139,7 @@ bool hasCtor(StructDeclaration *s) {
 
 bool TargetABI::isPOD(Type *t, bool excludeStructsWithCtor) {
   t = t->baseElemOf();
-  if (t->ty != Tstruct)
+  if (t->ty != TY::Tstruct)
     return true;
   StructDeclaration *sd = static_cast<TypeStruct *>(t)->sym;
   return sd->isPOD() && !(excludeStructsWithCtor && hasCtor(sd));
@@ -140,7 +151,22 @@ bool TargetABI::canRewriteAsInt(Type *t, bool include64bit) {
 }
 
 bool TargetABI::isExternD(TypeFunction *tf) {
-  return tf->linkage == LINKd && tf->parameterList.varargs != VARARGvariadic;
+  return tf->linkage == LINK::d && tf->parameterList.varargs != VARARGvariadic;
+}
+
+bool TargetABI::skipReturnValueRewrite(IrFuncTy &fty) {
+  if (fty.ret->byref)
+    return true;
+
+  auto ty = fty.ret->type->toBasetype()->ty;
+  return ty == TY::Tvoid || ty == TY::Tnoreturn;
+}
+
+//////////////////////////////////////////////////////////////////////////////
+
+bool TargetABI::preferPassByRef(Type *t) {
+  // simple base heuristic: use a ref for all types > 2 machine words
+  return t->size() > 2 * target.ptrsize;
 }
 
 //////////////////////////////////////////////////////////////////////////////
@@ -241,7 +267,7 @@ TargetABI *TargetABI::getTarget() {
   case llvm::Triple::mipsel:
   case llvm::Triple::mips64:
   case llvm::Triple::mips64el:
-    return getMIPS64TargetABI(global.params.is64bit);
+    return getMIPS64TargetABI(global.params.targetTriple->isArch64Bit());
   case llvm::Triple::ppc:
   case llvm::Triple::ppc64:
     return getPPCTargetABI(global.params.targetTriple->isArch64Bit());
@@ -275,7 +301,7 @@ struct IntrinsicABI : TargetABI {
 
   void rewriteArgument(IrFuncTy &fty, IrFuncTyArg &arg) override {
     Type *ty = arg.type->toBasetype();
-    if (ty->ty != Tstruct) {
+    if (ty->ty != TY::Tstruct) {
       return;
     }
     // TODO: Check that no unions are passed in or returned.
@@ -290,7 +316,7 @@ struct IntrinsicABI : TargetABI {
   void rewriteFunctionType(IrFuncTy &fty) override {
     if (!fty.arg_sret) {
       Type *rt = fty.ret->type->toBasetype();
-      if (rt->ty == Tstruct) {
+      if (rt->ty == TY::Tstruct) {
         Logger::println("Intrinsic ABI: Transforming return type");
         rewriteArgument(fty, *fty.ret);
       }
