@@ -42,7 +42,7 @@ LLValue *DtoSlice(LLValue *ptr, LLValue *length, LLType *elemType = nullptr) {
 
 LLValue *DtoSlice(Expression *e) {
   DValue *dval = toElem(e);
-  if (dval->type->toBasetype()->ty == Tsarray) {
+  if (dval->type->toBasetype()->ty == TY::Tsarray) {
     // Convert static array to slice
     return DtoSlice(DtoLVal(dval), DtoArrayLen(dval));
   }
@@ -57,11 +57,11 @@ static LLValue *DtoSlicePtr(Expression *e) {
   Loc loc;
   LLStructType *type = DtoArrayType(LLType::getInt8Ty(gIR->context()));
   Type *vt = dval->type->toBasetype();
-  if (vt->ty == Tarray) {
+  if (vt->ty == TY::Tarray) {
     return makeLValue(loc, dval);
   }
 
-  bool isStaticArray = vt->ty == Tsarray;
+  bool isStaticArray = vt->ty == TY::Tsarray;
   LLValue *val = isStaticArray ? DtoLVal(dval) : makeLValue(loc, dval);
   LLValue *array = DtoRawAlloca(type, 0, ".array");
   LLValue *len = isStaticArray ? DtoArrayLen(dval) : DtoConstSize_t(1);
@@ -87,7 +87,7 @@ LLStructType *DtoArrayType(LLType *t) {
 
 LLArrayType *DtoStaticArrayType(Type *t) {
   t = t->toBasetype();
-  assert(t->ty == Tsarray);
+  assert(t->ty == TY::Tsarray);
   TypeSArray *tsa = static_cast<TypeSArray *>(t);
   Type *tnext = tsa->nextOf();
 
@@ -100,15 +100,12 @@ void DtoSetArrayToNull(LLValue *v) {
   IF_LOG Logger::println("DtoSetArrayToNull");
   LOG_SCOPE;
 
-  assert(isaPointer(v));
-  LLType *t = v->getType()->getContainedType(0);
-
-  DtoStore(LLConstant::getNullValue(t), v);
+  DtoStore(LLConstant::getNullValue(getPointeeType(v)), v);
 }
 
 ////////////////////////////////////////////////////////////////////////////////
 
-static void DtoArrayInit(Loc &loc, LLValue *ptr, LLValue *length,
+static void DtoArrayInit(const Loc &loc, LLValue *ptr, LLValue *length,
                          DValue *elementValue) {
   IF_LOG Logger::println("DtoArrayInit");
   LOG_SCOPE;
@@ -179,7 +176,7 @@ static void DtoArrayInit(Loc &loc, LLValue *ptr, LLValue *length,
 static Type *DtoArrayElementType(Type *arrayType) {
   assert(arrayType->toBasetype()->nextOf());
   Type *t = arrayType->toBasetype()->nextOf()->toBasetype();
-  while (t->ty == Tsarray) {
+  while (t->ty == TY::Tsarray) {
     t = t->nextOf()->toBasetype();
   }
   return t;
@@ -193,8 +190,9 @@ static LLValue *computeSize(LLValue *length, size_t elementSize) {
              : gIR->ir->CreateMul(length, DtoConstSize_t(elementSize));
 };
 
-static void copySlice(Loc &loc, LLValue *dstarr, LLValue *dstlen, LLValue *srcarr,
-                      LLValue *srclen, size_t elementSize, bool knownInBounds) {
+static void copySlice(const Loc &loc, LLValue *dstarr, LLValue *dstlen,
+                      LLValue *srcarr, LLValue *srclen, size_t elementSize,
+                      bool knownInBounds) {
   const bool checksEnabled =
       global.params.useAssert == CHECKENABLEon || gIR->emitArrayBoundsChecks();
   if (checksEnabled && !knownInBounds) {
@@ -216,7 +214,7 @@ static void copySlice(Loc &loc, LLValue *dstarr, LLValue *dstlen, LLValue *srcar
 // Determine whether t is an array of structs that need a postblit.
 static bool arrayNeedsPostblit(Type *t) {
   t = DtoArrayElementType(t);
-  if (t->ty == Tstruct) {
+  if (t->ty == TY::Tstruct) {
     return static_cast<TypeStruct *>(t)->sym->postblit != nullptr;
   }
   return false;
@@ -224,7 +222,7 @@ static bool arrayNeedsPostblit(Type *t) {
 
 // Does array assignment (or initialization) from another array of the same
 // element type or from an appropriate single element.
-void DtoArrayAssign(Loc &loc, DValue *lhs, DValue *rhs, int op,
+void DtoArrayAssign(const Loc &loc, DValue *lhs, DValue *rhs, int op,
                     bool canSkipPostblit) {
   IF_LOG Logger::println("DtoArrayAssign");
   LOG_SCOPE;
@@ -234,8 +232,8 @@ void DtoArrayAssign(Loc &loc, DValue *lhs, DValue *rhs, int op,
   assert(t->nextOf());
 
   // reference assignment for dynamic array?
-  if (t->ty == Tarray && !lhs->isSlice()) {
-    assert(t2->ty == Tarray || t2->ty == Tsarray);
+  if (t->ty == TY::Tarray && !lhs->isSlice()) {
+    assert(t2->ty == TY::Tarray || t2->ty == TY::Tsarray);
     if (rhs->isNull()) {
       DtoSetArrayToNull(DtoLVal(lhs));
     } else {
@@ -258,8 +256,9 @@ void DtoArrayAssign(Loc &loc, DValue *lhs, DValue *rhs, int op,
   // Be careful to handle void arrays correctly when modifying this (see tests
   // for DMD issue 7493).
   // TODO: This should use AssignExp::memset.
-  LLValue *realRhsArrayPtr =
-      (t2->ty == Tarray || t2->ty == Tsarray) ? DtoArrayPtr(rhs) : nullptr;
+  LLValue *realRhsArrayPtr = (t2->ty == TY::Tarray || t2->ty == TY::Tsarray)
+                                 ? DtoArrayPtr(rhs)
+                                 : nullptr;
   if (realRhsArrayPtr && realRhsArrayPtr->getType() == realLhsPtr->getType()) {
     // T[]  = T[]      T[]  = T[n]
     // T[n] = T[n]     T[n] = T[]
@@ -267,7 +266,7 @@ void DtoArrayAssign(Loc &loc, DValue *lhs, DValue *rhs, int op,
     LLValue *rhsLength = DtoArrayLen(rhs);
 
     const bool needsPostblit = (op != TOKblit && arrayNeedsPostblit(t) &&
-                                (!canSkipPostblit || t2->ty == Tarray));
+                                (!canSkipPostblit || t2->ty == TY::Tarray));
 
     if (!needsDestruction && !needsPostblit) {
       // fast version
@@ -277,7 +276,7 @@ void DtoArrayAssign(Loc &loc, DValue *lhs, DValue *rhs, int op,
         DtoMemSetZero(lhsPtr, lhsSize);
       } else {
         bool knownInBounds =
-            isConstructing || (t->ty == Tsarray && t2->ty == Tsarray);
+            isConstructing || (t->ty == TY::Tsarray && t2->ty == TY::Tsarray);
         if (!knownInBounds) {
           if (auto constLhsLength = llvm::dyn_cast<LLConstantInt>(lhsLength)) {
             if (auto constRhsLength =
@@ -293,17 +292,16 @@ void DtoArrayAssign(Loc &loc, DValue *lhs, DValue *rhs, int op,
       }
     } else if (isConstructing) {
       LLFunction *fn = getRuntimeFunction(loc, gIR->module, "_d_arrayctor");
-      gIR->CreateCallOrInvoke(fn, DtoTypeInfoOf(elemType),
+      gIR->CreateCallOrInvoke(fn, DtoTypeInfoOf(loc, elemType),
                               DtoSlice(rhsPtr, rhsLength),
                               DtoSlice(lhsPtr, lhsLength));
-    } else // assigning
-    {
+    } else { // assigning
       LLValue *tmpSwap = DtoAlloca(elemType, "arrayAssign.tmpSwap");
       LLFunction *fn = getRuntimeFunction(
           loc, gIR->module,
           !canSkipPostblit ? "_d_arrayassign_l" : "_d_arrayassign_r");
       gIR->CreateCallOrInvoke(
-          fn, DtoTypeInfoOf(elemType), DtoSlice(rhsPtr, rhsLength),
+          fn, DtoTypeInfoOf(loc, elemType), DtoSlice(rhsPtr, rhsLength),
           DtoSlice(lhsPtr, lhsLength), DtoBitCast(tmpSwap, getVoidPtrType()));
     }
   } else {
@@ -337,7 +335,7 @@ void DtoArrayAssign(Loc &loc, DValue *lhs, DValue *rhs, int op,
           fn, lhsPtr, DtoBitCast(makeLValue(loc, rhs), getVoidPtrType()),
           gIR->ir->CreateTruncOrBitCast(lhsLength,
                                         LLType::getInt32Ty(gIR->context())),
-          DtoTypeInfoOf(stripModifiers(t2)));
+          DtoTypeInfoOf(loc, stripModifiers(t2)));
     }
   }
 }
@@ -368,7 +366,7 @@ LLConstant *DtoConstArrayInitializer(ArrayInitializer *arrinit,
 
   // for statis arrays, dmd does not include any trailing default
   // initialized elements in the value/index lists
-  if (arrty->ty == Tsarray) {
+  if (arrty->ty == TY::Tsarray) {
     TypeSArray *tsa = static_cast<TypeSArray *>(arrty);
     arrlen = static_cast<size_t>(tsa->dim->toInteger());
   }
@@ -383,7 +381,7 @@ LLConstant *DtoConstArrayInitializer(ArrayInitializer *arrinit,
 
   // get elem type
   Type *elemty;
-  if (arrty->ty == Tvector) {
+  if (arrty->ty == TY::Tvector) {
     elemty = static_cast<TypeVector *>(arrty)->elementType();
   } else {
     elemty = arrty->nextOf();
@@ -459,7 +457,7 @@ LLConstant *DtoConstArrayInitializer(ArrayInitializer *arrinit,
     constarr = LLConstantStruct::getAnon(gIR->context(),
                                          initvals); // FIXME should this pack?
   } else {
-    if (arrty->ty == Tvector) {
+    if (arrty->ty == TY::Tvector) {
       constarr = llvm::ConstantVector::get(initvals);
     } else {
       constarr =
@@ -470,7 +468,7 @@ LLConstant *DtoConstArrayInitializer(ArrayInitializer *arrinit,
   //     std::cout << "constarr: " << *constarr << std::endl;
 
   // if the type is a static array, we're done
-  if (arrty->ty == Tsarray || arrty->ty == Tvector) {
+  if (arrty->ty == TY::Tsarray || arrty->ty == TY::Tvector) {
     return constarr;
   }
 
@@ -482,15 +480,12 @@ LLConstant *DtoConstArrayInitializer(ArrayInitializer *arrinit,
                                    LLGlobalValue::InternalLinkage, constarr,
                                    ".constarray");
 
-  if (arrty->ty == Tpointer) {
+  if (arrty->ty == TY::Tpointer) {
     // we need to return pointer to the static array.
     return DtoBitCast(gvar, DtoType(arrty));
   }
 
-  LLConstant *idxs[2] = {DtoConstUint(0), DtoConstUint(0)};
-
-  LLConstant *gep = llvm::ConstantExpr::getGetElementPtr(
-      isaPointer(gvar)->getElementType(), gvar, idxs, true);
+  LLConstant *gep = DtoGEP(gvar, 0u, 0u);
   gep = llvm::ConstantExpr::getBitCast(gvar, getPtrToType(llelemty));
 
   return DtoConstSlice(DtoConstSize_t(arrlen), gep, arrty);
@@ -524,7 +519,7 @@ bool isConstLiteral(Expression *e, bool immutableType) {
       // If dynamic array: assume not constant because the array is expected to
       // be newly allocated. See GH 1924.
       Type *arrayType = ale->type->toBasetype();
-      if (arrayType->ty == Tarray)
+      if (arrayType->ty == TY::Tarray)
         return false;
     }
 
@@ -675,7 +670,7 @@ static DSliceValue *getSlice(Type *arrayType, LLValue *array) {
 }
 
 ////////////////////////////////////////////////////////////////////////////////
-DSliceValue *DtoNewDynArray(Loc &loc, Type *arrayType, DValue *dim,
+DSliceValue *DtoNewDynArray(const Loc &loc, Type *arrayType, DValue *dim,
                             bool defaultInit) {
   IF_LOG Logger::println("DtoNewDynArray : %s", arrayType->toChars());
   LOG_SCOPE;
@@ -693,7 +688,7 @@ DSliceValue *DtoNewDynArray(Loc &loc, Type *arrayType, DValue *dim,
   LLFunction *fn = getRuntimeFunction(loc, gIR->module, fnname);
 
   // typeinfo arg
-  LLValue *arrayTypeInfo = DtoTypeInfoOf(arrayType);
+  LLValue *arrayTypeInfo = DtoTypeInfoOf(loc, arrayType);
 
   // dim arg
   assert(DtoType(dim->type) == DtoSize_t());
@@ -710,8 +705,8 @@ DSliceValue *DtoNewDynArray(Loc &loc, Type *arrayType, DValue *dim,
 }
 
 ////////////////////////////////////////////////////////////////////////////////
-DSliceValue *DtoNewMulDimDynArray(Loc &loc, Type *arrayType, DValue **dims,
-                                  size_t ndims) {
+DSliceValue *DtoNewMulDimDynArray(const Loc &loc, Type *arrayType,
+                                  DValue **dims, size_t ndims) {
   IF_LOG Logger::println("DtoNewMulDimDynArray : %s", arrayType->toChars());
   LOG_SCOPE;
 
@@ -727,7 +722,7 @@ DSliceValue *DtoNewMulDimDynArray(Loc &loc, Type *arrayType, DValue **dims,
   LLFunction *fn = getRuntimeFunction(loc, gIR->module, fnname);
 
   // typeinfo arg
-  LLValue *arrayTypeInfo = DtoTypeInfoOf(arrayType);
+  LLValue *arrayTypeInfo = DtoTypeInfoOf(loc, arrayType);
 
   // Check if constant
   bool allDimsConst = true;
@@ -778,7 +773,7 @@ DSliceValue *DtoNewMulDimDynArray(Loc &loc, Type *arrayType, DValue **dims,
 }
 
 ////////////////////////////////////////////////////////////////////////////////
-DSliceValue *DtoResizeDynArray(Loc &loc, Type *arrayType, DValue *array,
+DSliceValue *DtoResizeDynArray(const Loc &loc, Type *arrayType, DValue *array,
                                LLValue *newdim) {
   IF_LOG Logger::println("DtoResizeDynArray : %s", arrayType->toChars());
   LOG_SCOPE;
@@ -786,7 +781,7 @@ DSliceValue *DtoResizeDynArray(Loc &loc, Type *arrayType, DValue *array,
   assert(array);
   assert(newdim);
   assert(arrayType);
-  assert(arrayType->toBasetype()->ty == Tarray);
+  assert(arrayType->toBasetype()->ty == TY::Tarray);
 
   // decide on what runtime function to call based on whether the type is zero
   // initialized
@@ -798,7 +793,7 @@ DSliceValue *DtoResizeDynArray(Loc &loc, Type *arrayType, DValue *array,
                                                     : "_d_arraysetlengthiT");
 
   LLValue *newArray = gIR->CreateCallOrInvoke(
-      fn, DtoTypeInfoOf(arrayType), newdim,
+      fn, DtoTypeInfoOf(loc, arrayType), newdim,
       DtoBitCast(DtoLVal(array), fn->getFunctionType()->getParamType(2)),
       ".gc_mem");
 
@@ -807,7 +802,7 @@ DSliceValue *DtoResizeDynArray(Loc &loc, Type *arrayType, DValue *array,
 
 ////////////////////////////////////////////////////////////////////////////////
 
-void DtoCatAssignElement(Loc &loc, DValue *array, Expression *exp) {
+void DtoCatAssignElement(const Loc &loc, DValue *array, Expression *exp) {
   IF_LOG Logger::println("DtoCatAssignElement");
   LOG_SCOPE;
 
@@ -822,7 +817,7 @@ void DtoCatAssignElement(Loc &loc, DValue *array, Expression *exp) {
   // potentially moved to a new block).
   LLFunction *fn = getRuntimeFunction(loc, gIR->module, "_d_arrayappendcTX");
   gIR->CreateCallOrInvoke(
-      fn, DtoTypeInfoOf(arrayType),
+      fn, DtoTypeInfoOf(loc, arrayType),
       DtoBitCast(DtoLVal(array), fn->getFunctionType()->getParamType(1)),
       DtoConstSize_t(1), ".appendedArray");
 
@@ -839,7 +834,7 @@ void DtoCatAssignElement(Loc &loc, DValue *array, Expression *exp) {
 
 ////////////////////////////////////////////////////////////////////////////////
 
-DSliceValue *DtoCatAssignArray(Loc &loc, DValue *arr, Expression *exp) {
+DSliceValue *DtoCatAssignArray(const Loc &loc, DValue *arr, Expression *exp) {
   IF_LOG Logger::println("DtoCatAssignArray");
   LOG_SCOPE;
   Type *arrayType = arr->type;
@@ -847,7 +842,7 @@ DSliceValue *DtoCatAssignArray(Loc &loc, DValue *arr, Expression *exp) {
   LLFunction *fn = getRuntimeFunction(loc, gIR->module, "_d_arrayappendT");
   // Call _d_arrayappendT(TypeInfo ti, byte[] *px, byte[] y)
   LLValue *newArray = gIR->CreateCallOrInvoke(
-      fn, DtoTypeInfoOf(arrayType),
+      fn, DtoTypeInfoOf(loc, arrayType),
       DtoBitCast(DtoLVal(arr), fn->getFunctionType()->getParamType(1)),
       DtoAggrPaint(DtoSlice(exp), fn->getFunctionType()->getParamType(2)),
       ".appendedArray");
@@ -857,7 +852,7 @@ DSliceValue *DtoCatAssignArray(Loc &loc, DValue *arr, Expression *exp) {
 
 ////////////////////////////////////////////////////////////////////////////////
 
-DSliceValue *DtoCatArrays(Loc &loc, Type *arrayType, Expression *exp1,
+DSliceValue *DtoCatArrays(const Loc &loc, Type *arrayType, Expression *exp1,
                           Expression *exp2) {
   IF_LOG Logger::println("DtoCatAssignArray");
   LOG_SCOPE;
@@ -879,7 +874,7 @@ DSliceValue *DtoCatArrays(Loc &loc, Type *arrayType, Expression *exp1,
     arrs.push_back(DtoSlicePtr(ce));
 
     // Create static array from slices
-    LLPointerType *ptrarraytype = isaPointer(arrs[0]->getType());
+    LLPointerType *ptrarraytype = isaPointer(arrs[0]);
     assert(ptrarraytype && "Expected pointer type");
     LLStructType *arraytype = isaStruct(ptrarraytype->getElementType());
     assert(arraytype && "Expected struct type");
@@ -901,14 +896,14 @@ DSliceValue *DtoCatArrays(Loc &loc, Type *arrayType, Expression *exp1,
                                        LLType::getInt8Ty(gIR->context()))))));
 
     // TypeInfo ti
-    args.push_back(DtoTypeInfoOf(arrayType));
+    args.push_back(DtoTypeInfoOf(loc, arrayType));
     // byte[][] arrs
     args.push_back(val);
   } else {
     fn = getRuntimeFunction(loc, gIR->module, "_d_arraycatT");
 
     // TypeInfo ti
-    args.push_back(DtoTypeInfoOf(arrayType));
+    args.push_back(DtoTypeInfoOf(loc, arrayType));
     // byte[] x
     LLValue *val = DtoLoad(DtoSlicePtr(exp1));
     val = DtoAggrPaint(val, fn->getFunctionType()->getParamType(1));
@@ -925,7 +920,7 @@ DSliceValue *DtoCatArrays(Loc &loc, Type *arrayType, Expression *exp1,
 
 ////////////////////////////////////////////////////////////////////////////////
 
-DSliceValue *DtoAppendDChar(Loc &loc, DValue *arr, Expression *exp,
+DSliceValue *DtoAppendDChar(const Loc &loc, DValue *arr, Expression *exp,
                             const char *func) {
   LLValue *valueToAppend = DtoRVal(exp);
 
@@ -943,7 +938,8 @@ DSliceValue *DtoAppendDChar(Loc &loc, DValue *arr, Expression *exp,
 
 ////////////////////////////////////////////////////////////////////////////////
 
-DSliceValue *DtoAppendDCharToString(Loc &loc, DValue *arr, Expression *exp) {
+DSliceValue *DtoAppendDCharToString(const Loc &loc, DValue *arr,
+                                    Expression *exp) {
   IF_LOG Logger::println("DtoAppendDCharToString");
   LOG_SCOPE;
   return DtoAppendDChar(loc, arr, exp, "_d_arrayappendcd");
@@ -951,7 +947,7 @@ DSliceValue *DtoAppendDCharToString(Loc &loc, DValue *arr, Expression *exp) {
 
 ////////////////////////////////////////////////////////////////////////////////
 
-DSliceValue *DtoAppendDCharToUnicodeString(Loc &loc, DValue *arr,
+DSliceValue *DtoAppendDCharToUnicodeString(const Loc &loc, DValue *arr,
                                            Expression *exp) {
   IF_LOG Logger::println("DtoAppendDCharToUnicodeString");
   LOG_SCOPE;
@@ -961,8 +957,8 @@ DSliceValue *DtoAppendDCharToUnicodeString(Loc &loc, DValue *arr,
 ////////////////////////////////////////////////////////////////////////////////
 namespace {
 // helper for eq and cmp
-LLValue *DtoArrayEqCmp_impl(Loc &loc, const char *func, DValue *l,
-                                   DValue *r, bool useti) {
+LLValue *DtoArrayEqCmp_impl(const Loc &loc, const char *func, DValue *l,
+                            DValue *r, bool useti) {
   IF_LOG Logger::println("comparing arrays");
   LLFunction *fn = getRuntimeFunction(loc, gIR->module, func);
   assert(fn);
@@ -985,7 +981,7 @@ LLValue *DtoArrayEqCmp_impl(Loc &loc, const char *func, DValue *l,
 
   // pass array typeinfo ?
   if (useti) {
-    LLValue *tival = DtoTypeInfoOf(l->type);
+    LLValue *tival = DtoTypeInfoOf(loc, l->type);
     args.push_back(DtoBitCast(tival, fn->getFunctionType()->getParamType(2)));
   }
 
@@ -996,40 +992,41 @@ LLValue *DtoArrayEqCmp_impl(Loc &loc, const char *func, DValue *l,
 /// See `validCompareWithMemcmp`.
 bool validCompareWithMemcmpType(Type *t) {
   switch (t->ty) {
-  case Tsarray: {
+  case TY::Tsarray: {
     auto *elemType = t->baseElemOf();
     return validCompareWithMemcmpType(elemType);
   }
 
-  case Tstruct:
+  case TY::Tstruct:
     // TODO: Implement when structs can be compared with memcmp. Remember that
     // structs can have a user-defined opEquals, alignment padding bytes (in
     // arrays), and padding bytes.
     return false;
 
-  case Tvoid:
-  case Tint8:
-  case Tuns8:
-  case Tint16:
-  case Tuns16:
-  case Tint32:
-  case Tuns32:
-  case Tint64:
-  case Tuns64:
-  case Tint128:
-  case Tuns128:
-  case Tbool:
-  case Tchar:
-  case Twchar:
-  case Tdchar:
-  case Tpointer:
+  case TY::Tvoid:
+  case TY::Tint8:
+  case TY::Tuns8:
+  case TY::Tint16:
+  case TY::Tuns16:
+  case TY::Tint32:
+  case TY::Tuns32:
+  case TY::Tint64:
+  case TY::Tuns64:
+  case TY::Tint128:
+  case TY::Tuns128:
+  case TY::Tbool:
+  case TY::Tchar:
+  case TY::Twchar:
+  case TY::Tdchar:
+  case TY::Tpointer:
     return true;
 
     // TODO: Determine whether this can be "return true" too:
-    // case Tvector:
-  }
+    // case TY::Tvector:
 
-  return false;
+  default:
+    return false;
+  }
 }
 
 /// When `true` is returned, `l` and `r` can be compared using `memcmp`.
@@ -1054,7 +1051,7 @@ bool validCompareWithMemcmp(DValue *l, DValue *r) {
 }
 
 // Create a call instruction to memcmp.
-llvm::CallInst *callMemcmp(Loc &loc, IRState &irs, LLValue *l_ptr,
+llvm::CallInst *callMemcmp(const Loc &loc, IRState &irs, LLValue *l_ptr,
                            LLValue *r_ptr, LLValue *numElements) {
   assert(l_ptr && r_ptr && numElements);
   LLFunction *fn = getRuntimeFunction(loc, gIR->module, "memcmp");
@@ -1076,7 +1073,8 @@ llvm::CallInst *callMemcmp(Loc &loc, IRState &irs, LLValue *l_ptr,
 /// with memcmp.
 ///
 /// Note: the dynamic array length check is not covered by (LDC's) PGO.
-LLValue *DtoArrayEqCmp_memcmp(Loc &loc, DValue *l, DValue *r, IRState &irs) {
+LLValue *DtoArrayEqCmp_memcmp(const Loc &loc, DValue *l, DValue *r,
+                              IRState &irs) {
   IF_LOG Logger::println("Comparing arrays using memcmp");
 
   auto *l_ptr = DtoArrayPtr(l);
@@ -1084,8 +1082,9 @@ LLValue *DtoArrayEqCmp_memcmp(Loc &loc, DValue *l, DValue *r, IRState &irs) {
   auto *l_length = DtoArrayLen(l);
 
   // Early return for the simple case of comparing two static arrays.
-  const bool staticArrayComparison = (l->type->toBasetype()->ty == Tsarray) &&
-                                     (r->type->toBasetype()->ty == Tsarray);
+  const bool staticArrayComparison =
+      (l->type->toBasetype()->ty == TY::Tsarray) &&
+      (r->type->toBasetype()->ty == TY::Tsarray);
   if (staticArrayComparison) {
     // TODO: simply codegen when comparing static arrays with different length (int[3] == int[2])
     return callMemcmp(loc, irs, l_ptr, r_ptr, l_length);
@@ -1120,7 +1119,7 @@ LLValue *DtoArrayEqCmp_memcmp(Loc &loc, DValue *l, DValue *r, IRState &irs) {
 } // end anonymous namespace
 
 ////////////////////////////////////////////////////////////////////////////////
-LLValue *DtoArrayEquals(Loc &loc, TOK op, DValue *l, DValue *r) {
+LLValue *DtoArrayEquals(const Loc &loc, TOK op, DValue *l, DValue *r) {
   LLValue *res = nullptr;
 
   if (r->isNull()) {
@@ -1163,7 +1162,7 @@ LLValue *DtoArrayLen(DValue *v) {
   LOG_SCOPE;
 
   Type *t = v->type->toBasetype();
-  if (t->ty == Tarray) {
+  if (t->ty == TY::Tarray) {
     if (v->isNull()) {
       return DtoConstSize_t(0);
     }
@@ -1174,7 +1173,7 @@ LLValue *DtoArrayLen(DValue *v) {
     assert(slice);
     return slice->getLength();
   }
-  if (t->ty == Tsarray) {
+  if (t->ty == TY::Tsarray) {
     assert(!v->isSlice());
     assert(!v->isNull());
     TypeSArray *sarray = static_cast<TypeSArray *>(t);
@@ -1194,7 +1193,7 @@ LLValue *DtoArrayPtr(DValue *v) {
   LLType *wantedLLPtrType = DtoPtrToType(t->nextOf());
   LLValue *ptr = nullptr;
 
-  if (t->ty == Tarray) {
+  if (t->ty == TY::Tarray) {
     if (v->isNull()) {
       ptr = getNullPtr(wantedLLPtrType);
     } else if (v->isLVal()) {
@@ -1204,7 +1203,7 @@ LLValue *DtoArrayPtr(DValue *v) {
       assert(slice);
       ptr = slice->getPtr();
     }
-  } else if (t->ty == Tsarray) {
+  } else if (t->ty == TY::Tsarray) {
     assert(!v->isSlice());
     assert(!v->isNull());
     ptr = DtoLVal(v);
@@ -1216,7 +1215,7 @@ LLValue *DtoArrayPtr(DValue *v) {
 }
 
 ////////////////////////////////////////////////////////////////////////////////
-DValue *DtoCastArray(Loc &loc, DValue *u, Type *to) {
+DValue *DtoCastArray(const Loc &loc, DValue *u, Type *to) {
   IF_LOG Logger::println("DtoCastArray");
   LOG_SCOPE;
 
@@ -1224,14 +1223,14 @@ DValue *DtoCastArray(Loc &loc, DValue *u, Type *to) {
 
   Type *totype = to->toBasetype();
   Type *fromtype = u->type->toBasetype();
-  if (fromtype->ty != Tarray && fromtype->ty != Tsarray) {
+  if (fromtype->ty != TY::Tarray && fromtype->ty != TY::Tsarray) {
     error(loc, "can't cast `%s` to `%s`", u->type->toChars(), to->toChars());
     fatal();
   }
 
   IF_LOG Logger::cout() << "from array or sarray" << '\n';
 
-  if (totype->ty == Tpointer) {
+  if (totype->ty == TY::Tpointer) {
     IF_LOG Logger::cout() << "to pointer" << '\n';
     LLValue *ptr = DtoArrayPtr(u);
     if (ptr->getType() != tolltype) {
@@ -1240,12 +1239,12 @@ DValue *DtoCastArray(Loc &loc, DValue *u, Type *to) {
     return new DImValue(to, ptr);
   }
 
-  if (totype->ty == Tarray) {
+  if (totype->ty == TY::Tarray) {
     IF_LOG Logger::cout() << "to array" << '\n';
 
     LLValue *length = nullptr;
     LLValue *ptr = nullptr;
-    if (fromtype->ty == Tsarray) {
+    if (fromtype->ty == TY::Tsarray) {
       length = DtoConstSize_t(
           static_cast<TypeSArray *>(fromtype)->dim->toUInteger());
       ptr = DtoLVal(u);
@@ -1280,11 +1279,11 @@ DValue *DtoCastArray(Loc &loc, DValue *u, Type *to) {
     return new DSliceValue(to, length, DtoBitCast(ptr, ptrty));
   }
 
-  if (totype->ty == Tsarray) {
+  if (totype->ty == TY::Tsarray) {
     IF_LOG Logger::cout() << "to sarray" << '\n';
 
     LLValue *ptr = nullptr;
-    if (fromtype->ty == Tsarray) {
+    if (fromtype->ty == TY::Tsarray) {
       ptr = DtoLVal(u);
     } else {
       size_t tosize = static_cast<TypeSArray *>(totype)->dim->toInteger();
@@ -1298,7 +1297,7 @@ DValue *DtoCastArray(Loc &loc, DValue *u, Type *to) {
     return new DLValue(to, DtoBitCast(ptr, getPtrToType(tolltype)));
   }
 
-  if (totype->ty == Tbool) {
+  if (totype->ty == TY::Tbool) {
     // return (arr.ptr !is null)
     LLValue *ptr = DtoArrayPtr(u);
     LLConstant *nul = getNullPtr(ptr->getType());
@@ -1309,25 +1308,30 @@ DValue *DtoCastArray(Loc &loc, DValue *u, Type *to) {
   return new DLValue(to, castedPtr);
 }
 
-void DtoIndexBoundsCheck(Loc &loc, DValue *arr, DValue *index) {
+void DtoIndexBoundsCheck(const Loc &loc, DValue *arr, DValue *index) {
   Type *arrty = arr->type->toBasetype();
-  assert(
-      (arrty->ty == Tsarray || arrty->ty == Tarray || arrty->ty == Tpointer) &&
-      "Can only array bounds check for static or dynamic arrays");
+  assert((arrty->ty == TY::Tsarray || arrty->ty == TY::Tarray ||
+          arrty->ty == TY::Tpointer) &&
+         "Can only array bounds check for static or dynamic arrays");
 
   if (!index) {
     // Caller supplied no index, known in-bounds.
     return;
   }
 
-  if (arrty->ty == Tpointer) {
-    // Length of pointers is unknown, ingore.
+  if (arrty->ty == TY::Tpointer) {
+    // Length of pointers is unknown, ignore.
     return;
   }
+  if (auto ts = arrty->isTypeSArray()) {
+    if (ts->isIncomplete()) // importC
+      return;
+  }
 
-  llvm::ICmpInst::Predicate cmpop = llvm::ICmpInst::ICMP_ULT;
-  llvm::Value *cond = gIR->ir->CreateICmp(cmpop, DtoRVal(index),
-                                          DtoArrayLen(arr), "bounds.cmp");
+  LLValue *const llIndex = DtoRVal(index);
+  LLValue *const llLength = DtoArrayLen(arr);
+  LLValue *const cond = gIR->ir->CreateICmp(llvm::ICmpInst::ICMP_ULT, llIndex,
+                                            llLength, "bounds.cmp");
 
   llvm::BasicBlock *okbb = gIR->insertBB("bounds.ok");
   llvm::BasicBlock *failbb = gIR->insertBBAfter(okbb, "bounds.fail");
@@ -1335,24 +1339,54 @@ void DtoIndexBoundsCheck(Loc &loc, DValue *arr, DValue *index) {
 
   // set up failbb to call the array bounds error runtime function
   gIR->ir->SetInsertPoint(failbb);
-  DtoBoundsCheckFailCall(gIR, loc);
+  emitArrayIndexError(gIR, loc, llIndex, llLength);
 
   // if ok, proceed in okbb
   gIR->ir->SetInsertPoint(okbb);
 }
 
-void DtoBoundsCheckFailCall(IRState *irs, Loc &loc) {
+static void emitRangeErrorImpl(IRState *irs, const Loc &loc,
+                               const char *cAssertMsg, const char *dFnName,
+                               llvm::ArrayRef<LLValue *> extraArgs) {
   Module *const module = irs->func()->decl->getModule();
 
-  if (global.params.checkAction == CHECKACTION_C) {
-    DtoCAssert(module, loc, DtoConstCString("array overflow"));
-  } else {
-    llvm::Function *errorfn =
-        getRuntimeFunction(loc, irs->module, "_d_arraybounds");
-    irs->CreateCallOrInvoke(errorfn, DtoModuleFileName(module, loc),
-                            DtoConstUint(loc.linnum));
-
-    // the function does not return
+  switch (global.params.checkAction) {
+  case CHECKACTION_C:
+    DtoCAssert(module, loc, DtoConstCString(cAssertMsg));
+    break;
+  case CHECKACTION_halt:
+    irs->ir->CreateCall(GET_INTRINSIC_DECL(trap), {});
     irs->ir->CreateUnreachable();
+    break;
+  case CHECKACTION_context:
+  case CHECKACTION_D: {
+    auto fn = getRuntimeFunction(loc, irs->module, dFnName);
+    LLSmallVector<LLValue *, 5> args;
+    args.reserve(2 + extraArgs.size());
+    args.push_back(DtoModuleFileName(module, loc));
+    args.push_back(DtoConstUint(loc.linnum));
+    args.insert(args.end(), extraArgs.begin(), extraArgs.end());
+    irs->CreateCallOrInvoke(fn, args);
+    irs->ir->CreateUnreachable();
+    break;
+  }
+  default:
+    llvm_unreachable("Unhandled checkAction");
   }
 }
+
+void emitRangeError(IRState *irs, const Loc &loc) {
+  emitRangeErrorImpl(irs, loc, "array overflow", "_d_arraybounds", {});
+}
+
+void emitArraySliceError(IRState *irs, const Loc &loc, LLValue *lower,
+                         LLValue *upper, LLValue *length) {
+  emitRangeErrorImpl(irs, loc, "array slice out of bounds",
+                     "_d_arraybounds_slice", {lower, upper, length});
+}
+
+void emitArrayIndexError(IRState *irs, const Loc &loc, LLValue *index,
+                         LLValue *length) {
+  emitRangeErrorImpl(irs, loc, "array index out of bounds",
+                     "_d_arraybounds_index", {index, length});
+}
