@@ -10,6 +10,7 @@
 #include "dmd/declaration.h"
 #include "dmd/dsymbol.h"
 #include "dmd/errors.h"
+#include "dmd/ldcbindings.h"
 #include "dmd/scope.h"
 #include "dmd/statement.h"
 #include "gen/dvalue.h"
@@ -109,6 +110,9 @@ Statement *asmSemantic(AsmStatement *s, Scope *sc) {
     return gccAsmSemantic(gas, sc);
   }
 
+  // this is DMD-style asm
+  sc->func->hasReturnExp |= 32;
+
   auto ias = createInlineAsmStatement(s->loc, s->tokens);
   s = ias;
 
@@ -156,7 +160,7 @@ void AsmStatement_toIR(InlineAsmStatement *stmt, IRState *irs) {
   LOG_SCOPE;
 
   // sanity check
-  assert(irs->func()->decl->hasReturnExp & 8);
+  assert((irs->func()->decl->hasReturnExp & 40) == 40);
 
   // get asm block
   IRAsmBlock *asmblock = irs->asmBlock;
@@ -603,7 +607,7 @@ void CompoundAsmStatement_toIR(CompoundAsmStatement *stmt, IRState *p) {
   bool useabiret = false;
   p->asmBlock->asmBlock->abiret = nullptr;
   if (thisfunc->fbody->endsWithAsm() == stmt &&
-      thisfunc->type->nextOf()->ty != Tvoid) {
+      thisfunc->type->nextOf()->ty != TY::Tvoid) {
     // there can't be goto forwarders in this case
     assert(gotoToVal.empty());
     emitABIReturnAsmStmt(asmblock, stmt->loc, thisfunc);
