@@ -30,6 +30,7 @@
 #include "gen/structs.h"
 #include "gen/tollvm.h"
 #include "ir/iraggr.h"
+#include "ir/irdsymbol.h"
 #include "ir/irfunction.h"
 #include "ir/irtypeclass.h"
 
@@ -56,7 +57,7 @@ void DtoResolveClass(ClassDeclaration *cd) {
   DtoType(cd->type);
 
   // create IrAggr
-  IrClass *irAggr = getIrAggr(cd, true);
+  getIrAggr(cd, true);
 
   // make sure all fields really get their ir field
   for (auto vd : cd->fields) {
@@ -67,16 +68,11 @@ void DtoResolveClass(ClassDeclaration *cd) {
     }
     getIrField(vd, true);
   }
-
-  // interface only emit typeinfo and classinfo
-  if (cd->isInterfaceDeclaration()) {
-    irAggr->initializeInterface();
-  }
 }
 
 ////////////////////////////////////////////////////////////////////////////////
 
-DValue *DtoNewClass(Loc &loc, TypeClass *tc, NewExp *newexp) {
+DValue *DtoNewClass(const Loc &loc, TypeClass *tc, NewExp *newexp) {
   // resolve type
   DtoResolveClass(tc->sym);
 
@@ -84,20 +80,9 @@ DValue *DtoNewClass(Loc &loc, TypeClass *tc, NewExp *newexp) {
   LLValue *mem;
   bool doInit = true;
   if (newexp->onstack) {
-    unsigned alignment = tc->sym->alignsize;
-    if (alignment == STRUCTALIGN_DEFAULT)
-      alignment = 0;
-    mem = DtoRawAlloca(DtoType(tc)->getContainedType(0), alignment,
+    mem = DtoRawAlloca(DtoType(tc)->getContainedType(0), tc->sym->alignsize,
                        ".newclass_alloca");
-  }
-  // custom allocator
-  else if (newexp->allocator) {
-    DFuncValue dfn(newexp->allocator, DtoCallee(newexp->allocator));
-    DValue *res = DtoCallFunction(newexp->loc, nullptr, &dfn, newexp->newargs);
-    mem = DtoBitCast(DtoRVal(res), DtoType(tc), ".newclass_custom");
-  }
-  // default allocator
-  else {
+  } else {
     const bool useEHAlloc = global.params.ehnogc && newexp->thrownew;
     llvm::Function *fn = getRuntimeFunction(
         loc, gIR->module, useEHAlloc ? "_d_newThrowable" : "_d_allocclass");
@@ -191,7 +176,7 @@ void DtoInitClass(TypeClass *tc, LLValue *dst) {
 
 ////////////////////////////////////////////////////////////////////////////////
 
-void DtoFinalizeClass(Loc &loc, LLValue *inst) {
+void DtoFinalizeClass(const Loc &loc, LLValue *inst) {
   // get runtime function
   llvm::Function *fn =
       getRuntimeFunction(loc, gIR->module, "_d_callfinalizer");
@@ -202,7 +187,7 @@ void DtoFinalizeClass(Loc &loc, LLValue *inst) {
 
 ////////////////////////////////////////////////////////////////////////////////
 
-void DtoFinalizeScopeClass(Loc &loc, LLValue *inst, bool hasDtor) {
+void DtoFinalizeScopeClass(const Loc &loc, LLValue *inst, bool hasDtor) {
   if (!isOptimizationEnabled() || hasDtor) {
     DtoFinalizeClass(loc, inst);
     return;
@@ -228,7 +213,7 @@ void DtoFinalizeScopeClass(Loc &loc, LLValue *inst, bool hasDtor) {
 
 ////////////////////////////////////////////////////////////////////////////////
 
-DValue *DtoCastClass(Loc &loc, DValue *val, Type *_to) {
+DValue *DtoCastClass(const Loc &loc, DValue *val, Type *_to) {
   IF_LOG Logger::println("DtoCastClass(%s, %s)", val->type->toChars(),
                          _to->toChars());
   LOG_SCOPE;
@@ -236,14 +221,14 @@ DValue *DtoCastClass(Loc &loc, DValue *val, Type *_to) {
   Type *to = _to->toBasetype();
 
   // class -> pointer
-  if (to->ty == Tpointer) {
+  if (to->ty == TY::Tpointer) {
     IF_LOG Logger::println("to pointer");
     LLType *tolltype = DtoType(_to);
     LLValue *rval = DtoBitCast(DtoRVal(val), tolltype);
     return new DImValue(_to, rval);
   }
   // class -> bool
-  if (to->ty == Tbool) {
+  if (to->ty == TY::Tbool) {
     IF_LOG Logger::println("to bool");
     LLValue *llval = DtoRVal(val);
     LLValue *zero = LLConstant::getNullValue(llval->getType());
@@ -262,13 +247,13 @@ DValue *DtoCastClass(Loc &loc, DValue *val, Type *_to) {
     return DtoCastInt(loc, &im, _to);
   }
   // class -> typeof(null)
-  if (to->ty == Tnull) {
+  if (to->ty == TY::Tnull) {
     IF_LOG Logger::println("to %s", to->toChars());
     return new DImValue(_to, LLConstant::getNullValue(DtoType(_to)));
   }
 
   // must be class/interface
-  assert(to->ty == Tclass);
+  assert(to->ty == TY::Tclass);
   TypeClass *tc = static_cast<TypeClass *>(to);
 
   // from type
@@ -346,7 +331,7 @@ static void resolveObjectAndClassInfoClasses() {
   DtoResolveClass(Type::typeinfoclass);
 }
 
-DValue *DtoDynamicCastObject(Loc &loc, DValue *val, Type *_to) {
+DValue *DtoDynamicCastObject(const Loc &loc, DValue *val, Type *_to) {
   // call:
   // Object _d_dynamic_cast(Object o, ClassInfo c)
 
@@ -383,7 +368,7 @@ DValue *DtoDynamicCastObject(Loc &loc, DValue *val, Type *_to) {
 
 ////////////////////////////////////////////////////////////////////////////////
 
-DValue *DtoDynamicCastInterface(Loc &loc, DValue *val, Type *_to) {
+DValue *DtoDynamicCastInterface(const Loc &loc, DValue *val, Type *_to) {
   // call:
   // Object _d_interface_cast(void* p, ClassInfo c)
 
@@ -421,7 +406,7 @@ LLValue *DtoVirtualFunctionPointer(DValue *inst, FuncDeclaration *fdecl) {
   // sanity checks
   assert(fdecl->isVirtual());
   assert(!fdecl->isFinalFunc());
-  assert(inst->type->toBasetype()->ty == Tclass);
+  assert(inst->type->toBasetype()->ty == TY::Tclass);
   // slot 0 is always ClassInfo/Interface* unless it is a CPP class
   assert(fdecl->vtblIndex > 0 ||
          (fdecl->vtblIndex == 0 &&
