@@ -12,13 +12,14 @@
 #include "dmd/declaration.h"
 #include "dmd/enum.h"
 #include "dmd/errors.h"
+#include "dmd/expression.h"
 #include "dmd/identifier.h"
 #include "dmd/import.h"
-#include "dmd/ldcbindings.h"
 #include "dmd/mangle.h"
 #include "dmd/module.h"
 #include "dmd/mtype.h"
 #include "dmd/nspace.h"
+#include "dmd/target.h"
 #include "dmd/template.h"
 #include "driver/cl_options.h"
 #include "driver/ldc-version.h"
@@ -53,8 +54,6 @@ namespace ldc {
 const char *convertDIdentifierToCPlusPlus(const char *name, size_t nameLength);
 
 namespace {
-DIType getNullDIType() { return nullptr; }
-
 llvm::StringRef uniqueIdent(Type *t) {
   if (t->deco)
     return t->deco;
@@ -107,8 +106,6 @@ DIBuilder::DIBuilder(IRState *const IR)
       // (https://reviews.llvm.org/D23720)
       emitColumnInfo(opts::getFlagOrDefault(::emitColumnInfo, !emitCodeView)) {}
 
-llvm::LLVMContext &DIBuilder::getContext() { return IR->context(); }
-
 unsigned DIBuilder::getColumn(const Loc &loc) const {
   return (loc.linnum && emitColumnInfo) ? loc.charnum : 0;
 }
@@ -127,7 +124,7 @@ DIScope DIBuilder::GetSymbolScope(Dsymbol *s) {
     // parent composite types have to get declared
     while (!parent->isModule()) {
       if (parent->isAggregateDeclaration())
-        CreateCompositeTypeDescription(parent->getType());
+        CreateCompositeType(parent->getType());
       parent = parent->toParent();
     }
   }
@@ -137,7 +134,7 @@ DIScope DIBuilder::GetSymbolScope(Dsymbol *s) {
   } else if (auto m = parent->isModule()) {
     return EmitModule(m);
   } else if (parent->isAggregateDeclaration()) {
-    return CreateCompositeTypeDescription(parent->getType());
+    return CreateCompositeType(parent->getType());
   } else if (auto fd = parent->isFuncDeclaration()) {
     DtoDeclareFunction(fd);
     return EmitSubProgram(fd);
@@ -146,7 +143,10 @@ DIScope DIBuilder::GetSymbolScope(Dsymbol *s) {
   } else if (auto fwd = parent->isForwardingScopeDsymbol()) {
     return GetSymbolScope(fwd);
   } else if (auto ed = parent->isEnumDeclaration()) {
-    return CreateEnumType(ed->getType());
+    auto et = CreateEnumType(ed->getType()->isTypeEnum());
+    if (llvm::isa<llvm::DICompositeType>(et))
+      return et;
+    return EmitNamespace(ed, ed->toChars());
   } else {
     error(parent->loc, "unknown debuginfo scope `%s`; please file an LDC issue",
           parent->toChars());
@@ -193,41 +193,52 @@ llvm::StringRef DIBuilder::GetNameAndScope(Dsymbol *sym, DIScope &scope) {
 // Sets the memory address for a debuginfo variable.
 void DIBuilder::Declare(const Loc &loc, llvm::Value *storage,
                         DILocalVariable divar, DIExpression diexpr) {
-  auto debugLoc =
-      llvm::DebugLoc::get(loc.linnum, getColumn(loc), GetCurrentScope());
+  auto debugLoc = llvm::DILocation::get(IR->context(), loc.linnum,
+                                        getColumn(loc), GetCurrentScope());
   DBuilder.insertDeclare(storage, divar, diexpr, debugLoc, IR->scopebb());
 }
 
 // Sets the (current) value for a debuginfo variable.
 void DIBuilder::SetValue(const Loc &loc, llvm::Value *value,
                          DILocalVariable divar, DIExpression diexpr) {
-  auto debugLoc =
-      llvm::DebugLoc::get(loc.linnum, getColumn(loc), GetCurrentScope());
+  auto debugLoc = llvm::DILocation::get(IR->context(), loc.linnum,
+                                        getColumn(loc), GetCurrentScope());
   DBuilder.insertDbgValueIntrinsic(value, divar, diexpr, debugLoc,
                                    IR->scopebb());
 }
 
-DIFile DIBuilder::CreateFile(const Loc &loc) {
-  const char *filename = loc.filename;
+DIFile DIBuilder::CreateFile(const char *filename) {
   if (!filename)
     filename = IR->dmodule->srcfile.toChars();
-  llvm::SmallString<128> path(filename);
-  llvm::sys::fs::make_absolute(path);
 
-  return DBuilder.createFile(llvm::sys::path::filename(path),
-                             llvm::sys::path::parent_path(path));
+  // clang appears to use the curent working dir as 'directory' for relative
+  // source paths, and the root path for absolute ones:
+  // clang -g -emit-llvm -S ..\blub.c =>
+  //   !DIFile(filename: "..\\blub.c", directory: "C:\\LDC\\ninja-ldc", ...)
+  //   !DIFile(filename: "Program
+  //   Files\\LLVM\\lib\\clang\\11.0.1\\include\\stddef.h", directory: "C:\\",
+  //   ...)
+
+  if (llvm::sys::path::is_absolute(filename)) {
+    return DBuilder.createFile(llvm::sys::path::relative_path(filename),
+                               llvm::sys::path::root_path(filename));
+  }
+
+  llvm::SmallString<128> cwd;
+  llvm::sys::fs::current_path(cwd);
+
+  return DBuilder.createFile(filename, cwd);
 }
 
-DIFile DIBuilder::CreateFile() {
-  Loc loc(IR->dmodule->srcfile.toChars(), 0, 0);
-  return CreateFile(loc);
+DIFile DIBuilder::CreateFile(const Loc &loc) {
+  return CreateFile(loc.filename);
 }
 
 DIFile DIBuilder::CreateFile(Dsymbol *decl) {
-  Loc loc;
-  for (Dsymbol *sym = decl; sym && !loc.filename; sym = sym->parent)
-    loc = sym->loc;
-  return loc.filename ? CreateFile(loc) : CreateFile();
+  const char *filename = nullptr;
+  for (Dsymbol *sym = decl; sym && !filename; sym = sym->parent)
+    filename = sym->loc.filename;
+  return CreateFile(filename);
 }
 
 DIType DIBuilder::CreateBasicType(Type *type) {
@@ -239,54 +250,54 @@ DIType DIBuilder::CreateBasicType(Type *type) {
   // find encoding
   unsigned Encoding;
   switch (t->ty) {
-  case Tbool:
+  case TY::Tbool:
     Encoding = DW_ATE_boolean;
     break;
-  case Tchar:
+  case TY::Tchar:
     if (emitCodeView) {
       // VS debugger does not support DW_ATE_UTF for char
       Encoding = DW_ATE_unsigned_char;
       break;
     }
     // fall through
-  case Twchar:
-  case Tdchar:
+  case TY::Twchar:
+  case TY::Tdchar:
     Encoding = DW_ATE_UTF;
     break;
-  case Tint8:
+  case TY::Tint8:
     if (emitCodeView) {
       // VS debugger does not support DW_ATE_signed for 8-bit
       Encoding = DW_ATE_signed_char;
       break;
     }
     // fall through
-  case Tint16:
-  case Tint32:
-  case Tint64:
-  case Tint128:
+  case TY::Tint16:
+  case TY::Tint32:
+  case TY::Tint64:
+  case TY::Tint128:
     Encoding = DW_ATE_signed;
     break;
-  case Tuns8:
+  case TY::Tuns8:
     if (emitCodeView) {
       // VS debugger does not support DW_ATE_unsigned for 8-bit
       Encoding = DW_ATE_unsigned_char;
       break;
     }
     // fall through
-  case Tuns16:
-  case Tuns32:
-  case Tuns64:
-  case Tuns128:
+  case TY::Tuns16:
+  case TY::Tuns32:
+  case TY::Tuns64:
+  case TY::Tuns128:
     Encoding = DW_ATE_unsigned;
     break;
-  case Tfloat32:
-  case Tfloat64:
-  case Tfloat80:
+  case TY::Tfloat32:
+  case TY::Tfloat64:
+  case TY::Tfloat80:
     Encoding = DW_ATE_float;
     break;
-  case Timaginary32:
-  case Timaginary64:
-  case Timaginary80:
+  case TY::Timaginary32:
+  case TY::Timaginary64:
+  case TY::Timaginary80:
     if (emitCodeView) {
       // DW_ATE_imaginary_float not supported by the LLVM DWARF->CodeView
       // conversion
@@ -295,9 +306,9 @@ DIType DIBuilder::CreateBasicType(Type *type) {
     }
     Encoding = DW_ATE_imaginary_float;
     break;
-  case Tcomplex32:
-  case Tcomplex64:
-  case Tcomplex80:
+  case TY::Tcomplex32:
+  case TY::Tcomplex64:
+  case TY::Tcomplex80:
     if (emitCodeView) {
       // DW_ATE_complex_float not supported by the LLVM DWARF->CodeView
       // conversion
@@ -315,34 +326,39 @@ DIType DIBuilder::CreateBasicType(Type *type) {
                                   Encoding);
 }
 
-DIType DIBuilder::CreateEnumType(Type *type) {
-  assert(type->ty == Tenum);
+DIType DIBuilder::CreateEnumType(TypeEnum *type) {
+  EnumDeclaration *const ed = type->sym;
 
-  llvm::Type *const T = DtoType(type);
-  EnumDeclaration *const ed = static_cast<TypeEnum *>(type)->sym;
-  assert(ed->memtype);
+  if (!ed->memtype) // opaque enum
+    return CreateUnspecifiedType(ed);
 
-  if (ed->isSpecial()) {
-    return CreateBasicType(ed->memtype);
-  }
-
-  llvm::SmallVector<LLMetadata *, 8> subscripts;
-  for (auto m : *ed->members) {
-    EnumMember *em = m->isEnumMember();
-    if (auto ie = em->value()->isIntegerExp()) {
-      subscripts.push_back(
-          DBuilder.createEnumerator(em->toChars(), ie->toInteger()));
-    } else {
-      subscripts.clear();
-      break;
-    }
-  }
+  if (ed->isSpecial()) // magic enums: forward to base type
+    return CreateTypeDescription(ed->memtype);
 
   DIScope scope = nullptr;
   const auto name = GetNameAndScope(ed, scope);
   const auto lineNumber = ed->loc.linnum;
   const auto file = CreateFile(ed);
 
+  // just emit a typedef for non-integral base types
+  auto tb = type->toBasetype();
+  if (!tb->isintegral()) {
+    auto tbase = CreateTypeDescription(tb);
+    return DBuilder.createTypedef(tbase, name, file, lineNumber, scope);
+  }
+
+  llvm::SmallVector<LLMetadata *, 8> subscripts;
+  if (ed->members) {
+    for (auto m : *ed->members) {
+      EnumMember *em = m->isEnumMember();
+      if (auto ie = em->value()->isIntegerExp()) {
+        subscripts.push_back(
+            DBuilder.createEnumerator(em->toChars(), ie->toInteger()));
+      }
+    }
+  }
+
+  llvm::Type *const T = DtoType(type);
   return DBuilder.createEnumerationType(
       scope, name, file, lineNumber,
       getTypeAllocSize(T) * 8,               // size (bits)
@@ -351,16 +367,8 @@ DIType DIBuilder::CreateEnumType(Type *type) {
       CreateTypeDescription(ed->memtype));
 }
 
-DIType DIBuilder::CreatePointerType(Type *type) {
+DIType DIBuilder::CreatePointerType(TypePointer *type) {
   llvm::Type *T = DtoType(type);
-  Type *t = type->toBasetype();
-  assert(t->ty == Tpointer);
-
-  // find base type
-  Type *nt = t->nextOf();
-  // translate void pointers to byte pointers
-  if (nt->toBasetype()->ty == Tvoid)
-    nt = Type::tuns8;
 
   // TODO: The addressspace is important for dcompute targets. See e.g.
   // https://www.mail-archive.com/dwarf-discuss@lists.dwarfstd.org/msg00326.html
@@ -368,24 +376,17 @@ DIType DIBuilder::CreatePointerType(Type *type) {
 
   const auto name = processDIName(type->toPrettyChars(true));
 
-  return DBuilder.createPointerType(CreateTypeDescription(nt),
-                                    getTypeAllocSize(T) * 8, // size (bits)
-                                    getABITypeAlign(T) * 8,  // align (bits)
-                                    DWARFAddressSpace, name);
+  return DBuilder.createPointerType(
+      CreateTypeDescription(type->nextOf(), /*voidToUbyte=*/true),
+      getTypeAllocSize(T) * 8, // size (bits)
+      getABITypeAlign(T) * 8,  // align (bits)
+      DWARFAddressSpace, name);
 }
 
-DIType DIBuilder::CreateVectorType(Type *type) {
+DIType DIBuilder::CreateVectorType(TypeVector *type) {
   LLType *T = DtoType(type);
-  Type *t = type->toBasetype();
 
-  assert(t->ty == Tvector &&
-         "Only vectors allowed for debug info in DIBuilder::CreateVectorType");
-  TypeVector *tv = static_cast<TypeVector *>(t);
-  Type *te = tv->elementType();
-  // translate void vectors to byte vectors
-  if (te->toBasetype()->ty == Tvoid)
-    te = Type::tuns8;
-  const auto dim = tv->size(Loc()) / te->size(Loc());
+  const auto dim = type->basetype->isTypeSArray()->dim->toInteger();
 #if LDC_LLVM_VER >= 1100
   const auto Dim = llvm::ConstantAsMetadata::get(DtoConstSize_t(dim));
   auto subscript = DBuilder.getOrCreateSubrange(Dim, nullptr, nullptr, nullptr);
@@ -394,26 +395,26 @@ DIType DIBuilder::CreateVectorType(Type *type) {
 #endif
 
   return DBuilder.createVectorType(
-      getTypeAllocSize(T) * 8,               // size (bits)
-      getABITypeAlign(T) * 8,                // align (bits)
-      CreateTypeDescription(te),             // element type
+      getTypeAllocSize(T) * 8, // size (bits)
+      getABITypeAlign(T) * 8,  // align (bits)
+      CreateTypeDescription(type->elementType(), /*voidToUbyte=*/true),
       DBuilder.getOrCreateArray({subscript}) // subscripts
   );
 }
 
 DIType DIBuilder::CreateComplexType(Type *type) {
-  llvm::Type *T = DtoType(type);
   Type *t = type->toBasetype();
+  llvm::Type *T = DtoType(type);
 
   Type *elemtype = nullptr;
   switch (t->ty) {
-  case Tcomplex32:
+  case TY::Tcomplex32:
     elemtype = Type::tfloat32;
     break;
-  case Tcomplex64:
+  case TY::Tcomplex64:
     elemtype = Type::tfloat64;
     break;
-  case Tcomplex80:
+  case TY::Tcomplex80:
     elemtype = Type::tfloat80;
     break;
   default:
@@ -424,8 +425,8 @@ DIType DIBuilder::CreateComplexType(Type *type) {
 
   auto imoffset = getTypeAllocSize(DtoType(elemtype));
   LLMetadata *elems[] = {
-      CreateMemberType(0, elemtype, file, "re", 0, Prot::public_),
-      CreateMemberType(0, elemtype, file, "im", imoffset, Prot::public_)};
+      CreateMemberType(0, elemtype, file, "re", 0, Visibility::public_),
+      CreateMemberType(0, elemtype, file, "im", imoffset, Visibility::public_)};
 
   return DBuilder.createStructType(GetCU(),
                                    t->toChars(),            // Name
@@ -434,51 +435,31 @@ DIType DIBuilder::CreateComplexType(Type *type) {
                                    getTypeAllocSize(T) * 8, // size in bits
                                    getABITypeAlign(T) * 8,  // alignment
                                    DIFlags::FlagZero,       // What here?
-                                   getNullDIType(),         // derived from
+                                   nullptr,                 // derived from
                                    DBuilder.getOrCreateArray(elems),
                                    0,               // RunTimeLang
-                                   getNullDIType(), // VTableHolder
+                                   nullptr,         // VTableHolder
                                    uniqueIdent(t)); // UniqueIdentifier
 }
 
-DIType DIBuilder::CreateTypedef(unsigned linnum, Type *type, DIFile file,
-                                const char *c_name) {
-  Type *t = type->toBasetype();
-
-  // translate functions to function pointers
-  if (t->ty == Tfunction)
-    t = t->pointerTo();
-
-  // find base type
-  DIType basetype = CreateTypeDescription(t);
-
-  return DBuilder.createTypedef(basetype, c_name, file, linnum, GetCU());
-}
-
 DIType DIBuilder::CreateMemberType(unsigned linnum, Type *type, DIFile file,
                                    const char *c_name, unsigned offset,
-                                   Prot::Kind prot, bool isStatic,
+                                   Visibility::Kind visibility, bool isStatic,
                                    DIScope scope) {
-  Type *t = type->toBasetype();
-
-  // translate functions to function pointers
-  if (t->ty == Tfunction)
-    t = t->pointerTo();
-
-  llvm::Type *T = DtoType(t);
+  llvm::Type *T = DtoType(type);
 
   // find base type
-  DIType basetype = CreateTypeDescription(t);
+  DIType basetype = CreateTypeDescription(type);
 
   auto Flags = DIFlags::FlagZero;
-  switch (prot) {
-  case Prot::private_:
+  switch (visibility) {
+  case Visibility::private_:
     Flags = DIFlags::FlagPrivate;
     break;
-  case Prot::protected_:
+  case Visibility::protected_:
     Flags = DIFlags::FlagProtected;
     break;
-  case Prot::public_:
+  case Visibility::public_:
     Flags = DIFlags::FlagPublic;
     break;
   default:
@@ -505,9 +486,12 @@ void DIBuilder::AddFields(AggregateDeclaration *ad, DIFile file,
   size_t narr = ad->fields.length;
   elems.reserve(narr);
   for (auto vd : ad->fields) {
+    if (vd->type->toBasetype()->isTypeNoreturn())
+      continue;
+
     elems.push_back(CreateMemberType(vd->loc.linnum, vd->type, file,
                                      vd->toChars(), vd->offset,
-                                     vd->prot().kind));
+                                     vd->visibility.kind));
   }
 }
 
@@ -517,7 +501,7 @@ void DIBuilder::AddStaticMembers(AggregateDeclaration *ad, DIFile file,
   if (!members)
     return;
 
-  auto scope = CreateCompositeTypeDescription(ad->getType());
+  auto scope = CreateCompositeType(ad->getType());
 
   std::function<void(Dsymbols *)> visitMembers = [&](Dsymbols *members) {
     for (auto s : *members) {
@@ -532,10 +516,12 @@ void DIBuilder::AddStaticMembers(AggregateDeclaration *ad, DIFile file,
         // currently does nothing with DIImportedEntity except at CU-level.
         visitMembers(tmixin->members);
       } else if (auto vd = s->isVarDeclaration())
-        if (vd->isDataseg() && !vd->aliassym /* TODO: tuples*/) {
-          llvm::MDNode *elem = CreateMemberType(
-              vd->loc.linnum, vd->type, file, vd->toChars(), 0, vd->prot().kind,
-              /*isStatic = */ true, scope);
+        if (vd->isDataseg() && !vd->aliassym /* TODO: tuples*/ &&
+            !vd->type->toBasetype()->isTypeNoreturn()) {
+          llvm::MDNode *elem =
+              CreateMemberType(vd->loc.linnum, vd->type, file, vd->toChars(), 0,
+                               vd->visibility.kind,
+                               /*isStatic = */ true, scope);
           elems.push_back(elem);
           StaticDataMemberCache[vd].reset(elem);
         }
@@ -548,28 +534,25 @@ void DIBuilder::AddStaticMembers(AggregateDeclaration *ad, DIFile file,
   visitMembers(members);
 }
 
-DIType DIBuilder::CreateCompositeType(Type *type) {
-  Type *t = type->toBasetype();
-  assert((t->ty == Tstruct || t->ty == Tclass) &&
+DIType DIBuilder::CreateCompositeType(Type *t) {
+  assert((t->ty == TY::Tstruct || t->ty == TY::Tclass) &&
          "Unsupported type for debug info in DIBuilder::CreateCompositeType");
+
   AggregateDeclaration *ad;
-  if (t->ty == Tstruct) {
-    TypeStruct *ts = static_cast<TypeStruct *>(t);
-    ad = ts->sym;
+  if (t->ty == TY::Tstruct) {
+    ad = static_cast<TypeStruct *>(t)->sym;
   } else {
-    TypeClass *tc = static_cast<TypeClass *>(t);
-    ad = tc->sym;
+    ad = static_cast<TypeClass *>(t)->sym;
   }
-  assert(ad);
 
   // Use the actual type associated with the declaration, ignoring any
   // const/wrappers.
   LLType *T = DtoType(ad->type);
-  if (t->ty == Tclass)
+  if (t->ty == TY::Tclass)
     T = T->getPointerElementType();
   IrAggr *irAggr = getIrAggr(ad, true);
 
-  if (static_cast<llvm::MDNode *>(irAggr->diCompositeType) != nullptr) {
+  if (irAggr->diCompositeType) {
     return irAggr->diCompositeType;
   }
 
@@ -578,8 +561,8 @@ DIType DIBuilder::CreateCompositeType(Type *type) {
 
   // if we don't know the aggregate's size, we don't know enough about it
   // to provide debug info. probably a forward-declared struct?
-  if (ad->sizeok == SIZEOKnone) {
-    return DBuilder.createUnspecifiedType(name);
+  if (ad->sizeok == Sizeok::none) {
+    return CreateUnspecifiedType(ad);
   }
 
   assert(GetCU() && "Compilation unit missing or corrupted");
@@ -593,14 +576,14 @@ DIType DIBuilder::CreateCompositeType(Type *type) {
   const auto sizeInBits = T->isSized() ? getTypeAllocSize(T) * 8 : 0;
   const auto alignmentInBits = T->isSized() ? getABITypeAlign(T) * 8 : 0;
   const auto classOffsetInBits = 0;
-  auto derivedFrom = getNullDIType();
-  const auto vtableHolder = getNullDIType();
+  DIType derivedFrom = nullptr;
+  const auto vtableHolder = nullptr;
   const auto templateParams = nullptr;
   const auto uniqueIdentifier = uniqueIdent(t);
 
   // set diCompositeType to handle recursive types properly
-  unsigned tag = (t->ty == Tstruct) ? llvm::dwarf::DW_TAG_structure_type
-                                    : llvm::dwarf::DW_TAG_class_type;
+  unsigned tag = (t->ty == TY::Tstruct) ? llvm::dwarf::DW_TAG_structure_type
+                                        : llvm::dwarf::DW_TAG_class_type;
   irAggr->diCompositeType =
       DBuilder.createReplaceableCompositeType(tag, name, scope, file, lineNum);
 
@@ -609,13 +592,7 @@ DIType DIBuilder::CreateCompositeType(Type *type) {
     ClassDeclaration *classDecl = ad->isClassDeclaration();
     if (classDecl && classDecl->baseClass) {
       derivedFrom = CreateCompositeType(classDecl->baseClass->getType());
-      // needs a forward declaration to add inheritence information to elems
-      const auto elemsArray = nullptr;
-      DIType fwd = DBuilder.createClassType(
-          scope, name, file, lineNum, sizeInBits, alignmentInBits,
-          classOffsetInBits, DIFlags::FlagFwdDecl, derivedFrom, elemsArray,
-          vtableHolder, templateParams, uniqueIdentifier);
-      auto dt = DBuilder.createInheritance(fwd,
+      auto dt = DBuilder.createInheritance(irAggr->diCompositeType,
                                            derivedFrom, // base class type
                                            0,           // offset of base class
 #if LDC_LLVM_VER >= 700
@@ -631,7 +608,7 @@ DIType DIBuilder::CreateCompositeType(Type *type) {
   const auto elemsArray = DBuilder.getOrCreateArray(elems);
 
   DIType ret;
-  if (t->ty == Tclass) {
+  if (t->ty == TY::Tclass) {
     ret = DBuilder.createClassType(
         scope, name, file, lineNum, sizeInBits, alignmentInBits,
         classOffsetInBits, DIFlags::FlagZero, derivedFrom, elemsArray,
@@ -646,46 +623,42 @@ DIType DIBuilder::CreateCompositeType(Type *type) {
 
   irAggr->diCompositeType =
       DBuilder.replaceTemporary(llvm::TempDINode(irAggr->diCompositeType), ret);
-  irAggr->diCompositeType = ret;
 
-  return ret;
+  return irAggr->diCompositeType;
 }
 
-DIType DIBuilder::CreateArrayType(Type *type) {
+DIType DIBuilder::CreateArrayType(TypeArray *type) {
   llvm::Type *T = DtoType(type);
-  Type *t = type->toBasetype();
-  assert(t->ty == Tarray);
 
   const auto scope = GetCU();
   const auto name = processDIName(type->toPrettyChars(true));
   const auto file = CreateFile();
 
-  LLMetadata *elems[] = {
-      CreateMemberType(0, Type::tsize_t, file, "length", 0, Prot::public_),
-      CreateMemberType(0, t->nextOf()->pointerTo(), file, "ptr",
-                       global.params.is64bit ? 8 : 4, Prot::public_)};
+  LLMetadata *elems[] = {CreateMemberType(0, Type::tsize_t, file, "length", 0,
+                                          Visibility::public_),
+                         CreateMemberType(0, type->nextOf()->pointerTo(), file,
+                                          "ptr", target.ptrsize,
+                                          Visibility::public_)};
 
   return DBuilder.createStructType(scope, name, file,
                                    0,                       // LineNo
                                    getTypeAllocSize(T) * 8, // size in bits
                                    getABITypeAlign(T) * 8,  // alignment in bits
                                    DIFlags::FlagZero,       // What here?
-                                   getNullDIType(),         // derived from
+                                   nullptr,                 // derived from
                                    DBuilder.getOrCreateArray(elems),
-                                   0,               // RunTimeLang
-                                   getNullDIType(), // VTableHolder
-                                   uniqueIdent(t)); // UniqueIdentifier
+                                   0,                  // RunTimeLang
+                                   nullptr,            // VTableHolder
+                                   uniqueIdent(type)); // UniqueIdentifier
 }
 
-DIType DIBuilder::CreateSArrayType(Type *type) {
+DIType DIBuilder::CreateSArrayType(TypeSArray *type) {
   llvm::Type *T = DtoType(type);
-  Type *t = type->toBasetype();
-  assert(t->ty == Tsarray);
 
-  // find base type
+  Type *te = type;
   llvm::SmallVector<LLMetadata *, 8> subscripts;
-  while (t->ty == Tsarray) {
-    TypeSArray *tsa = static_cast<TypeSArray *>(t);
+  for (; te->ty == TY::Tsarray; te = te->nextOf()) {
+    TypeSArray *tsa = static_cast<TypeSArray *>(te);
     const auto count = tsa->dim->toInteger();
 #if LDC_LLVM_VER >= 1100
     const auto Count = llvm::ConstantAsMetadata::get(DtoConstSize_t(count));
@@ -695,53 +668,41 @@ DIType DIBuilder::CreateSArrayType(Type *type) {
     const auto subscript = DBuilder.getOrCreateSubrange(0, count);
 #endif
     subscripts.push_back(subscript);
-    t = t->nextOf();
   }
 
-  // element type: void => byte, function => function pointer
-  t = t->toBasetype();
-  if (t->ty == Tvoid)
-    t = Type::tuns8;
-  else if (t->ty == Tfunction)
-    t = t->pointerTo();
-
   return DBuilder.createArrayType(
       getTypeAllocSize(T) * 8,              // size (bits)
       getABITypeAlign(T) * 8,               // align (bits)
-      CreateTypeDescription(t),             // element type
+      CreateTypeDescription(te, /*voidToUbyte=*/true),
       DBuilder.getOrCreateArray(subscripts) // subscripts
   );
 }
 
-DIType DIBuilder::CreateAArrayType(Type *type) {
+DIType DIBuilder::CreateAArrayType(TypeAArray *type) {
   llvm::Type *T = DtoType(type);
-  Type *t = type->toBasetype();
-  assert(t->ty == Taarray);
 
-  TypeAArray *typeAArray = static_cast<TypeAArray *>(t);
-
-  Type *index = typeAArray->index;
-  Type *value = typeAArray->nextOf();
+  auto tindex = CreateTypeDescription(type->index);
+  auto tvalue = CreateTypeDescription(type->nextOf());
 
   const auto scope = GetCU();
   const auto name = processDIName(type->toPrettyChars(true));
   const auto file = CreateFile();
 
   LLMetadata *elems[] = {
-      CreateTypedef(0, index, file, "__key_t"),
-      CreateTypedef(0, value, file, "__val_t"),
-      CreateMemberType(0, Type::tvoidptr, file, "ptr", 0, Prot::public_)};
+      DBuilder.createTypedef(tindex, "__key_t", file, 0, scope),
+      DBuilder.createTypedef(tvalue, "__val_t", file, 0, scope),
+      CreateMemberType(0, Type::tvoidptr, file, "ptr", 0, Visibility::public_)};
 
   return DBuilder.createStructType(scope, name, file,
                                    0,                       // LineNo
                                    getTypeAllocSize(T) * 8, // size in bits
                                    getABITypeAlign(T) * 8,  // alignment in bits
                                    DIFlags::FlagZero,       // What here?
-                                   getNullDIType(),         // derived from
+                                   nullptr,                 // derived from
                                    DBuilder.getOrCreateArray(elems),
-                                   0,               // RunTimeLang
-                                   getNullDIType(), // VTableHolder
-                                   uniqueIdent(t)); // UniqueIdentifier
+                                   0,                  // RunTimeLang
+                                   nullptr,            // VTableHolder
+                                   uniqueIdent(type)); // UniqueIdentifier
 }
 
 ////////////////////////////////////////////////////////////////////////////////
@@ -750,9 +711,9 @@ DIType DIBuilder::CreateAArrayType(Type *type) {
 const unsigned DW_CC_D_dmd = 0x43;
 
 DISubroutineType DIBuilder::CreateFunctionType(Type *type) {
-  assert(type->toBasetype()->ty == Tfunction);
+  TypeFunction *t = type->isTypeFunction();
+  assert(t);
 
-  TypeFunction *t = static_cast<TypeFunction *>(type);
   Type *retType = t->next;
 
   // Create "dummy" subroutine type for the return type
@@ -761,9 +722,8 @@ DISubroutineType DIBuilder::CreateFunctionType(Type *type) {
 
   // The calling convention has to be recorded to distinguish
   // extern(D) functions from extern(C++) ones.
-  DtoType(t);
-  assert(t->ctype);
-  unsigned CC = t->ctype->getIrFuncTy().reverseParams ? DW_CC_D_dmd : 0;
+  unsigned CC =
+      getIrType(t, true)->getIrFuncTy().reverseParams ? DW_CC_D_dmd : 0;
 
   return DBuilder.createSubroutineType(paramsArray, DIFlags::FlagZero, CC);
 }
@@ -773,11 +733,7 @@ DISubroutineType DIBuilder::CreateEmptyFunctionType() {
   return DBuilder.createSubroutineType(paramsArray);
 }
 
-DIType DIBuilder::CreateDelegateType(Type *type) {
-  Type *const tb = type->toBasetype();
-  assert(tb->ty == Tdelegate);
-  auto t = static_cast<TypeDelegate *>(tb);
-
+DIType DIBuilder::CreateDelegateType(TypeDelegate *type) {
   llvm::Type *T = DtoType(type);
 
   const auto scope = GetCU();
@@ -785,87 +741,76 @@ DIType DIBuilder::CreateDelegateType(Type *type) {
   const auto file = CreateFile();
 
   LLMetadata *elems[] = {
-      CreateMemberType(0, Type::tvoidptr, file, "context", 0, Prot::public_),
-      CreateMemberType(0, t->next, file, "funcptr",
-                       global.params.is64bit ? 8 : 4, Prot::public_)};
+      CreateMemberType(0, Type::tvoidptr, file, "context", 0,
+                       Visibility::public_),
+      CreateMemberType(0, type->next->pointerTo(), file, "funcptr",
+                       target.ptrsize, Visibility::public_)};
 
   return DBuilder.createStructType(scope, name, file,
                                    0, // line number where defined
                                    getTypeAllocSize(T) * 8, // size in bits
                                    getABITypeAlign(T) * 8,  // alignment in bits
                                    DIFlags::FlagZero,       // flags
-                                   getNullDIType(),         // derived from
+                                   nullptr,                 // derived from
                                    DBuilder.getOrCreateArray(elems),
-                                   0,               // RunTimeLang
-                                   getNullDIType(), // VTableHolder
+                                   0,                  // RunTimeLang
+                                   nullptr,            // VTableHolder
                                    uniqueIdent(type)); // UniqueIdentifier
 }
 
-////////////////////////////////////////////////////////////////////////////////
-bool isOpaqueEnumType(Type *type) {
-  if (type->ty != Tenum)
-    return false;
-
-  TypeEnum *te = static_cast<TypeEnum *>(type);
-  return !te->sym->memtype;
+DIType DIBuilder::CreateUnspecifiedType(Dsymbol *sym) {
+  return DBuilder.createUnspecifiedType(
+      processDIName(sym->toPrettyChars(true)));
 }
 
-DIType DIBuilder::CreateTypeDescription(Type *type) {
-  // Check for opaque enum first, Bugzilla 13792
-  if (isOpaqueEnumType(type)) {
-    const auto ed = static_cast<TypeEnum *>(type)->sym;
-    return DBuilder.createUnspecifiedType(ed->toChars());
-  }
+////////////////////////////////////////////////////////////////////////////////
 
-  Type *t = type->toBasetype();
+DIType DIBuilder::CreateTypeDescription(Type *t, bool voidToUbyte) {
+  if (voidToUbyte && t->toBasetype()->ty == TY::Tvoid)
+    t = Type::tuns8;
 
-  if (t->ty == Tvoid)
+  if (t->ty == TY::Tvoid || t->ty == TY::Tnoreturn)
     return nullptr;
-  if (t->ty == Tnull) {
+  if (t->ty == TY::Tnull) {
     // display null as void*
     return DBuilder.createPointerType(CreateTypeDescription(Type::tvoid), 8, 8,
                                       /* DWARFAddressSpace */ llvm::None,
                                       "typeof(null)");
   }
-  if (t->ty == Tvector)
-    return CreateVectorType(type);
-  if (t->isintegral() || t->isfloating()) {
-    if (type->ty == Tenum)
-      return CreateEnumType(type);
-    return CreateBasicType(type);
-  }
-  if (t->ty == Tpointer)
-    return CreatePointerType(type);
-  if (t->ty == Tarray)
-    return CreateArrayType(type);
-  if (t->ty == Tsarray)
-    return CreateSArrayType(type);
-  if (t->ty == Taarray)
-    return CreateAArrayType(type);
-  if (t->ty == Tstruct)
-    return CreateCompositeType(type);
-  if (t->ty == Tclass) {
+  if (auto te = t->isTypeEnum())
+    return CreateEnumType(te);
+  if (auto tv = t->isTypeVector())
+    return CreateVectorType(tv);
+  if (t->isintegral() || t->isfloating())
+    return CreateBasicType(t);
+  if (auto tp = t->isTypePointer())
+    return CreatePointerType(tp);
+  if (auto ta = t->isTypeDArray())
+    return CreateArrayType(ta);
+  if (auto tsa = t->isTypeSArray())
+    return CreateSArrayType(tsa);
+  if (auto taa = t->isTypeAArray())
+    return CreateAArrayType(taa);
+  if (t->ty == TY::Tstruct)
+    return CreateCompositeType(t);
+  if (auto tc = t->isTypeClass()) {
     LLType *T = DtoType(t);
-    const auto aggregateDIType = CreateCompositeType(type);
-    const auto name = (aggregateDIType->getName() + "*").str();
+    const auto aggregateDIType = CreateCompositeType(t);
+    const auto name =
+        (tc->sym->toPrettyChars(true) + llvm::StringRef("*")).str();
     return DBuilder.createPointerType(aggregateDIType, getTypeAllocSize(T) * 8,
-                                      getABITypeAlign(T) * 8, llvm::None, name);
+                                      getABITypeAlign(T) * 8, llvm::None,
+                                      processDIName(name));
   }
-  if (t->ty == Tfunction)
-    return CreateFunctionType(type);
-  if (t->ty == Tdelegate)
-    return CreateDelegateType(type);
+  if (auto tf = t->isTypeFunction())
+    return CreateFunctionType(tf);
+  if (auto td = t->isTypeDelegate())
+    return CreateDelegateType(td);
 
   // Crash if the type is not supported.
   llvm_unreachable("Unsupported type in debug info");
 }
 
-DICompositeType DIBuilder::CreateCompositeTypeDescription(Type *type) {
-  DIType ret = type->toBasetype()->ty == Tclass ? CreateCompositeType(type)
-                                                : CreateTypeDescription(type);
-  return llvm::cast<llvm::DICompositeType>(ret);
-}
-
 ////////////////////////////////////////////////////////////////////////////////
 
 llvm::DICompileUnit::DebugEmissionKind getDebugEmissionKind() {
@@ -892,10 +837,6 @@ void DIBuilder::EmitCompileUnit(Module *m) {
 
   assert(!CUNode && "Already created compile unit for this DIBuilder instance");
 
-  // prepare srcpath
-  llvm::SmallString<128> srcpath(m->srcfile.toChars());
-  llvm::sys::fs::make_absolute(srcpath);
-
   // prepare producer name string
   auto producerName =
       std::string("LDC ") + ldc_version + " (LLVM " + llvm_version + ")";
@@ -923,9 +864,7 @@ void DIBuilder::EmitCompileUnit(Module *m) {
   CUNode = DBuilder.createCompileUnit(
       global.params.symdebug == 2 ? llvm::dwarf::DW_LANG_C_plus_plus
                                   : llvm::dwarf::DW_LANG_D,
-      DBuilder.createFile(llvm::sys::path::filename(srcpath),
-                          llvm::sys::path::parent_path(srcpath)),
-      producerName,
+      CreateFile(m->srcfile.toChars()), producerName,
       isOptimizationEnabled(), // isOptimized
       llvm::StringRef(),       // Flags TODO
       1,                       // Runtime Version TODO
@@ -947,7 +886,7 @@ DIModule DIBuilder::EmitModule(Module *m) {
   const auto name = processDIName(m->toPrettyChars(true));
 
   irm->diModule = DBuilder.createModule(
-      CUNode,
+      CreateFile(m->srcfile.toChars()),
       name,              // qualified module name
       llvm::StringRef(), // (clang modules specific) ConfigurationMacros
       llvm::StringRef(), // (clang modules specific) IncludePath
@@ -998,7 +937,7 @@ DISubprogram DIBuilder::EmitSubProgram(FuncDeclaration *fd) {
   if (emitCodeView && fd->toParent2()->isFuncDeclaration()) {
     // emit into module & use fully qualified name
     scope = GetCU();
-    name = fd->toPrettyChars(true);
+    name = processDIName(fd->toPrettyChars(true));
   } else if (fd->isMain()) {
     scope = GetSymbolScope(fd);
     name = fd->toPrettyChars(true); // `D main`
@@ -1009,7 +948,7 @@ DISubprogram DIBuilder::EmitSubProgram(FuncDeclaration *fd) {
   const auto linkageName = irFunc->getLLVMFuncName();
   const auto file = CreateFile(fd);
   const auto lineNo = fd->loc.linnum;
-  const auto isLocalToUnit = fd->protection.kind == Prot::private_;
+  const auto isLocalToUnit = fd->visibility.kind == Visibility::private_;
   const auto isDefinition = true;
   const auto scopeLine = lineNo; // FIXME
   const auto flags = DIFlags::FlagPrototyped;
@@ -1040,7 +979,7 @@ DISubprogram DIBuilder::EmitSubProgram(FuncDeclaration *fd) {
     );
 
     // Now create subroutine type.
-    diFnType = CreateFunctionType(static_cast<TypeFunction *>(fd->type));
+    diFnType = CreateFunctionType(fd->type);
   }
 
   // FIXME: duplicates?
@@ -1097,7 +1036,7 @@ DISubprogram DIBuilder::EmitThunk(llvm::Function *Thunk, FuncDeclaration *fd) {
   const auto linkageName = Thunk->getName();
   const auto file = CreateFile(fd);
   const auto lineNo = fd->loc.linnum;
-  const bool isLocalToUnit = fd->protection.kind == Prot::private_;
+  const bool isLocalToUnit = fd->visibility.kind == Visibility::private_;
   const bool isDefinition = true;
   const bool isOptimized = isOptimizationEnabled();
   const auto scopeLine = lineNo; // FIXME
@@ -1203,8 +1142,9 @@ void DIBuilder::EmitStopPoint(const Loc &loc) {
   unsigned col = getColumn(loc);
   Logger::println("D to dwarf stoppoint at line %u, column %u", linnum, col);
   LOG_SCOPE;
+
   IR->ir->SetCurrentDebugLocation(
-      llvm::DebugLoc::get(linnum, col, GetCurrentScope()));
+      llvm::DILocation::get(IR->context(), linnum, col, GetCurrentScope()));
 }
 
 void DIBuilder::EmitValue(llvm::Value *val, VarDeclaration *vd) {
@@ -1232,6 +1172,11 @@ void DIBuilder::EmitLocalVariable(llvm::Value *ll, VarDeclaration *vd,
   Logger::println("D to dwarf local variable");
   LOG_SCOPE;
 
+  if (vd->type->toBasetype()->isTypeNoreturn()) {
+    Logger::println("of type noreturn, skip");
+    return;
+  }
+
   auto &variableMap = IR->func()->variableMap;
   auto sub = variableMap.find(vd);
   if (sub != variableMap.end())
@@ -1255,9 +1200,9 @@ void DIBuilder::EmitLocalVariable(llvm::Value *ll, VarDeclaration *vd,
     if (isaArgument(ll) && addr.empty()) {
       forceAsLocal = true;
     } else {
-      // 2) dynamic arrays and vectors
+      // 2) dynamic arrays, delegates and vectors
       TY ty = type->toBasetype()->ty;
-      if (ty == Tarray || ty == Tvector)
+      if (ty == TY::Tarray || ty == TY::Tdelegate || ty == TY::Tvector)
         forceAsLocal = true;
     }
   }
@@ -1337,6 +1282,11 @@ void DIBuilder::EmitGlobalVariable(llvm::GlobalVariable *llVar,
   Logger::println("D to dwarf global_variable");
   LOG_SCOPE;
 
+  if (vd->type->toBasetype()->isTypeNoreturn()) {
+    Logger::println("of type noreturn, skip");
+    return;
+  }
+
   assert(vd->isDataseg() ||
          (vd->storage_class & (STCconst | STCimmutable) && vd->_init));
 
@@ -1359,7 +1309,7 @@ void DIBuilder::EmitGlobalVariable(llvm::GlobalVariable *llVar,
       CreateFile(vd),                        // file
       vd->loc.linnum,                        // line num
       CreateTypeDescription(vd->type),       // type
-      vd->protection.kind == Prot::private_, // is local to unit
+      vd->visibility.kind == Visibility::private_, // is local to unit
 #if LDC_LLVM_VER >= 1000
       !(vd->storage_class & STCextern),      // bool isDefined
 #endif
