@@ -103,9 +103,10 @@ FuncGenState::FuncGenState(IrFunction &irFunc, IRState &irs)
     : irFunc(irFunc), scopes(irs), jumpTargets(scopes), switchTargets(),
       irs(irs) {}
 
-llvm::CallSite FuncGenState::callOrInvoke(llvm::Value *callee,
-                                          llvm::ArrayRef<llvm::Value *> args,
-                                          const char *name, bool isNothrow) {
+LLCallBasePtr FuncGenState::callOrInvoke(llvm::Value *callee,
+                                         llvm::FunctionType *calleeType,
+                                         llvm::ArrayRef<llvm::Value *> args,
+                                         const char *name, bool isNothrow) {
   // If this is a direct call, we might be able to use the callee attributes
   // to our advantage.
   llvm::Function *calleeFn = llvm::dyn_cast<llvm::Function>(callee);
@@ -117,14 +118,20 @@ llvm::CallSite FuncGenState::callOrInvoke(llvm::Value *callee,
 
   // Intrinsics don't support invoking and 'nounwind' functions don't need it.
   const bool doesNotThrow =
-      isNothrow ||
+      isNothrow || global.params.betterC ||
       (calleeFn && (calleeFn->isIntrinsic() || calleeFn->doesNotThrow()));
 
   // calls inside a funclet must be annotated with its value
   llvm::SmallVector<llvm::OperandBundleDef, 2> BundleList;
 
+#if LDC_LLVM_VER >= 1100
+  llvm::FunctionCallee calleeArg(calleeType, callee);
+#else
+  auto calleeArg = callee;
+#endif
+
   if (doesNotThrow || scopes.empty()) {
-    llvm::CallInst *call = irs.ir->CreateCall(callee, args, BundleList, name);
+    auto call = irs.ir->CreateCall(calleeArg, args, BundleList, name);
     if (calleeFn) {
       call->setAttributes(calleeFn->getAttributes());
     }
@@ -134,12 +141,12 @@ llvm::CallSite FuncGenState::callOrInvoke(llvm::Value *callee,
   llvm::BasicBlock *landingPad = scopes.getLandingPad();
 
   llvm::BasicBlock *postinvoke = irs.insertBB("postinvoke");
-  llvm::InvokeInst *invoke = irs.ir->CreateInvoke(
-      callee, postinvoke, landingPad, args, BundleList, name);
+  auto invoke = irs.ir->CreateInvoke(calleeArg, postinvoke, landingPad, args,
+                                     BundleList, name);
   if (calleeFn) {
     invoke->setAttributes(calleeFn->getAttributes());
   }
 
-  irs.scope() = IRScope(postinvoke);
+  irs.ir->SetInsertPoint(postinvoke);
   return invoke;
 }
