@@ -37,8 +37,13 @@ struct TempDisableDiscardValueNames {
 /// Note: don't add function _parameter_ attributes
 void copyFnAttributes(llvm::Function *wannabe, llvm::Function *idol) {
   auto attrSet = idol->getAttributes();
+#if LDC_LLVM_VER >= 1400
+  auto fnAttrSet = attrSet.getFnAttrs();
+  wannabe->addFnAttrs(fnAttrSet);
+#else
   auto fnAttrSet = attrSet.getFnAttributes();
   wannabe->addAttributes(LLAttributeList::FunctionIndex, fnAttrSet);
+#endif
 }
 
 llvm::StringRef exprToString(StringExp *strexp) {
@@ -99,7 +104,7 @@ void DtoCheckInlineIRPragma(Identifier *ident, Dsymbol *s) {
   }
 }
 
-DValue *DtoInlineIRExpr(Loc &loc, FuncDeclaration *fdecl,
+DValue *DtoInlineIRExpr(const Loc &loc, FuncDeclaration *fdecl,
                         Expressions *arguments, LLValue *sretPointer) {
   IF_LOG Logger::println("DtoInlineIRExpr @ %s", loc.toChars());
   LOG_SCOPE;
@@ -177,7 +182,7 @@ DValue *DtoInlineIRExpr(Loc &loc, FuncDeclaration *fdecl,
     }
 
     stream << ")\n{\n" << code;
-    if (ret->ty == Tvoid) {
+    if (ret->ty == TY::Tvoid) {
       stream << "\nret void";
     }
     stream << "\n}";
