@@ -128,13 +128,15 @@ bool IRState::emitArrayBoundsChecks() {
   }
 
   Type *t = func()->decl->type;
-  return t->ty == Tfunction && ((TypeFunction *)t)->trust == TRUST::safe;
+  return t->ty == TY::Tfunction && ((TypeFunction *)t)->trust == TRUST::safe;
 }
 
-LLConstant *IRState::setGlobalVarInitializer(LLGlobalVariable *&globalVar,
-                                             LLConstant *initializer) {
+LLConstant *
+IRState::setGlobalVarInitializer(LLGlobalVariable *&globalVar,
+                                 LLConstant *initializer,
+                                 Dsymbol *symbolForLinkageAndVisibility) {
   if (initializer->getType() == globalVar->getType()->getContainedType(0)) {
-    globalVar->setInitializer(initializer);
+    defineGlobal(globalVar, initializer, symbolForLinkageAndVisibility);
     return globalVar;
   }
 
@@ -142,7 +144,7 @@ LLConstant *IRState::setGlobalVarInitializer(LLGlobalVariable *&globalVar,
   // It inherits most properties from the existing globalVar.
   auto globalHelperVar = new LLGlobalVariable(
       module, initializer->getType(), globalVar->isConstant(),
-      globalVar->getLinkage(), initializer, "", nullptr,
+      globalVar->getLinkage(), nullptr, "", nullptr,
       globalVar->getThreadLocalMode());
   globalHelperVar->setAlignment(LLMaybeAlign(globalVar->getAlignment()));
   globalHelperVar->setComdat(globalVar->getComdat());
@@ -150,6 +152,8 @@ LLConstant *IRState::setGlobalVarInitializer(LLGlobalVariable *&globalVar,
   globalHelperVar->setSection(globalVar->getSection());
   globalHelperVar->takeName(globalVar);
 
+  defineGlobal(globalHelperVar, initializer, symbolForLinkageAndVisibility);
+
   // Replace all existing uses of globalVar by the bitcast pointer.
   auto castHelperVar = DtoBitCast(globalHelperVar, globalVar->getType());
   globalVar->replaceAllUsesWith(castHelperVar);
