@@ -147,8 +147,8 @@ public:
   ObjCState objc;
 
   // Stack of currently codegen'd functions (more than one for lambdas or other
-  // nested functions, inlining-only codegen'ing, etc.), and some convenience
-  // accessors for the top-most one.
+  // nested functions, inlining-only codegen'ing, -linkonce-templates etc.),
+  // and some convenience accessors for the top-most one.
   std::vector<std::unique_ptr<FuncGenState>> funcGenStates;
   FuncGenState &funcGen();
   IrFunction *func();
@@ -229,8 +229,10 @@ public:
   // Returns either the specified globalVar if the types match, or the bitcast
   // pointer replacing globalVar (and resets globalVar to the new helper
   // global).
-  llvm::Constant *setGlobalVarInitializer(llvm::GlobalVariable *&globalVar,
-                                          llvm::Constant *initializer);
+  llvm::Constant *
+  setGlobalVarInitializer(llvm::GlobalVariable *&globalVar,
+                          llvm::Constant *initializer,
+                          Dsymbol *symbolForLinkageAndVisibility);
 
   // To be called when finalizing the IR module in order to perform a second
   // replacement pass for global variables replaced (and registered) by
@@ -270,7 +272,7 @@ public:
 
   // MS C++ compatible type descriptors
   llvm::DenseMap<size_t, llvm::StructType *> TypeDescriptorTypeMap;
-  llvm::DenseMap<llvm::Constant *, llvm::GlobalVariable *> TypeDescriptorMap;
+  llvm::DenseMap<ClassDeclaration *, llvm::GlobalVariable *> TypeDescriptorMap;
 
   // Target for dcompute. If not nullptr, it owns this.
   DComputeTarget *dcomputetarget = nullptr;
