@@ -59,11 +59,11 @@ llvm::Function *buildForwarderFunction(
   const auto fnTy =
       LLFunctionType::get(LLType::getVoidTy(gIR->context()), {}, false);
 
-  const auto irMangle = getIRMangledFuncName(name, LINKd);
+  const auto irMangle = getIRMangledFuncName(name, LINK::d);
   assert(gIR->module.getFunction(irMangle) == NULL);
   llvm::Function *fn = llvm::Function::Create(
       fnTy, llvm::GlobalValue::InternalLinkage, irMangle, &gIR->module);
-  fn->setCallingConv(gABI->callingConv(LINKd));
+  fn->setCallingConv(gABI->callingConv(LINK::d));
 
   // Emit the body, consisting of...
   const auto bb = llvm::BasicBlock::Create(gIR->context(), "", fn);
@@ -72,7 +72,8 @@ llvm::Function *buildForwarderFunction(
   ldc::DISubprogram dis = gIR->DBuilder.EmitModuleCTor(fn, name.c_str());
   if (global.params.symdebug) {
     // Need _some_ debug info to avoid inliner bug, see GitHub issue #998.
-    builder.SetCurrentDebugLocation(llvm::DebugLoc::get(0, 0, dis));
+    builder.SetCurrentDebugLocation(
+        llvm::DILocation::get(gIR->context(), 0, 0, dis));
   }
 
   // ... calling the given functions, and...
@@ -174,7 +175,7 @@ llvm::Constant *buildLocalClasses(Module *m, size_t &count) {
       continue;
     }
 
-    if (cd->sizeok != SIZEOKdone) {
+    if (cd->sizeok != Sizeok::done) {
       IF_LOG Logger::println(
           "skipping opaque class declaration '%s' in moduleinfo",
           cd->toPrettyChars());
@@ -311,5 +312,8 @@ llvm::GlobalVariable *genModuleInfo(Module *m) {
   LLGlobalVariable *moduleInfoSym = getIrModule(m)->moduleInfoSymbol();
   b.finalize(moduleInfoSym);
   setLinkage({LLGlobalValue::ExternalLinkage, needsCOMDAT()}, moduleInfoSym);
+  if (global.params.dllexport) {
+    moduleInfoSym->setDLLStorageClass(LLGlobalValue::DLLExportStorageClass);
+  }
   return moduleInfoSym;
 }
