@@ -23,6 +23,7 @@
 #include "dmd/target.h"
 #include "dmd/template.h"
 #include "driver/cl_options_instrumentation.h"
+#include "driver/timetrace.h"
 #include "gen/abi.h"
 #include "gen/arrays.h"
 #include "gen/functions.h"
@@ -81,19 +82,6 @@ void Module::checkAndAddOutputFile(const FileName &file) {
   files.emplace(std::move(key), this);
 }
 
-void Module::makeObjectFilenameUnique() {
-  assert(objfile.toChars());
-
-  const char *ext = FileName::ext(objfile.toChars());
-  const char *stem = FileName::removeExt(objfile.toChars());
-
-  llvm::SmallString<128> unique;
-  auto EC = llvm::sys::fs::createUniqueFile(
-      llvm::Twine(stem) + "-%%%%%%%." + ext, unique);
-  if (!EC) // success
-    objfile.reset(unique.c_str());
-}
-
 namespace {
 /// Ways the druntime module registry system can be implemented.
 enum class RegistryStyle {
@@ -120,8 +108,8 @@ enum class RegistryStyle {
 RegistryStyle getModuleRegistryStyle() {
   const auto &t = *global.params.targetTriple;
 
-  if (t.isWindowsMSVCEnvironment() ||
-      t.getEnvironment() == llvm::Triple::Android || t.isOSBinFormatWasm()) {
+  if (t.isOSWindows() || t.getEnvironment() == llvm::Triple::Android ||
+      t.isOSBinFormatWasm()) {
     return RegistryStyle::sectionSimple;
   }
 
@@ -140,7 +128,7 @@ RegistryStyle getModuleRegistryStyle() {
 LLGlobalVariable *declareDSOGlobal(llvm::StringRef mangledName, LLType *type,
                                    bool isThreadLocal = false) {
   auto global = declareGlobal(Loc(), gIR->module, type, mangledName, false,
-                              isThreadLocal);
+                              isThreadLocal, false);
   global->setVisibility(LLGlobalValue::HiddenVisibility);
   return global;
 }
@@ -181,7 +169,7 @@ LLFunction *build_module_reference_and_ctor(const char *moduleMangle,
   // linked list
   LLFunction *ctor =
       LLFunction::Create(fty, LLGlobalValue::InternalLinkage,
-                         getIRMangledFuncName(fname, LINKd), &gIR->module);
+                         getIRMangledFuncName(fname, LINK::d), &gIR->module);
 
   // provide the default initializer
   LLStructType *modulerefTy = DtoModuleReferenceType();
@@ -198,11 +186,13 @@ LLFunction *build_module_reference_and_ctor(const char *moduleMangle,
       defineGlobal(Loc(), gIR->module, thismrefIRMangle, thismrefinit,
                    LLGlobalValue::InternalLinkage, false);
   // make sure _Dmodule_ref is declared
-  const auto mrefIRMangle = getIRMangledVarName("_Dmodule_ref", LINKc);
+  const auto mrefIRMangle = getIRMangledVarName("_Dmodule_ref", LINK::c);
   LLConstant *mref = gIR->module.getNamedGlobal(mrefIRMangle);
   LLType *modulerefPtrTy = getPtrToType(modulerefTy);
   if (!mref) {
-    mref = declareGlobal(Loc(), gIR->module, modulerefPtrTy, mrefIRMangle, false);
+    mref =
+        declareGlobal(Loc(), gIR->module, modulerefPtrTy, mrefIRMangle, false,
+                      false, global.params.dllimport != DLLImport::none);
   }
   mref = DtoBitCast(mref, getPtrToType(modulerefPtrTy));
 
@@ -475,8 +465,8 @@ void addCoverageAnalysis(Module *m) {
         LLFunctionType::get(LLType::getVoidTy(gIR->context()), {}, false);
     ctor =
         LLFunction::Create(ctorTy, LLGlobalValue::InternalLinkage,
-                           getIRMangledFuncName(ctorname, LINKd), &gIR->module);
-    ctor->setCallingConv(gABI->callingConv(LINKd));
+                           getIRMangledFuncName(ctorname, LINK::d), &gIR->module);
+    ctor->setCallingConv(gABI->callingConv(LINK::d));
     // Set function attributes. See functions.cpp:DtoDefineFunction()
     if (global.params.targetTriple->getArch() == llvm::Triple::x86_64) {
       ctor->addFnAttr(LLAttribute::UWTable);
@@ -585,6 +575,8 @@ void registerModuleInfo(Module *m) {
 }
 
 void codegenModule(IRState *irs, Module *m) {
+  TimeTraceScope timeScope("Generate IR", m->toChars(), m->loc);
+
   assert(!irs->dmodule &&
          "irs->module not null, codegen already in progress?!");
   irs->dmodule = m;
@@ -608,7 +600,7 @@ void codegenModule(IRState *irs, Module *m) {
 
   // process module members
   // NOTE: m->members may grow during codegen
-  for (unsigned k = 0; k < m->members->length; k++) {
+  for (d_size_t k = 0; k < m->members->length; k++) {
     Dsymbol *dsym = (*m->members)[k];
     assert(dsym);
     Declaration_codegen(dsym);
