@@ -142,8 +142,6 @@ void DtoDefineNakedFunction(FuncDeclaration *fd) {
   IF_LOG Logger::println("DtoDefineNakedFunction(%s)", mangleExact(fd));
   LOG_SCOPE;
 
-  gIR->funcGenStates.emplace_back(new FuncGenState(*getIrFunc(fd), *gIR));
-
   // we need to do special processing on the body, since we only want
   // to allow actual inline asm blocks to reach the final asm output
 
@@ -170,7 +168,7 @@ void DtoDefineNakedFunction(FuncDeclaration *fd) {
     asmstr << "\t.section\t__TEXT,__text,regular,pure_instructions"
            << std::endl;
     asmstr << "\t.globl\t" << mangle << std::endl;
-    if (DtoIsTemplateInstance(fd)) {
+    if (fd->isInstantiated()) {
       asmstr << "\t.weak_definition\t" << mangle << std::endl;
     }
     asmstr << "\t.p2align\t4, 0x90" << std::endl;
@@ -198,7 +196,7 @@ void DtoDefineNakedFunction(FuncDeclaration *fd) {
     asmstr << "\t.type\t32;" << std::endl;
     asmstr << "\t.endef" << std::endl;
 
-    if (DtoIsTemplateInstance(fd)) {
+    if (fd->isInstantiated()) {
       asmstr << "\t.section\t.text,\"xr\",discard," << mangle << std::endl;
     } else {
       asmstr << "\t.text" << std::endl;
@@ -207,7 +205,7 @@ void DtoDefineNakedFunction(FuncDeclaration *fd) {
     asmstr << "\t.p2align\t4, 0x90" << std::endl;
     asmstr << mangle << ":" << std::endl;
   } else {
-    if (DtoIsTemplateInstance(fd)) {
+    if (fd->isInstantiated()) {
       asmstr << "\t.section\t.text." << mangle << ",\"axG\",@progbits,"
              << mangle << ",comdat" << std::endl;
       asmstr << "\t.weak\t" << mangle << std::endl;
@@ -240,20 +238,18 @@ void DtoDefineNakedFunction(FuncDeclaration *fd) {
   gIR->module.appendModuleInlineAsm(asmstr.str());
   asmstr.str("");
 
-  if (global.params.targetTriple->isWindowsMSVCEnvironment() &&
-      fd->isExport()) {
+  if (global.params.dllexport ||
+      (global.params.targetTriple->isOSWindows() && fd->isExport())) {
     // Embed a linker switch telling the MS linker to export the naked function.
     // This mimics the effect of the dllexport attribute for regular functions.
     const auto linkerSwitch = std::string("/EXPORT:") + mangle;
     gIR->addLinkerOption(llvm::StringRef(linkerSwitch));
   }
-
-  gIR->funcGenStates.pop_back();
 }
 
 ////////////////////////////////////////////////////////////////////////////////
 
-void emitABIReturnAsmStmt(IRAsmBlock *asmblock, Loc &loc,
+void emitABIReturnAsmStmt(IRAsmBlock *asmblock, const Loc &loc,
                           FuncDeclaration *fdecl) {
   IF_LOG Logger::println("emitABIReturnAsmStmt(%s)", mangleExact(fdecl));
   LOG_SCOPE;
@@ -272,8 +268,8 @@ void emitABIReturnAsmStmt(IRAsmBlock *asmblock, Loc &loc,
 
   // x86
   if (triple.getArch() == llvm::Triple::x86) {
-    if (rt->isintegral() || rt->ty == Tpointer || rt->ty == Tclass ||
-        rt->ty == Taarray) {
+    if (rt->isintegral() || rt->ty == TY::Tpointer || rt->ty == TY::Tclass ||
+        rt->ty == TY::Taarray) {
       if (rt->size() == 8) {
         as->out_c = "=A,";
       } else {
@@ -281,11 +277,11 @@ void emitABIReturnAsmStmt(IRAsmBlock *asmblock, Loc &loc,
       }
     } else if (rt->isfloating()) {
       if (rt->iscomplex()) {
-        if (fdecl->linkage == LINKd) {
+        if (fdecl->linkage == LINK::d) {
           // extern(D) always returns on the FPU stack
           as->out_c = "={st},={st(1)},";
           asmblock->retn = 2;
-        } else if (rt->ty == Tcomplex32) {
+        } else if (rt->ty == TY::Tcomplex32) {
           // non-extern(D) cfloat is returned as i64
           as->out_c = "=A,";
           asmblock->retty = LLType::getInt64Ty(gIR->context());
@@ -299,7 +295,7 @@ void emitABIReturnAsmStmt(IRAsmBlock *asmblock, Loc &loc,
       } else {
         as->out_c = "={st},";
       }
-    } else if (rt->ty == Tarray || rt->ty == Tdelegate) {
+    } else if (rt->ty == TY::Tarray || rt->ty == TY::Tdelegate) {
       as->out_c = "={ax},={dx},";
       asmblock->retn = 2;
 #if 0
@@ -336,8 +332,8 @@ void emitABIReturnAsmStmt(IRAsmBlock *asmblock, Loc &loc,
 
   // x86_64
   else if (triple.getArch() == llvm::Triple::x86_64) {
-    if (rt->isintegral() || rt->ty == Tpointer || rt->ty == Tclass ||
-        rt->ty == Taarray) {
+    if (rt->isintegral() || rt->ty == TY::Tpointer || rt->ty == TY::Tclass ||
+        rt->ty == TY::Taarray) {
       as->out_c = "={ax},";
     } else if (rt->isfloating()) {
       const bool isWin64 = triple.isOSWindows();
@@ -376,7 +372,7 @@ void emitABIReturnAsmStmt(IRAsmBlock *asmblock, Loc &loc,
         // Plain float/double/ifloat/idouble
         as->out_c = "={xmm0},";
       }
-    } else if (rt->ty == Tarray || rt->ty == Tdelegate) {
+    } else if (rt->ty == TY::Tarray || rt->ty == TY::Tdelegate) {
       as->out_c = "={ax},={dx},";
       asmblock->retn = 2;
     } else {
@@ -403,8 +399,8 @@ void emitABIReturnAsmStmt(IRAsmBlock *asmblock, Loc &loc,
 
 // sort of kinda related to naked ...
 
-DValue *DtoInlineAsmExpr(Loc &loc, FuncDeclaration *fd, Expressions *arguments,
-                         LLValue *sretPointer) {
+DValue *DtoInlineAsmExpr(const Loc &loc, FuncDeclaration *fd,
+                         Expressions *arguments, LLValue *sretPointer) {
   assert(fd->toParent()->isTemplateInstance() && "invalid inline __asm expr");
   assert(arguments->length >= 2 && "invalid __asm call");
 
@@ -446,7 +442,7 @@ DValue *DtoInlineAsmExpr(Loc &loc, FuncDeclaration *fd, Expressions *arguments,
       DtoInlineAsmExpr(loc, code, constraints, operands, irReturnType);
 
   // work around missing tuple support for users of the return value
-  if (sretPointer || returnType->ty == Tstruct) {
+  if (sretPointer || returnType->ty == TY::Tstruct) {
     auto lvalue = sretPointer;
     if (!lvalue)
       lvalue = DtoAlloca(returnType, ".__asm_tuple_ret");
