@@ -10,7 +10,6 @@
 #include "gen/nested.h"
 
 #include "dmd/errors.h"
-#include "dmd/ldcbindings.h"
 #include "dmd/target.h"
 #include "gen/dvalue.h"
 #include "gen/funcgenstate.h"
@@ -29,7 +28,7 @@ static unsigned getVthisIdx(AggregateDeclaration *ad) {
 
 static void DtoCreateNestedContextType(FuncDeclaration *fd);
 
-DValue *DtoNestedVariable(Loc &loc, Type *astype, VarDeclaration *vd,
+DValue *DtoNestedVariable(const Loc &loc, Type *astype, VarDeclaration *vd,
                           bool byref) {
   IF_LOG Logger::println("DtoNestedVariable for %s @ %s", vd->toChars(),
                          loc.toChars());
@@ -178,7 +177,7 @@ DValue *DtoNestedVariable(Loc &loc, Type *astype, VarDeclaration *vd,
   return makeVarDValue(astype, vd, val);
 }
 
-void DtoResolveNestedContext(Loc &loc, AggregateDeclaration *decl,
+void DtoResolveNestedContext(const Loc &loc, AggregateDeclaration *decl,
                              LLValue *value) {
   IF_LOG Logger::println("Resolving nested context");
   LOG_SCOPE;
@@ -202,7 +201,7 @@ void DtoResolveNestedContext(Loc &loc, AggregateDeclaration *decl,
   }
 }
 
-LLValue *DtoNestedContext(Loc &loc, Dsymbol *sym) {
+LLValue *DtoNestedContext(const Loc &loc, Dsymbol *sym) {
   IF_LOG Logger::println("DtoNestedContext for %s", sym->toPrettyChars());
   LOG_SCOPE;
 
@@ -316,6 +315,12 @@ static void DtoCreateNestedContextType(FuncDeclaration *fd) {
                          fd->toPrettyChars());
   LOG_SCOPE
 
+  FuncDeclaration *parentFunc = getParentFunc(fd);
+  // Make sure the parent has already been analyzed.
+  if (parentFunc) {
+    DtoCreateNestedContextType(parentFunc);
+  }
+
   DtoDeclareFunction(fd);
 
   IrFunction &irFunc = *getIrFunc(fd);
@@ -326,12 +331,6 @@ static void DtoCreateNestedContextType(FuncDeclaration *fd) {
   }
   irFunc.nestedContextCreated = true;
 
-  FuncDeclaration *parentFunc = getParentFunc(fd);
-  // Make sure the parent has already been analyzed.
-  if (parentFunc) {
-    DtoCreateNestedContextType(parentFunc);
-  }
-
   if (fd->closureVars.length == 0) {
     // No local variables of this function are captured.
     if (parentFunc) {
@@ -401,9 +400,9 @@ static void DtoCreateNestedContextType(FuncDeclaration *fd) {
       t = DtoType(vd->type->pointerTo());
     } else if (isParam && (vd->storage_class & STClazy)) {
       // the type is a delegate (LL struct)
-      Type *dt = TypeFunction::create(nullptr, vd->type, VARARGnone, LINKd);
-      dt = createTypeDelegate(dt);
-      t = DtoType(dt);
+      auto tf = TypeFunction::create(nullptr, vd->type, VARARGnone, LINK::d);
+      auto td = TypeDelegate::create(tf);
+      t = DtoType(td);
     } else {
       t = DtoMemType(vd->type);
     }
