@@ -290,17 +290,8 @@ public:
     // with destructors are ignored for now.
     auto hasDestructor =
         mdconst::dyn_extract<Constant>(node->getOperand(CD_Finalize));
-    // We can't stack-allocate if the class has a custom deallocator
-    // (Custom allocators don't get turned into this runtime call, so
-    // those can be ignored)
-    auto hasCustomDelete =
-        mdconst::dyn_extract<Constant>(node->getOperand(CD_CustomDelete));
-    if (hasDestructor == nullptr || hasCustomDelete == nullptr) {
-      return false;
-    }
-
-    if (ConstantExpr::getOr(hasDestructor, hasCustomDelete) !=
-        ConstantInt::getFalse(A.M.getContext())) {
+    if (hasDestructor == nullptr ||
+        hasDestructor != ConstantInt::getFalse(A.M.getContext())) {
       return false;
     }
 
@@ -442,6 +433,8 @@ static void RemoveCall(LLCallBasePtr CB, const Analysis &A) {
   if (A.CGNode) {
 #if LDC_LLVM_VER >= 900
     A.CGNode->removeCallEdgeFor(*CB);
+#elif LDC_LLVM_VER >= 800
+    A.CGNode->removeCallEdgeFor(llvm::CallSite(CB));
 #else
     A.CGNode->removeCallEdgeFor(CB);
 #endif
@@ -569,28 +562,12 @@ llvm::Type *Analysis::getTypeFor(Value *typeinfo) const {
   const auto metaname = getMetadataName(TD_PREFIX, ti_global);
 
   NamedMDNode *meta = M.getNamedMetadata(metaname);
-  if (!meta) {
+  if (!meta || meta->getNumOperands() != 1) {
     return nullptr;
   }
 
-  MDNode *node = static_cast<MDNode *>(meta->getOperand(0));
-  if (!node) {
-    return nullptr;
-  }
-
-  if (node->getNumOperands() != TD_NumFields) {
-    return nullptr;
-  }
-
-  auto md = llvm::dyn_cast<llvm::ValueAsMetadata>(
-              node->getOperand(TD_TypeInfo).get());
-  if (md == nullptr || md->getValue()->stripPointerCasts() != ti_global) {
-    return nullptr;
-  }
-
-
-  return llvm::cast<llvm::ValueAsMetadata>(node->getOperand(TD_Type))
-      ->getType();
+  MDNode *node = meta->getOperand(0);
+  return llvm::cast<llvm::ConstantAsMetadata>(node->getOperand(0))->getType();
 }
 
 /// Returns whether Def is used by any instruction that is reachable from Alloc
