@@ -24,6 +24,7 @@
 #include "gen/logger.h"
 #include "gen/tollvm.h"
 #include "ir/iraggr.h"
+#include "ir/irdsymbol.h"
 #include "llvm/ADT/DenseMap.h"
 #include "llvm/Support/ManagedStatic.h"
 #include <algorithm>
@@ -32,7 +33,7 @@
 
 void DtoResolveStruct(StructDeclaration *sd) { DtoResolveStruct(sd, sd->loc); }
 
-void DtoResolveStruct(StructDeclaration *sd, Loc &callerLoc) {
+void DtoResolveStruct(StructDeclaration *sd, const Loc &callerLoc) {
   // Make sure to resolve each struct type exactly once.
   if (sd->ir->isResolved()) {
     return;
@@ -47,7 +48,7 @@ void DtoResolveStruct(StructDeclaration *sd, Loc &callerLoc) {
   DtoType(sd->type);
 
   // if it's a forward declaration, all bets are off. The type should be enough
-  if (sd->sizeok != SIZEOKdone) {
+  if (sd->sizeok != Sizeok::done) {
     error(callerLoc, "struct `%s.%s` unknown size", sd->getModule()->toChars(),
           sd->toChars());
     fatal();
@@ -74,7 +75,7 @@ void DtoResolveStruct(StructDeclaration *sd, Loc &callerLoc) {
 
 LLValue *DtoStructEquals(TOK op, DValue *lhs, DValue *rhs) {
   Type *t = lhs->type->toBasetype();
-  assert(t->ty == Tstruct);
+  assert(t->ty == TY::Tstruct);
 
   // set predicate
   llvm::ICmpInst::Predicate cmpop;
@@ -102,7 +103,7 @@ LLValue *DtoStructEquals(TOK op, DValue *lhs, DValue *rhs) {
 /// specified type.
 /// Union types will get expanded into a struct, with a type for each member.
 LLType *DtoUnpaddedStructType(Type *dty) {
-  assert(dty->ty == Tstruct);
+  assert(dty->ty == TY::Tstruct);
 
   typedef llvm::DenseMap<Type *, llvm::StructType *> CacheT;
   static llvm::ManagedStatic<CacheT> cache;
@@ -119,7 +120,7 @@ LLType *DtoUnpaddedStructType(Type *dty) {
 
   for (unsigned i = 0; i < fields.length; i++) {
     LLType *fty;
-    if (fields[i]->type->ty == Tstruct) {
+    if (fields[i]->type->ty == TY::Tstruct) {
       // Nested structs are the only members that can contain padding
       fty = DtoUnpaddedStructType(fields[i]->type);
     } else {
@@ -137,7 +138,7 @@ LLType *DtoUnpaddedStructType(Type *dty) {
 /// Note: v must be a pointer to a struct, but the return value will be a
 ///       first-class struct value.
 LLValue *DtoUnpaddedStruct(Type *dty, LLValue *v) {
-  assert(dty->ty == Tstruct);
+  assert(dty->ty == TY::Tstruct);
   TypeStruct *sty = static_cast<TypeStruct *>(dty);
   VarDeclarations &fields = sty->sym->fields;
 
@@ -146,7 +147,7 @@ LLValue *DtoUnpaddedStruct(Type *dty, LLValue *v) {
   for (unsigned i = 0; i < fields.length; i++) {
     LLValue *fieldptr = DtoIndexAggregate(v, sty->sym, fields[i]);
     LLValue *fieldval;
-    if (fields[i]->type->ty == Tstruct) {
+    if (fields[i]->type->ty == TY::Tstruct) {
       // Nested structs are the only members that can contain padding
       fieldval = DtoUnpaddedStruct(fields[i]->type, fieldptr);
     } else {
@@ -159,14 +160,14 @@ LLValue *DtoUnpaddedStruct(Type *dty, LLValue *v) {
 
 /// Undo the transformation performed by DtoUnpaddedStruct, writing to lval.
 void DtoPaddedStruct(Type *dty, LLValue *v, LLValue *lval) {
-  assert(dty->ty == Tstruct);
+  assert(dty->ty == TY::Tstruct);
   TypeStruct *sty = static_cast<TypeStruct *>(dty);
   VarDeclarations &fields = sty->sym->fields;
 
   for (unsigned i = 0; i < fields.length; i++) {
     LLValue *fieldptr = DtoIndexAggregate(lval, sty->sym, fields[i]);
     LLValue *fieldval = DtoExtractValue(v, i);
-    if (fields[i]->type->ty == Tstruct) {
+    if (fields[i]->type->ty == TY::Tstruct) {
       // Nested structs are the only members that can contain padding
       DtoPaddedStruct(fields[i]->type, fieldval, fieldptr);
     } else {
