@@ -10,6 +10,7 @@
 //===----------------------------------------------------------------------===//
 
 #include "dmd/errors.h"
+#include "dmd/expression.h"
 #include "dmd/ldcbindings.h"
 #include "dmd/mtype.h"
 #include "dmd/target.h"
@@ -30,85 +31,108 @@ TypeTuple *toArgTypes_sysv_x64(Type *t);
 TypeTuple *toArgTypes_aarch64(Type *t);
 
 namespace {
-/******************************
- * Return size of alias Mutex in druntime/src/rt/monitor_.d, or, more precisely,
- * the size of the native critical section as 2nd field in struct
- * D_CRITICAL_SECTION (after a pointer). D_CRITICAL_SECTION is pointer-size
- * aligned, so the returned field size is a multiple of pointer-size.
- */
-unsigned getCriticalSectionSize(const Param &params) {
-  const bool is64bit = params.is64bit;
+llvm::Type *getRealType(const llvm::Triple &triple) {
+  auto &ctx = getGlobalContext();
 
-  // Windows: sizeof(CRITICAL_SECTION)
-  if (params.isWindows)
-    return is64bit ? 40 : 24;
+  const auto a = triple.getArch();
+  const bool anyX86 = (a == llvm::Triple::x86) || (a == llvm::Triple::x86_64);
+  const bool anyAarch64 =
+      (a == llvm::Triple::aarch64) || (a == llvm::Triple::aarch64_be);
+  const bool isAndroid = triple.getEnvironment() == llvm::Triple::Android;
 
-  // POSIX: sizeof(pthread_mutex_t)
-  // based on druntime/src/core/sys/posix/sys/types.d
-  const auto &triple = *params.targetTriple;
-
-  if (triple.isOSDarwin())
-    return is64bit ? 64 : 44;
-
-  const auto arch = triple.getArch();
-  switch (triple.getOS()) {
-  case llvm::Triple::Linux:
-    if (triple.getEnvironment() == llvm::Triple::Android) {
-      // 32-bit integer rounded up to pointer size
-      return gDataLayout->getPointerSize();
-    }
-    if (arch == llvm::Triple::aarch64 || arch == llvm::Triple::aarch64_be)
-      return 48;
-    return is64bit ? 40 : 24;
-
-  case llvm::Triple::NetBSD:
-    return is64bit ? 48 : 28;
-
-  case llvm::Triple::FreeBSD:
-  case llvm::Triple::OpenBSD:
-  case llvm::Triple::DragonFly:
-    return gDataLayout->getPointerSize();
-
-  case llvm::Triple::Solaris:
-    return 24;
-
-  default:
-    return 0; // leads to an error whenever requested
+  // Only x86 has 80-bit extended precision.
+  // MSVC and Android/x86 use double precision, Android/x64 quadruple.
+  if (anyX86 && !triple.isWindowsMSVCEnvironment() && !isAndroid) {
+    return llvm::Type::getX86_FP80Ty(ctx);
   }
+
+  // AArch64 targets except Darwin (64-bit) use 128-bit quadruple precision.
+  // FIXME: PowerPC, SystemZ, ...
+  if ((anyAarch64 && !triple.isOSDarwin()) ||
+      (isAndroid && a == llvm::Triple::x86_64)) {
+    return llvm::Type::getFP128Ty(ctx);
+  }
+
+  // 64-bit double precision for all other targets.
+  return llvm::Type::getDoubleTy(ctx);
+}
 }
-} // anonymous namespace
 
 void Target::_init(const Param &params) {
+  this->params = &params;
+
   CTFloat::initialize();
   initFPTypeProperties();
 
   const auto &triple = *params.targetTriple;
   const bool isMSVC = triple.isWindowsMSVCEnvironment();
-  llvm::Type *const real = DtoType(Type::basic[Tfloat80]);
+
+  if (triple.isOSLinux()) {
+    os = OS_linux;
+  } else if (triple.isOSDarwin()) {
+    os = OS_OSX;
+  } else if (triple.isOSWindows()) {
+    os = OS_Windows;
+  } else if (triple.isOSFreeBSD()) {
+    os = OS_FreeBSD;
+  } else if (triple.isOSOpenBSD()) {
+    os = OS_OpenBSD;
+  } else if (triple.isOSDragonFly()) {
+    os = OS_DragonFlyBSD;
+  } else if (triple.isOSSolaris()) {
+    os = OS_Solaris;
+  }
+
+  osMajor = triple.getOSMajorVersion();
 
   ptrsize = gDataLayout->getPointerSize();
-  realsize = gDataLayout->getTypeAllocSize(real);
-  realpad = realsize - gDataLayout->getTypeStoreSize(real);
-  realalignsize = gDataLayout->getABITypeAlignment(real);
+  realType = getRealType(triple);
+  realsize = gDataLayout->getTypeAllocSize(realType);
+  realpad = realsize - gDataLayout->getTypeStoreSize(realType);
+  realalignsize = gDataLayout->getABITypeAlignment(realType);
   classinfosize = 0; // unused
   maxStaticDataSize = std::numeric_limits<unsigned long long>::max();
 
-  c.longsize = params.is64bit && !isMSVC ? 8 : 4;
+  c.longsize = (ptrsize == 8) && !isMSVC ? 8 : 4;
   c.long_doublesize = realsize;
-  c.criticalSectionSize = getCriticalSectionSize(params);
+  c.wchar_tsize = triple.isOSWindows() ? 2 : 4;
+  c.bitFieldStyle =
+      isMSVC ? TargetC::BitFieldStyle::MS : TargetC::BitFieldStyle::Gcc_Clang;
 
   cpp.reverseOverloads = isMSVC; // according to DMD, only for MSVC++
   cpp.exceptions = true;
   cpp.twoDtorInVtable = !isMSVC;
+  cpp.wrapDtorInExternD = triple.getArch() == llvm::Triple::x86;
 
-  objc.supported = params.hasObjectiveC;
+  objc.supported = objc_isSupported(triple);
 
   const llvm::StringRef archName = triple.getArchName();
   architectureName = {archName.size(), archName.data()};
 
+  is64bit = triple.isArch64Bit();
+  isLP64 = gDataLayout->getPointerSizeInBits() == 64;
+  run_noext = !triple.isOSWindows();
+  mscoff = isMSVC;
+
+  if (isMSVC) {
+    obj_ext = {3, "obj"};
+    lib_ext = {3, "lib"};
+  } else {
+    obj_ext = {1, "o"};
+    lib_ext = {1, "a"};
+  }
+
+  if (triple.isOSWindows()) {
+    dll_ext = {3, "dll"};
+  } else if (triple.isOSDarwin()) {
+    dll_ext = {5, "dylib"};
+  } else {
+    dll_ext = {2, "so"};
+  }
+
   // Finalize RealProperties for the target's `real` type.
 
-  const auto targetRealSemantics = &real->getFltSemantics();
+  const auto targetRealSemantics = &realType->getFltSemantics();
   const auto IEEEdouble = &APFloat::IEEEdouble();
   const auto x87DoubleExtended = &APFloat::x87DoubleExtended();
   const auto IEEEquad = &APFloat::IEEEquad();
@@ -162,7 +186,7 @@ void Target::_init(const Param &params) {
  */
 unsigned Target::alignsize(Type *type) {
   assert(type->isTypeBasic());
-  if (type->ty == Tvoid) {
+  if (type->ty == TY::Tvoid) {
     return 1;
   }
   return gDataLayout->getABITypeAlignment(DtoType(type));
@@ -188,7 +212,7 @@ Type *Target::va_listType(const Loc &loc, Scope *sc) {
  *      null if unhandled
  */
 const char *TargetCPP::typeMangle(Type *t) {
-  if (t->ty == Tfloat80) {
+  if (t->ty == TY::Tfloat80) {
     const auto &triple = *global.params.targetTriple;
     // `long double` on Android/x64 is __float128 and mangled as `g`
     bool isAndroidX64 = triple.getEnvironment() == llvm::Triple::Android &&
@@ -214,6 +238,8 @@ bool Target::isReturnOnStack(TypeFunction *tf, bool needsThis) {
   return gABI->returnInArg(tf, needsThis);
 }
 
+bool Target::preferPassByRef(Type *t) { return gABI->preferPassByRef(t); }
+
 Expression *Target::getTargetInfo(const char *name_, const Loc &loc) {
   const llvm::StringRef name(name_);
   const auto &triple = *global.params.targetTriple;
@@ -257,12 +283,14 @@ Expression *Target::getTargetInfo(const char *name_, const Loc &loc) {
     return createStringExp(cppRuntimeLibrary);
   }
 
-  if (name == "cppStd")
-    return createIntegerExp(static_cast<unsigned>(global.params.cplusplus));
+  if (name == "cppStd") {
+    return IntegerExp::create(
+        Loc(), static_cast<unsigned>(global.params.cplusplus), Type::tint32);
+  }
 
 #if LDC_LLVM_SUPPORTED_TARGET_SPIRV || LDC_LLVM_SUPPORTED_TARGET_NVPTX
   if (name == "dcomputeTargets") {
-    Expressions* exps = new Expressions();
+    Expressions* exps = createExpressions();
     for (auto &targ : opts::dcomputeTargets) {
         exps->push(createStringExp(mem.xstrdup(targ.c_str())));
     }
@@ -277,3 +305,9 @@ Expression *Target::getTargetInfo(const char *name_, const Loc &loc) {
 
   return nullptr;
 }
+
+bool Target::isCalleeDestroyingArgs(TypeFunction* tf) {
+  // callEE for extern(D) and MSVC++; callER for non-MSVC extern(C++)
+  return global.params.targetTriple->isWindowsMSVCEnvironment() ||
+         tf->linkage != LINK::cpp;
+}
