@@ -40,18 +40,15 @@ IrFuncTy &DtoIrTypeFunction(DValue *fnval) {
     }
   }
 
-  Type *type = stripModifiers(fnval->type->toBasetype());
-  DtoType(type);
-  assert(type->ctype);
-  return type->ctype->getIrFuncTy();
+  return getIrType(fnval->type->toBasetype(), true)->getIrFuncTy();
 }
 
 TypeFunction *DtoTypeFunction(DValue *fnval) {
   Type *type = fnval->type->toBasetype();
-  if (type->ty == Tfunction) {
+  if (type->ty == TY::Tfunction) {
     return static_cast<TypeFunction *>(type);
   }
-  if (type->ty == Tdelegate) {
+  if (type->ty == TY::Tdelegate) {
     // FIXME: There is really no reason why the function type should be
     // unmerged at this stage, but the frontend still seems to produce such
     // cases; for example for the uint(uint) next type of the return type of
@@ -66,7 +63,7 @@ TypeFunction *DtoTypeFunction(DValue *fnval) {
     // root cause.
 
     Type *next = merge(type->nextOf());
-    assert(next->ty == Tfunction);
+    assert(next->ty == TY::Tfunction);
     return static_cast<TypeFunction *>(next);
   }
 
@@ -77,10 +74,10 @@ TypeFunction *DtoTypeFunction(DValue *fnval) {
 
 LLValue *DtoCallableValue(DValue *fn) {
   Type *type = fn->type->toBasetype();
-  if (type->ty == Tfunction) {
+  if (type->ty == TY::Tfunction) {
     return DtoRVal(fn);
   }
-  if (type->ty == Tdelegate) {
+  if (type->ty == TY::Tdelegate) {
     if (fn->isLVal()) {
       LLValue *dg = DtoLVal(fn);
       LLValue *funcptr = DtoGEP(dg, 0, 1);
@@ -136,7 +133,11 @@ static void addExplicitArguments(std::vector<LLValue *> &args, AttrSet &attrs,
 
     llvm::AttrBuilder initialAttrs;
     if (passByVal) {
+#if LDC_LLVM_VER >= 1200
+      initialAttrs.addByValAttr(DtoType(argType));
+#else
       initialAttrs.addAttribute(LLAttribute::ByVal);
+#endif
       if (auto alignment = DtoAlignment(argType))
         initialAttrs.addAlignmentAttr(alignment);
     } else {
@@ -182,7 +183,20 @@ static void addExplicitArguments(std::vector<LLValue *> &args, AttrSet &attrs,
     DValue *const dval = DtoArgument(formalParam, argexp);
 
     // load from lvalue/let TargetABI rewrite it/...
-    llvm::Value *llVal = irFty.putArg(*irArg, dval, argexp->isLvalue(),
+    bool isLValueExp = argexp->isLvalue();
+    // regard temporaries as rvalues here
+    if (isLValueExp) {
+      auto ae = argexp;
+      if (auto ce = ae->isCommaExp())
+        ae = ce->getTail();
+      if (auto ve = ae->isVarExp()) {
+        if (auto vd = ve->var->isVarDeclaration()) {
+          if (vd->storage_class & STCtemp)
+            isLValueExp = false;
+        }
+      }
+    }
+    llvm::Value *llVal = irFty.putArg(*irArg, dval, isLValueExp,
                                       dArgIndex == explicitDArgCount - 1);
 
     const size_t llArgIdx =
@@ -253,7 +267,8 @@ static LLValue *getTypeinfoArrayArgumentForDVarArg(Expressions *argexps,
   std::vector<LLConstant *> vtypeinfos;
   vtypeinfos.reserve(numVariadicArgs);
   for (size_t i = begin; i < numArgExps; i++) {
-    vtypeinfos.push_back(DtoTypeInfoOf((*argexps)[i]->type));
+    Expression *argExp = (*argexps)[i];
+    vtypeinfos.push_back(DtoTypeInfoOf(argExp->loc, argExp->type));
   }
 
   // apply initializer
@@ -350,7 +365,7 @@ bool DtoLowerMagicIntrinsic(IRState *p, FuncDeclaration *fndecl, CallExp *e,
     }
     Expression *exp = (*e->arguments)[0];
     DValue *expv = toElem(exp);
-    if (expv->type->toBasetype()->ty != Tint32) {
+    if (expv->type->toBasetype()->ty != TY::Tint32) {
       expv = DtoCast(e->loc, expv, Type::tint32);
     }
     result = new DImValue(e->type,
@@ -457,7 +472,7 @@ bool DtoLowerMagicIntrinsic(IRState *p, FuncDeclaration *fndecl, CallExp *e,
       e->error("`cmpxchg` instruction expects 6 arguments");
       fatal();
     }
-    if (e->type->ty != Tstruct) {
+    if (e->type->ty != TY::Tstruct) {
       e->error("`cmpxchg` instruction returns a struct");
       fatal();
     }
@@ -493,8 +508,12 @@ bool DtoLowerMagicIntrinsic(IRState *p, FuncDeclaration *fndecl, CallExp *e,
       fatal();
     }
 
-    auto ret = p->ir->CreateAtomicCmpXchg(ptr, cmp, val, successOrdering,
-                                           failureOrdering);
+    auto ret =
+      p->ir->CreateAtomicCmpXchg(ptr, cmp, val,
+#if LDC_LLVM_VER >= 1300
+                                 LLMaybeAlign(getABITypeAlign(val->getType())),
+#endif
+                                 successOrdering, failureOrdering);
     ret->setWeak(isWeak);
 
     // we return a struct; allocate on stack and store to both fields manually
@@ -539,6 +558,9 @@ bool DtoLowerMagicIntrinsic(IRState *p, FuncDeclaration *fndecl, CallExp *e,
     LLValue *val = DtoRVal(exp2);
     LLValue *ret =
         p->ir->CreateAtomicRMW(llvm::AtomicRMWInst::BinOp(op), ptr, val,
+#if LDC_LLVM_VER >= 1300
+                               LLMaybeAlign(getABITypeAlign(val->getType())),
+#endif
                                llvm::AtomicOrdering(atomicOrdering));
     result = new DImValue(exp2->type, ret);
     return true;
@@ -639,13 +661,13 @@ bool DtoLowerMagicIntrinsic(IRState *p, FuncDeclaration *fndecl, CallExp *e,
 class ImplicitArgumentsBuilder {
 public:
   ImplicitArgumentsBuilder(std::vector<LLValue *> &args, AttrSet &attrs,
-                           Loc &loc, DValue *fnval,
+                           const Loc &loc, DValue *fnval,
                            LLFunctionType *llCalleeType, Expressions *argexps,
                            Type *resulttype, LLValue *sretPointer)
       : args(args), attrs(attrs), loc(loc), fnval(fnval), argexps(argexps),
         resulttype(resulttype), sretPointer(sretPointer),
         // computed:
-        isDelegateCall(fnval->type->toBasetype()->ty == Tdelegate),
+        isDelegateCall(fnval->type->toBasetype()->ty == TY::Tdelegate),
         dfnval(fnval->isFunc()), irFty(DtoIrTypeFunction(fnval)),
         tf(DtoTypeFunction(fnval)),
         llArgTypesBegin(llCalleeType->param_begin()) {}
@@ -666,7 +688,7 @@ private:
   // passed:
   std::vector<LLValue *> &args;
   AttrSet &attrs;
-  Loc &loc;
+  const Loc &loc;
   DValue *const fnval;
   Expressions *const argexps;
   Type *const resulttype;
@@ -798,7 +820,7 @@ private:
 ////////////////////////////////////////////////////////////////////////////////
 
 // FIXME: this function is a mess !
-DValue *DtoCallFunction(Loc &loc, Type *resulttype, DValue *fnval,
+DValue *DtoCallFunction(const Loc &loc, Type *resulttype, DValue *fnval,
                         Expressions *arguments, LLValue *sretPointer) {
   IF_LOG Logger::println("DtoCallFunction()");
   LOG_SCOPE
@@ -889,7 +911,7 @@ DValue *DtoCallFunction(Loc &loc, Type *resulttype, DValue *fnval,
   LLValue *retllval = irFty.arg_sret ? args[sretArgIndex]
                                      : static_cast<llvm::Instruction *>(call);
   bool retValIsLVal =
-      (tf->isref() && returnTy != Tvoid) || (irFty.arg_sret != nullptr);
+      (tf->isref() && returnTy != TY::Tvoid) || (irFty.arg_sret != nullptr);
 
   if (!retValIsLVal) {
     // let the ABI transform the return value back
@@ -908,7 +930,7 @@ DValue *DtoCallFunction(Loc &loc, Type *resulttype, DValue *fnval,
     IF_LOG Logger::println("repainting return value from '%s' to '%s'",
                            returntype->toChars(), rbase->toChars());
     switch (rbase->ty) {
-    case Tarray:
+    case TY::Tarray:
       if (tf->isref()) {
         retllval = DtoBitCast(retllval, DtoType(rbase->pointerTo()));
       } else {
@@ -916,8 +938,8 @@ DValue *DtoCallFunction(Loc &loc, Type *resulttype, DValue *fnval,
       }
       break;
 
-    case Tsarray:
-      if (nextbase->ty == Tvector && !tf->isref()) {
+    case TY::Tsarray:
+      if (nextbase->ty == TY::Tvector && !tf->isref()) {
         if (retValIsLVal) {
           retllval = DtoBitCast(retllval, DtoType(rbase->pointerTo()));
         } else {
@@ -931,9 +953,9 @@ DValue *DtoCallFunction(Loc &loc, Type *resulttype, DValue *fnval,
       }
       goto unknownMismatch;
 
-    case Tclass:
-    case Taarray:
-    case Tpointer:
+    case TY::Tclass:
+    case TY::Taarray:
+    case TY::Tpointer:
       if (tf->isref()) {
         retllval = DtoBitCast(retllval, DtoType(rbase->pointerTo()));
       } else {
@@ -941,8 +963,8 @@ DValue *DtoCallFunction(Loc &loc, Type *resulttype, DValue *fnval,
       }
       break;
 
-    case Tstruct:
-      if (nextbase->ty == Taarray && !tf->isref()) {
+    case TY::Tstruct:
+      if (nextbase->ty == TY::Taarray && !tf->isref()) {
         // In the D2 frontend, the associative array type and its
         // object.AssociativeArray representation are used
         // interchangably in some places. However, AAs are returned
@@ -1013,9 +1035,15 @@ DValue *DtoCallFunction(Loc &loc, Type *resulttype, DValue *fnval,
     call->setCallingConv(callconv);
   }
   // merge in function attributes set in callOrInvoke
+#if LDC_LLVM_VER >= 1400
+  attrlist = attrlist.addFnAttributes(
+    gIR->context(),
+    llvm::AttrBuilder(call->getAttributes(), LLAttributeList::FunctionIndex));
+#else
   attrlist = attrlist.addAttributes(
-      gIR->context(), LLAttributeList::FunctionIndex,
-      llvm::AttrBuilder(call->getAttributes(), LLAttributeList::FunctionIndex));
+    gIR->context(), LLAttributeList::FunctionIndex,
+    llvm::AttrBuilder(call->getAttributes(), LLAttributeList::FunctionIndex));
+#endif
   call->setAttributes(attrlist);
 
   // Special case for struct constructor calls: For temporaries, using the
@@ -1032,7 +1060,7 @@ DValue *DtoCallFunction(Loc &loc, Type *resulttype, DValue *fnval,
     return new DLValue(resulttype, retllval);
   }
 
-  if (rbase->ty == Tarray) {
+  if (rbase->ty == TY::Tarray) {
     return new DSliceValue(resulttype, retllval);
   }
 
