@@ -15,7 +15,6 @@
 #include "dmd/id.h"
 #include "dmd/identifier.h"
 #include "dmd/init.h"
-#include "dmd/ldcbindings.h"
 #include "dmd/module.h"
 #include "dmd/mtype.h"
 #include "dmd/root/port.h"
@@ -236,7 +235,7 @@ public:
       // Thus, dump the result to a temporary stack slot (created in the entry
       // bb) if it is not guaranteed to dominate the end bb after possibly
       // adding more control flow.
-      if (result && result->type->ty != Tvoid &&
+      if (result && result->type->ty != TY::Tvoid &&
           !result->definedInFuncEntryBB()) {
         if (result->isRVal()) {
           LLValue *lval = DtoAllocaDump(result, ".toElemRValResult");
@@ -354,13 +353,13 @@ public:
       switch (e->type->toBasetype()->ty) {
       default:
         llvm_unreachable("Unexpected complex floating point type");
-      case Tcomplex32:
+      case TY::Tcomplex32:
         c = DtoConstFP(Type::tfloat32, ldouble(0));
         break;
-      case Tcomplex64:
+      case TY::Tcomplex64:
         c = DtoConstFP(Type::tfloat64, ldouble(0));
         break;
-      case Tcomplex80:
+      case TY::Tcomplex80:
         c = DtoConstFP(Type::tfloat80, ldouble(0));
         break;
       }
@@ -380,29 +379,22 @@ public:
     LOG_SCOPE;
 
     Type *dtype = e->type->toBasetype();
-    Type *cty = dtype->nextOf()->toBasetype();
-    LLType *ct = DtoMemType(cty);
 
     llvm::GlobalVariable *gvar = p->getCachedStringLiteral(e);
+    LLConstant *arrptr = DtoGEP(gvar, 0u, 0u);
 
-    llvm::ConstantInt *zero =
-        LLConstantInt::get(LLType::getInt32Ty(gIR->context()), 0, false);
-    LLConstant *idxs[2] = {zero, zero};
-    LLConstant *arrptr = llvm::ConstantExpr::getGetElementPtr(
-        isaPointer(gvar)->getElementType(), gvar, idxs, true);
-
-    if (dtype->ty == Tarray) {
+    if (dtype->ty == TY::Tarray) {
       LLConstant *clen =
           LLConstantInt::get(DtoSize_t(), e->numberOfCodeUnits(), false);
       result = new DSliceValue(e->type, DtoConstSlice(clen, arrptr, dtype));
-    } else if (dtype->ty == Tsarray) {
+    } else if (dtype->ty == TY::Tsarray) {
+      Type *cty = dtype->nextOf()->toBasetype();
+      LLType *ct = DtoMemType(cty);
       LLType *dstType =
           getPtrToType(LLArrayType::get(ct, e->numberOfCodeUnits()));
-      LLValue *emem =
-          (gvar->getType() == dstType) ? gvar : DtoBitCast(gvar, dstType);
-      result = new DLValue(e->type, emem);
-    } else if (dtype->ty == Tpointer) {
-      result = new DImValue(e->type, arrptr);
+      result = new DLValue(e->type, DtoBitCast(gvar, dstType));
+    } else if (dtype->ty == TY::Tpointer) {
+      result = new DImValue(e->type, DtoBitCast(arrptr, DtoType(dtype)));
     } else {
       llvm_unreachable("Unknown type for StringExp.");
     }
@@ -463,7 +455,7 @@ public:
     DValue *rewrittenLhsStaticArray = nullptr;
     if (auto se = e->e1->isSliceExp()) {
       Type *sliceeBaseType = se->e1->type->toBasetype();
-      if (se->lwr == nullptr && sliceeBaseType->ty == Tsarray &&
+      if (se->lwr == nullptr && sliceeBaseType->ty == TY::Tsarray &&
           se->type->toBasetype()->nextOf() == sliceeBaseType->nextOf())
         rewrittenLhsStaticArray = toElem(se->e1, true);
     }
@@ -513,7 +505,7 @@ public:
 
     DValue *r = toElem(e->e2);
 
-    if (e->e1->type->toBasetype()->ty == Tstruct && e->e2->op == TOKint64) {
+    if (e->e1->type->toBasetype()->ty == TY::Tstruct && e->e2->op == TOKint64) {
       Logger::println("performing aggregate zero initialization");
       assert(e->e2->toInteger() == 0);
       LLValue *lval = DtoLVal(lhs);
@@ -548,8 +540,8 @@ public:
     Type *t2 = e2->type->toBasetype();
 
     // valid array ops would have been transformed by optimize
-    if ((t1->ty == Tarray || t1->ty == Tsarray) &&
-        (t2->ty == Tarray || t2->ty == Tsarray)) {
+    if ((t1->ty == TY::Tarray || t1->ty == TY::Tsarray) &&
+        (t2->ty == TY::Tarray || t2->ty == TY::Tsarray)) {
       base->error("Array operation `%s` not recognized", base->toChars());
       fatal();
     }
@@ -587,7 +579,7 @@ public:
 
   //////////////////////////////////////////////////////////////////////////////
 
-  using BinOpFunc = DValue *(Loc &, Type *, DValue *, Expression *, bool);
+  using BinOpFunc = DValue *(const Loc &, Type *, DValue *, Expression *, bool);
 
   static Expression *getLValExp(Expression *e) {
     e = skipOverCasts(e);
@@ -710,7 +702,7 @@ public:
       FuncDeclaration *fdecl = dve->var->isFuncDeclaration();
       assert(fdecl);
       Expression *thisExp = dve->e1;
-      LLValue *thisArg = thisExp->type->toBasetype()->ty == Tclass
+      LLValue *thisArg = thisExp->type->toBasetype()->ty == TY::Tclass
                              ? DtoRVal(thisExp)
                              : DtoLVal(thisExp); // when calling a struct method
       fnval = new DFuncValue(fdecl, DtoCallee(fdecl), thisArg);
@@ -728,7 +720,7 @@ public:
       // as requested by bearophile, see if it's a C printf call and that it's
       // valid.
       if (global.params.warnings != DIAGNOSTICoff && checkPrintf) {
-        if (fndecl->linkage == LINKc &&
+        if (fndecl->linkage == LINK::c &&
             strcmp(fndecl->ident->toChars(), "printf") == 0) {
           warnInvalidPrintfCall(e->loc, (*e->arguments)[0],
                                 e->arguments->length);
@@ -901,7 +893,7 @@ public:
     PGO.setCurrentStmt(e);
 
     // function pointers are special
-    if (e->type->toBasetype()->ty == Tfunction) {
+    if (e->type->toBasetype()->ty == TY::Tfunction) {
       DValue *dv = toElem(e->e1);
       LLValue *llVal = DtoRVal(dv);
       if (DFuncValue *dfv = dv->isFunc()) {
@@ -938,18 +930,18 @@ public:
     if (VarDeclaration *vd = e->var->isVarDeclaration()) {
       LLValue *arrptr;
       // indexing struct pointer
-      if (e1type->ty == Tpointer) {
-        assert(e1type->nextOf()->ty == Tstruct);
+      if (e1type->ty == TY::Tpointer) {
+        assert(e1type->nextOf()->ty == TY::Tstruct);
         TypeStruct *ts = static_cast<TypeStruct *>(e1type->nextOf());
         arrptr = DtoIndexAggregate(DtoRVal(l), ts->sym, vd);
       }
       // indexing normal struct
-      else if (e1type->ty == Tstruct) {
+      else if (e1type->ty == TY::Tstruct) {
         TypeStruct *ts = static_cast<TypeStruct *>(e1type);
         arrptr = DtoIndexAggregate(DtoLVal(l), ts->sym, vd);
       }
       // indexing class
-      else if (e1type->ty == Tclass) {
+      else if (e1type->ty == TY::Tclass) {
         TypeClass *tc = static_cast<TypeClass *>(e1type);
         arrptr = DtoIndexAggregate(DtoRVal(l), tc->sym, vd);
       } else {
@@ -965,8 +957,8 @@ public:
       // Also, private/package methods are always non-virtual.
       const bool nonFinal = !fdecl->isFinalFunc() &&
                             (fdecl->isAbstract() || fdecl->isVirtual()) &&
-                            fdecl->prot().kind != Prot::private_ &&
-                            fdecl->prot().kind != Prot::package_;
+                            fdecl->visibility.kind != Visibility::private_ &&
+                            fdecl->visibility.kind != Visibility::package_;
 
       // Get the actual function value to call.
       LLValue *funcval = nullptr;
@@ -997,22 +989,23 @@ public:
 
     VarDeclaration *vd = nullptr;
 
-    // special cases: `this(int) { this(); }` and `this(int) { super(); }`
-    if (!e->var) {
+    if (e->var) {
+      vd = e->var->isVarDeclaration();
+    } else {
+      // special cases: `this(int) { this(); }` and `this(int) { super(); }`
       Logger::println("this exp without var declaration");
       if (auto thisArg = p->func()->thisArg) {
         result = new DLValue(e->type, thisArg);
         return;
       }
       // use the inner-most parent's `vthis`
-      for (int i = p->funcGenStates.size() - 2; i >= 0; --i) {
-        if (auto vthis = p->funcGenStates[i]->irFunc.decl->vthis) {
+      for (auto fd = getParentFunc(p->func()->decl); fd;
+           fd = getParentFunc(fd)) {
+        if (auto vthis = fd->vthis) {
           vd = vthis;
           break;
         }
       }
-    } else {
-      vd = e->var->isVarDeclaration();
     }
 
     assert(vd);
@@ -1026,7 +1019,8 @@ public:
       result = new DLValue(e->type, DtoBitCast(v, DtoPtrToType(e->type)));
     } else if (vd->toParent2() != p->func()->decl) {
       Logger::println("nested this exp");
-      result = DtoNestedVariable(e->loc, e->type, vd, e->type->ty == Tstruct);
+      result =
+          DtoNestedVariable(e->loc, e->type, vd, e->type->ty == TY::Tstruct);
     } else {
       Logger::println("normal this exp");
       LLValue *v = p->func()->thisArg;
@@ -1053,19 +1047,19 @@ public:
     p->arrays.pop_back();
 
     LLValue *arrptr = nullptr;
-    if (e1type->ty == Tpointer) {
+    if (e1type->ty == TY::Tpointer) {
       arrptr = DtoGEP1(DtoRVal(l), DtoRVal(r));
-    } else if (e1type->ty == Tsarray) {
+    } else if (e1type->ty == TY::Tsarray) {
       if (p->emitArrayBoundsChecks() && !e->indexIsInBounds) {
         DtoIndexBoundsCheck(e->loc, l, r);
       }
       arrptr = DtoGEP(DtoLVal(l), DtoConstUint(0), DtoRVal(r));
-    } else if (e1type->ty == Tarray) {
+    } else if (e1type->ty == TY::Tarray) {
       if (p->emitArrayBoundsChecks() && !e->indexIsInBounds) {
         DtoIndexBoundsCheck(e->loc, l, r);
       }
       arrptr = DtoGEP1(DtoArrayPtr(l), DtoRVal(r));
-    } else if (e1type->ty == Taarray) {
+    } else if (e1type->ty == TY::Taarray) {
       result = DtoAAIndex(e->loc, e->type, l, r, e->modifiable);
       return;
     } else {
@@ -1093,10 +1087,9 @@ public:
     // evaluate the base expression but delay getting its pointer until the
     // potential bounds have been evaluated
     DValue *v = toElem(e->e1);
-    auto getBasePointer = [e, v, etype]() {
-      if (etype->ty == Tpointer) {
+    auto getBasePointer = [v, etype]() {
+      if (etype->ty == TY::Tpointer) {
         // pointer slicing
-        assert(e->lwr);
         return DtoRVal(v);
       } else {
         // array slice
@@ -1119,22 +1112,24 @@ public:
       LLValue *vup = DtoRVal(e->upr);
       p->arrays.pop_back();
 
-      const bool needCheckUpper =
-          (etype->ty != Tpointer) && !e->upperIsInBounds;
+      const bool hasLength = etype->ty != TY::Tpointer;
+      const bool needCheckUpper = hasLength && !e->upperIsInBounds;
       const bool needCheckLower = !e->lowerIsLessThanUpper;
       if (p->emitArrayBoundsChecks() && (needCheckUpper || needCheckLower)) {
         llvm::BasicBlock *okbb = p->insertBB("bounds.ok");
         llvm::BasicBlock *failbb = p->insertBBAfter(okbb, "bounds.fail");
 
-        llvm::Value *okCond = nullptr;
+        LLValue *const vlen = hasLength ? DtoArrayLen(v) : nullptr;
+
+        LLValue *okCond = nullptr;
         if (needCheckUpper) {
-          okCond = p->ir->CreateICmp(llvm::ICmpInst::ICMP_ULE, vup,
-                                     DtoArrayLen(v), "bounds.cmp.lo");
+          okCond = p->ir->CreateICmp(llvm::ICmpInst::ICMP_ULE, vup, vlen,
+                                     "bounds.cmp.up");
         }
 
         if (needCheckLower) {
-          llvm::Value *cmp = p->ir->CreateICmp(llvm::ICmpInst::ICMP_ULE, vlo,
-                                               vup, "bounds.cmp.up");
+          LLValue *cmp = p->ir->CreateICmp(llvm::ICmpInst::ICMP_ULE, vlo, vup,
+                                           "bounds.cmp.lo");
           if (okCond) {
             okCond = p->ir->CreateAnd(okCond, cmp);
           } else {
@@ -1145,7 +1140,8 @@ public:
         p->ir->CreateCondBr(okCond, okbb, failbb);
 
         p->ir->SetInsertPoint(failbb);
-        DtoBoundsCheckFailCall(p, e->loc);
+        emitArraySliceError(p, e->loc, vlo, vup,
+                            vlen ? vlen : DtoConstSize_t(0));
 
         p->ir->SetInsertPoint(okbb);
       }
@@ -1158,13 +1154,13 @@ public:
     }
     // no bounds or full slice -> just convert to slice
     else {
-      assert(etype->ty != Tpointer);
+      assert(etype->ty != TY::Tpointer);
       eptr = getBasePointer();
       // if the slicee is a static array, we use the length of that as DMD seems
       // to give contrary inconsistent sizesin some multidimensional static
       // array cases.
       // (namely default initialization, int[16][16] arr; -> int[256] arr = 0;)
-      if (etype->ty == Tsarray) {
+      if (etype->ty == TY::Tsarray) {
         TypeSArray *tsa = static_cast<TypeSArray *>(etype);
         elen = DtoConstSize_t(tsa->dim->toUInteger());
 
@@ -1177,12 +1173,12 @@ public:
     // The frontend generates a SliceExp of static array type when assigning a
     // fixed-width slice to a static array.
     Type *const ety = e->type->toBasetype();
-    if (ety->ty == Tsarray) {
+    if (ety->ty == TY::Tsarray) {
       result = new DLValue(e->type, DtoBitCast(eptr, DtoPtrToType(e->type)));
       return;
     }
 
-    assert(ety->ty == Tarray);
+    assert(ety->ty == TY::Tarray);
     if (!elen)
       elen = DtoArrayLen(v);
     eptr = DtoBitCast(eptr, DtoPtrToType(ety->nextOf()));
@@ -1207,7 +1203,7 @@ public:
 
     LLValue *eval = nullptr;
 
-    if (t->isintegral() || t->ty == Tpointer || t->ty == Tnull) {
+    if (t->isintegral() || t->ty == TY::Tpointer || t->ty == TY::Tnull) {
       llvm::ICmpInst::Predicate icmpPred;
       tokToICmpPred(e->op, isLLVMUnsigned(t), &icmpPred, &eval);
 
@@ -1243,9 +1239,9 @@ public:
         llvm_unreachable("Unsupported floating point comparison operator.");
       }
       eval = p->ir->CreateFCmp(cmpop, DtoRVal(l), DtoRVal(r));
-    } else if (t->ty == Taarray) {
+    } else if (t->ty == TY::Taarray) {
       eval = LLConstantInt::getFalse(gIR->context());
-    } else if (t->ty == Tdelegate) {
+    } else if (t->ty == TY::Tdelegate) {
       llvm::ICmpInst::Predicate icmpPred;
       tokToICmpPred(e->op, isLLVMUnsigned(t), &icmpPred, &eval);
 
@@ -1310,8 +1306,8 @@ public:
 
     // the Tclass catches interface comparisons, regular
     // class equality should be rewritten as a.opEquals(b) by this time
-    if (t->isintegral() || t->ty == Tpointer || t->ty == Tclass ||
-        t->ty == Tnull) {
+    if (t->isintegral() || t->ty == TY::Tpointer || t->ty == TY::Tclass ||
+        t->ty == TY::Tnull) {
       Logger::println("integral or pointer or interface");
       llvm::ICmpInst::Predicate cmpop;
       switch (e->op) {
@@ -1334,22 +1330,22 @@ public:
         Logger::cout() << "rv: " << *rv << '\n';
       }
       eval = p->ir->CreateICmp(cmpop, lv, rv);
-      if (t->ty == Tvector) {
+      if (t->ty == TY::Tvector) {
         eval = mergeVectorEquals(eval, e->op);
       }
     } else if (t->isfloating()) // includes iscomplex
     {
       eval = DtoBinNumericEquals(e->loc, l, r, e->op);
-    } else if (t->ty == Tsarray || t->ty == Tarray) {
+    } else if (t->ty == TY::Tsarray || t->ty == TY::Tarray) {
       Logger::println("static or dynamic array");
       eval = DtoArrayEquals(e->loc, e->op, l, r);
-    } else if (t->ty == Taarray) {
+    } else if (t->ty == TY::Taarray) {
       Logger::println("associative array");
       eval = DtoAAEquals(e->loc, e->op, l, r);
-    } else if (t->ty == Tdelegate) {
+    } else if (t->ty == TY::Tdelegate) {
       Logger::println("delegate");
       eval = DtoDelegateEquals(e->op, DtoRVal(l), DtoRVal(r));
-    } else if (t->ty == Tstruct) {
+    } else if (t->ty == TY::Tstruct) {
       Logger::println("struct");
       // when this is reached it means there is no opEquals overload.
       eval = DtoStructEquals(e->op, l, r);
@@ -1389,7 +1385,7 @@ public:
       } else if (e->op == TOKminusminus) {
         post = llvm::BinaryOperator::CreateSub(val, one, "", p->scopebb());
       }
-    } else if (e1type->ty == Tpointer) {
+    } else if (e1type->ty == TY::Tpointer) {
       assert(e->e2->op == TOKint64);
       LLConstant *offset =
           e->op == TOKplusplus ? DtoConstUint(1) : DtoConstInt(-1);
@@ -1441,13 +1437,13 @@ public:
     Type *ntype = e->newtype->toBasetype();
 
     // new class
-    if (ntype->ty == Tclass) {
+    if (ntype->ty == TY::Tclass) {
       Logger::println("new class");
       result = DtoNewClass(e->loc, static_cast<TypeClass *>(ntype), e);
       isArgprefixHandled = true; // by DtoNewClass()
     }
     // new dynamic array
-    else if (ntype->ty == Tarray) {
+    else if (ntype->ty == TY::Tarray) {
       IF_LOG Logger::println("new dynamic array: %s", e->newtype->toChars());
       assert(e->argprefix == NULL);
       // get dim
@@ -1468,27 +1464,17 @@ public:
       }
     }
     // new static array
-    else if (ntype->ty == Tsarray) {
+    else if (ntype->ty == TY::Tsarray) {
       llvm_unreachable("Static array new should decay to dynamic array.");
     }
     // new struct
-    else if (ntype->ty == Tstruct) {
+    else if (ntype->ty == TY::Tstruct) {
       IF_LOG Logger::println("new struct on heap: %s\n", e->newtype->toChars());
 
       TypeStruct *ts = static_cast<TypeStruct *>(ntype);
 
       // allocate
-      LLValue *mem = nullptr;
-      if (e->allocator) {
-        // custom allocator
-        DFuncValue dfn(e->allocator, DtoCallee(e->allocator));
-        DValue *res = DtoCallFunction(e->loc, nullptr, &dfn, e->newargs);
-        mem = DtoBitCast(DtoRVal(res), DtoType(ntype->pointerTo()),
-                         ".newstruct_custom");
-      } else {
-        // default allocator
-        mem = DtoNewStruct(e->loc, ts);
-      }
+      LLValue *mem = DtoNewStruct(e->loc, ts);
 
       if (!e->member && e->arguments) {
         IF_LOG Logger::println("Constructing using literal");
@@ -1562,16 +1548,16 @@ public:
     Type *et = e->e1->type->toBasetype();
 
     // pointer
-    if (et->ty == Tpointer) {
+    if (et->ty == TY::Tpointer) {
       Type *elementType = et->nextOf()->toBasetype();
-      if (elementType->ty == Tstruct && elementType->needsDestruction()) {
+      if (elementType->ty == TY::Tstruct && elementType->needsDestruction()) {
         DtoDeleteStruct(e->loc, dval);
       } else {
         DtoDeleteMemory(e->loc, dval);
       }
     }
     // class
-    else if (et->ty == Tclass) {
+    else if (et->ty == TY::Tclass) {
       bool onstack = false;
       TypeClass *tc = static_cast<TypeClass *>(et);
       if (tc->sym->isInterfaceDeclaration()) {
@@ -1595,7 +1581,7 @@ public:
       }
     }
     // dyn array
-    else if (et->ty == Tarray) {
+    else if (et->ty == TY::Tarray) {
       DtoDeleteArray(e->loc, dval);
       if (dval->isLVal()) {
         DtoSetArrayToNull(DtoLVal(dval));
@@ -1691,14 +1677,15 @@ public:
       Logger::println("calling class invariant");
 
       const auto fnMangle =
-          getIRMangledFuncName("_D9invariant12_d_invariantFC6ObjectZv", LINKd);
+          getIRMangledFuncName("_D9invariant12_d_invariantFC6ObjectZv", LINK::d);
       const auto fn = getRuntimeFunction(e->loc, gIR->module, fnMangle.c_str());
 
       const auto arg =
           DtoBitCast(DtoRVal(cond), fn->getFunctionType()->getParamType(0));
 
       gIR->CreateCallOrInvoke(fn, arg);
-    } else if (condty->ty == Tpointer && condty->nextOf()->ty == Tstruct) {
+    } else if (condty->ty == TY::Tpointer &&
+               condty->nextOf()->ty == TY::Tstruct) {
       const auto invDecl =
           static_cast<TypeStruct *>(condty->nextOf())->sym->inv;
       if (!invDecl)
@@ -1778,7 +1765,7 @@ public:
     p->ir->SetInsertPoint(endBB);
 
     // DMD allows stuff like `x == 0 && assert(false)`
-    if (e->type->toBasetype()->ty == Tvoid) {
+    if (e->type->toBasetype()->ty == TY::Tvoid) {
       result = nullptr;
       return;
     }
@@ -1838,7 +1825,7 @@ public:
 
     LLPointerType *int8ptrty = getPtrToType(LLType::getInt8Ty(gIR->context()));
 
-    assert(e->type->toBasetype()->ty == Tdelegate);
+    assert(e->type->toBasetype()->ty == TY::Tdelegate);
     LLType *dgty = DtoType(e->type);
 
     DValue *u = toElem(e->e1);
@@ -1903,12 +1890,12 @@ public:
     Type *t1 = e->e1->type->toBasetype();
 
     // handle dynarray specially
-    if (t1->ty == Tarray) {
+    if (t1->ty == TY::Tarray) {
       result = new DImValue(e->type, DtoDynArrayIs(e->op, l, r));
       return;
     }
     // also structs
-    if (t1->ty == Tstruct) {
+    if (t1->ty == TY::Tstruct) {
       result = new DImValue(e->type, DtoStructEquals(e->op, l, r));
       return;
     }
@@ -1916,7 +1903,7 @@ public:
     // FIXME this stuff isn't pretty
     LLValue *eval = nullptr;
 
-    if (t1->ty == Tdelegate) {
+    if (t1->ty == TY::Tdelegate) {
       LLValue *lv = DtoRVal(l);
       LLValue *rv = nullptr;
       if (!r->isNull()) {
@@ -1927,19 +1914,19 @@ public:
     } else if (t1->isfloating()) // includes iscomplex
     {
       eval = DtoBinNumericEquals(e->loc, l, r, e->op);
-    } else if (t1->ty == Tpointer || t1->ty == Tclass) {
+    } else if (t1->ty == TY::Tpointer || t1->ty == TY::Tclass) {
       LLValue *lv = DtoRVal(l);
       LLValue *rv = DtoRVal(r);
       if (lv->getType() != rv->getType()) {
         if (r->isNull()) {
-          rv = llvm::ConstantPointerNull::get(isaPointer(lv->getType()));
+          rv = llvm::ConstantPointerNull::get(isaPointer(lv));
         } else {
           rv = DtoBitCast(rv, lv->getType());
         }
       }
       eval = (e->op == TOKidentity) ? p->ir->CreateICmpEQ(lv, rv)
                                     : p->ir->CreateICmpNE(lv, rv);
-    } else if (t1->ty == Tsarray) {
+    } else if (t1->ty == TY::Tsarray) {
       LLValue *lptr = DtoLVal(l);
       LLValue *rptr = DtoLVal(r);
       assert(lptr->getType() == rptr->getType());
@@ -1951,7 +1938,7 @@ public:
       assert(lv->getType() == rv->getType());
       eval = (e->op == TOKidentity) ? p->ir->CreateICmpEQ(lv, rv)
                                     : p->ir->CreateICmpNE(lv, rv);
-      if (t1->ty == Tvector) {
+      if (t1->ty == TY::Tvector) {
         eval = mergeVectorEquals(eval,
                                  e->op == TOKidentity ? TOKequal : TOKnotequal);
       }
@@ -1985,7 +1972,7 @@ public:
 
     Type *dtype = e->type->toBasetype();
     LLValue *retPtr = nullptr;
-    if (dtype->ty != Tvoid) {
+    if (!(dtype->ty == TY::Tvoid || dtype->ty == TY::Tnoreturn)) {
       // allocate a temporary for pointer to the final result.
       retPtr = DtoAlloca(dtype->pointerTo(), "condtmp");
     }
@@ -2005,7 +1992,7 @@ public:
     p->ir->SetInsertPoint(condtrue);
     PGO.emitCounterIncrement(e);
     DValue *u = toElem(e->e1);
-    if (retPtr) {
+    if (retPtr && u->type->toBasetype()->ty != TY::Tnoreturn) {
       LLValue *lval = makeLValue(e->loc, u);
       DtoStore(lval, DtoBitCast(retPtr, lval->getType()->getPointerTo()));
     }
@@ -2013,7 +2000,7 @@ public:
 
     p->ir->SetInsertPoint(condfalse);
     DValue *v = toElem(e->e2);
-    if (retPtr) {
+    if (retPtr && v->type->toBasetype()->ty != TY::Tnoreturn) {
       LLValue *lval = makeLValue(e->loc, v);
       DtoStore(lval, DtoBitCast(retPtr, lval->getType()->getPointerTo()));
     }
@@ -2096,16 +2083,16 @@ public:
     result = toElem(e->e1);
 
     Type *e1type = e->e1->type->toBasetype();
-    assert(e1type->ty == Tarray);
+    assert(e1type->ty == TY::Tarray);
     Type *elemtype = e1type->nextOf()->toBasetype();
     Type *e2type = e->e2->type->toBasetype();
 
-    if (e1type->ty == Tarray && e2type->ty == Tdchar &&
-        (elemtype->ty == Tchar || elemtype->ty == Twchar)) {
-      if (elemtype->ty == Tchar) {
+    if (e1type->ty == TY::Tarray && e2type->ty == TY::Tdchar &&
+        (elemtype->ty == TY::Tchar || elemtype->ty == TY::Twchar)) {
+      if (elemtype->ty == TY::Tchar) {
         // append dchar to char[]
         DtoAppendDCharToString(e->loc, result, e->e2);
-      } else { /*if (elemtype->ty == Twchar)*/
+      } else { /*if (elemtype->ty == TY::Twchar)*/
         // append dchar to wchar[]
         DtoAppendDCharToUnicodeString(e->loc, result, e->e2);
       }
@@ -2123,7 +2110,7 @@ public:
 
   void genFuncLiteral(FuncLiteralDeclaration *fd, FuncExp *e) {
     if ((fd->tok == TOKreserved || fd->tok == TOKdelegate) &&
-        (e && e->type->ty == Tpointer)) {
+        (e && e->type->ty == TY::Tpointer)) {
       // This is a lambda that was inferred to be a function literal instead
       // of a delegate, so set tok here in order to get correct types/mangling.
       // Horrible hack, but DMD does the same thing.
@@ -2187,7 +2174,7 @@ public:
     Type *elemType = arrayType->nextOf()->toBasetype();
 
     // is dynamic ?
-    bool const dyn = (arrayType->ty == Tarray);
+    bool const dyn = (arrayType->ty == TY::Tarray);
     // length
     size_t const len = e->elements->length;
 
@@ -2228,9 +2215,9 @@ public:
       llvm::Value *storage =
           DtoRawAlloca(llStoType, DtoAlignment(e->type), "arrayliteral");
       initializeArrayLiteral(p, e, storage);
-      if (arrayType->ty == Tsarray) {
+      if (arrayType->ty == TY::Tsarray) {
         result = new DLValue(e->type, storage);
-      } else if (arrayType->ty == Tpointer) {
+      } else if (arrayType->ty == TY::Tpointer) {
         storage = DtoBitCast(storage, llElemType->getPointerTo());
         result = new DImValue(e->type, storage);
       } else {
@@ -2366,7 +2353,7 @@ public:
       goto LruntimeInit;
     }
 
-    if (aatype->ty != Taarray) {
+    if (aatype->ty != TY::Taarray) {
       // It's the AssociativeArray type.
       // Turn it back into a TypeAArray
       vtype = e->values->tdata()[0]->type;
@@ -2384,36 +2371,31 @@ public:
         IF_LOG Logger::println("(%llu) aa[%s] = %s",
                                static_cast<unsigned long long>(i),
                                ekey->toChars(), eval->toChars());
-        unsigned errors = global.startGagging();
-        LLConstant *ekeyConst = toConstElem(ekey, p);
-        LLConstant *evalConst = toConstElem(eval, p);
-        if (global.endGagging(errors)) {
+        LLConstant *ekeyConst = tryToConstElem(ekey, p);
+        LLConstant *evalConst = tryToConstElem(eval, p);
+        if (!ekeyConst || !evalConst) {
           goto LruntimeInit;
         }
-        assert(ekeyConst && evalConst);
         keysInits.push_back(ekeyConst);
         valuesInits.push_back(evalConst);
       }
 
-      assert(aatype->ty == Taarray);
+      assert(aatype->ty == TY::Taarray);
       Type *indexType = static_cast<TypeAArray *>(aatype)->index;
       assert(indexType && vtype);
 
       llvm::Function *func =
           getRuntimeFunction(e->loc, gIR->module, "_d_assocarrayliteralTX");
       LLFunctionType *funcTy = func->getFunctionType();
-      LLValue *aaTypeInfo =
-          DtoBitCast(DtoTypeInfoOf(stripModifiers(aatype), /*base=*/false),
-                     DtoType(getAssociativeArrayTypeInfoType()));
-
-      LLConstant *idxs[2] = {DtoConstUint(0), DtoConstUint(0)};
+      LLValue *aaTypeInfo = DtoBitCast(
+          DtoTypeInfoOf(e->loc, stripModifiers(aatype), /*base=*/false),
+          DtoType(getAssociativeArrayTypeInfoType()));
 
       LLConstant *initval = arrayConst(keysInits, indexType);
       LLConstant *globalstore = new LLGlobalVariable(
           gIR->module, initval->getType(), false,
           LLGlobalValue::InternalLinkage, initval, ".aaKeysStorage");
-      LLConstant *slice = llvm::ConstantExpr::getGetElementPtr(
-          isaPointer(globalstore)->getElementType(), globalstore, idxs, true);
+      LLConstant *slice = DtoGEP(globalstore, 0u, 0u);
       slice = DtoConstSlice(DtoConstSize_t(e->keys->length), slice);
       LLValue *keysArray = DtoAggrPaint(slice, funcTy->getParamType(1));
 
@@ -2421,14 +2403,13 @@ public:
       globalstore = new LLGlobalVariable(gIR->module, initval->getType(), false,
                                          LLGlobalValue::InternalLinkage,
                                          initval, ".aaValuesStorage");
-      slice = llvm::ConstantExpr::getGetElementPtr(
-          isaPointer(globalstore)->getElementType(), globalstore, idxs, true);
+      slice = DtoGEP(globalstore, 0u, 0u);
       slice = DtoConstSlice(DtoConstSize_t(e->keys->length), slice);
       LLValue *valuesArray = DtoAggrPaint(slice, funcTy->getParamType(2));
 
       LLValue *aa = gIR->CreateCallOrInvoke(func, aaTypeInfo, keysArray,
                                             valuesArray, "aa");
-      if (basetype->ty != Taarray) {
+      if (basetype->ty != TY::Taarray) {
         LLValue *tmp = DtoAlloca(e->type, "aaliteral");
         DtoStore(aa, DtoGEP(tmp, 0u, 0));
         result = new DLValue(e->type, tmp);
@@ -2534,7 +2515,7 @@ public:
       LLValue *gep = DtoGEP(val, 0, i);
       if (DtoIsInMemoryOnly(el->type)) {
         DtoMemCpy(gep, DtoLVal(ep));
-      } else if (el->type->ty != Tvoid) {
+      } else if (el->type->ty != TY::Tvoid) {
         DtoStoreZextI8(DtoRVal(ep), gep);
       } else {
         DtoStore(LLConstantInt::get(LLType::getInt8Ty(p->context()), 0, false),
@@ -2553,7 +2534,7 @@ public:
     const unsigned N = e->dim;
 
     Type *elementType = type->elementType();
-    if (elementType->ty == Tvoid)
+    if (elementType->ty == TY::Tvoid)
       elementType = Type::tuns8;
 
     const auto getCastElement = [e, elementType](DValue *element) {
@@ -2590,11 +2571,12 @@ public:
           DtoStore(llElements[i], DtoGEP(dstMem, 0, i));
         }
       }
-    } else if (tsrc->ty == Tarray || tsrc->ty == Tsarray) {
+    } else if (tsrc->ty == TY::Tarray || tsrc->ty == TY::Tsarray) {
       // Arrays: prefer a memcpy if the LL element types match, otherwise cast
       // and store element-wise.
       if (auto ts = tsrc->isTypeSArray()) {
         Logger::println("static array expression");
+        (void)ts;
         assert(ts->dim->toInteger() == N &&
                "Static array vector initializer length mismatch, should have "
                "been handled in frontend.");
@@ -2621,7 +2603,9 @@ public:
       DValue *val = toElem(e->e1);
       LLValue *llElement = getCastElement(val);
       if (auto llConstant = isaConstant(llElement)) {
-#if LDC_LLVM_VER >= 1100
+#if LDC_LLVM_VER >= 1200
+        const auto elementCount = llvm::ElementCount::getFixed(N);
+#elif LDC_LLVM_VER >= 1100
         const auto elementCount = llvm::ElementCount(N, false);
 #else
         const auto elementCount = N;
@@ -2669,13 +2653,13 @@ public:
 
   void visit(TypeidExp *e) override {
     if (Type *t = isType(e->obj)) {
-      result = DtoSymbolAddress(
-          e->loc, e->type, getOrCreateTypeInfoDeclaration(e->loc, t, nullptr));
+      result = DtoSymbolAddress(e->loc, e->type,
+                                getOrCreateTypeInfoDeclaration(e->loc, t));
       return;
     }
     if (Expression *ex = isExpression(e->obj)) {
       Type *t = ex->type->toBasetype();
-      assert(t->ty == Tclass);
+      assert(t->ty == TY::Tclass);
 
       LLValue *val = DtoRVal(ex);
 
@@ -2742,26 +2726,16 @@ bool basetypesAreEqualWithoutModifiers(Type *l, Type *r) {
   return l->equals(r);
 }
 
-Expression *getCommaHead(CommaExp *e) {
-  auto l = e->e1;
-  for (;;) {
-    if (auto ce = l->isCommaExp())
-      l = ce->e1;
-    else
-      break;
-  }
-  return l;
-}
+VarDeclaration *isTemporaryVar(Expression *e) {
+  if (auto ce = e->isCommaExp())
+    if (auto de = ce->getHead()->isDeclarationExp())
+      if (auto vd = de->declaration->isVarDeclaration())
+        if (vd->storage_class & STCtemp)
+          if (auto ve = ce->getTail()->isVarExp())
+            if (ve->var == vd)
+              return vd;
 
-Expression *getCommaTail(CommaExp *e) {
-  auto r = e->e2;
-  for (;;) {
-    if (auto ce = r->isCommaExp())
-      r = ce->e2;
-    else
-      break;
-  }
-  return r;
+  return nullptr;
 }
 }
 
@@ -2774,13 +2748,11 @@ bool toInPlaceConstruction(DLValue *lhs, Expression *rhs) {
   // to initialize a `S[1]` lhs with a `S` rhs.
   if (auto ve = rhs->isVarExp()) {
     if (auto symdecl = ve->var->isSymbolDeclaration()) {
-      Type *t = symdecl->type->toBasetype();
-      if (auto ts = t->isTypeStruct()) {
-        // this is the static initializer for a struct (init symbol)
-        StructDeclaration *sd = ts->sym;
+      // exclude void[]-typed `__traits(initSymbol)` (LDC extension)
+      if (symdecl->type->toBasetype()->ty == TY::Tstruct) {
+        auto sd = symdecl->dsym->isStructDeclaration();
         assert(sd);
         DtoResolveStruct(sd);
-
         if (sd->zeroInit) {
           Logger::println("success, zeroing out");
           DtoMemSetZero(DtoLVal(lhs));
@@ -2813,74 +2785,53 @@ bool toInPlaceConstruction(DLValue *lhs, Expression *rhs) {
       return true;
     }
 
-    // DMD issue 17457: detect structliteral.ctor(args)
+    // detect <structliteral | temporary>.ctor(args)
     if (auto dve = ce->e1->isDotVarExp()) {
       auto fd = dve->var->isFuncDeclaration();
       if (fd && fd->isCtorDeclaration()) {
-        if (auto sle = dve->e1->isStructLiteralExp()) {
-          Logger::println("success, in-place-constructing struct literal and "
-                          "invoking ctor");
-          // emit the struct literal directly into the lhs lvalue...
-          auto lval = DtoLVal(lhs);
-          ToElemVisitor::emitStructLiteral(sle, lval);
-          // ... and invoke the ctor directly on it
-          auto fnval = new DFuncValue(fd, DtoCallee(fd), lval);
+        Logger::println("is a constructor call, checking lhs of DotVarExp");
+        if (toInPlaceConstruction(lhs, dve->e1)) {
+          Logger::println("success, calling ctor on in-place constructed lhs");
+          auto fnval = new DFuncValue(fd, DtoCallee(fd), DtoLVal(lhs));
           DtoCallFunction(ce->loc, ce->type, fnval, ce->arguments);
           return true;
         }
       }
     }
   }
-
   // emit struct literals directly into the lhs lvalue
-  if (auto sle = rhs->isStructLiteralExp()) {
+  else if (auto sle = rhs->isStructLiteralExp()) {
     Logger::println("success, in-place-constructing struct literal");
     ToElemVisitor::emitStructLiteral(sle, DtoLVal(lhs));
     return true;
   }
-
-  // static array literals too
-  Type *lhsBasetype = lhs->type->toBasetype();
-  if (auto al = rhs->isArrayLiteralExp()) {
-    if (lhsBasetype->ty == Tsarray) {
+  // and static array literals
+  else if (auto al = rhs->isArrayLiteralExp()) {
+    if (lhs->type->toBasetype()->ty == TY::Tsarray) {
       Logger::println("success, in-place-constructing array literal");
       initializeArrayLiteral(gIR, al, DtoLVal(lhs));
       return true;
     }
   }
-
-  // vector literals too
-  if (auto ve = rhs->isVectorExp()) {
+  // and vector literals
+  else if (auto ve = rhs->isVectorExp()) {
     Logger::println("success, in-place-constructing vector");
     ToElemVisitor::emitVector(ve, DtoLVal(lhs));
     return true;
   }
-
-  // temporary structs and static arrays too
-  if (DtoIsInMemoryOnly(lhsBasetype)) {
-    if (auto ce = rhs->isCommaExp()) {
-      Expression *head = getCommaHead(ce);
-      Expression *tail = getCommaTail(ce);
-
-      if (auto de = head->isDeclarationExp()) {
-        if (auto vd = de->declaration->isVarDeclaration()) {
-          if (auto ve = tail->isVarExp()) {
-            if (ve->var == vd) {
-              Logger::println("success, in-place-constructing temporary");
-              auto lhsLVal = DtoLVal(lhs);
-              auto rhsLVal = DtoLVal(rhs);
-              if (!llvm::isa<llvm::AllocaInst>(rhsLVal)) {
-                error(rhs->loc, "lvalue of temporary is not an alloca, please "
-                                "file an LDC issue");
-                fatal();
-              }
-              rhsLVal->replaceAllUsesWith(lhsLVal);
-              return true;
-            }
-          }
-        }
-      }
+  // and temporaries
+  else if (isTemporaryVar(rhs)) {
+    Logger::println("success, in-place-constructing temporary");
+    auto lhsLVal = DtoLVal(lhs);
+    auto rhsLVal = DtoLVal(rhs);
+    if (!llvm::isa<llvm::AllocaInst>(rhsLVal)) {
+      error(rhs->loc, "lvalue of temporary is not an alloca, please "
+                      "file an LDC issue");
+      fatal();
     }
+    if (lhsLVal != rhsLVal)
+      rhsLVal->replaceAllUsesWith(lhsLVal);
+    return true;
   }
 
   return false;
