@@ -16,6 +16,7 @@
 #include "dmd/id.h"
 #include "dmd/init.h"
 #include "dmd/module.h"
+#include "dmd/target.h"
 #include "driver/cl_options.h"
 #include "gen/abi.h"
 #include "gen/arrays.h"
@@ -41,12 +42,12 @@
 bool DtoIsInMemoryOnly(Type *type) {
   Type *typ = type->toBasetype();
   TY t = typ->ty;
-  return (t == Tstruct || t == Tsarray);
+  return (t == TY::Tstruct || t == TY::Tsarray);
 }
 
 bool DtoIsReturnInArg(CallExp *ce) {
   Type *t = ce->e1->type->toBasetype();
-  if (t->ty == Tfunction && (!ce->f || !DtoIsIntrinsic(ce->f))) {
+  if (t->ty == TY::Tfunction && (!ce->f || !DtoIsIntrinsic(ce->f))) {
     return gABI->returnInArg(static_cast<TypeFunction *>(t),
                              ce->f && ce->f->needThis());
   }
@@ -55,7 +56,7 @@ bool DtoIsReturnInArg(CallExp *ce) {
 
 void DtoAddExtendAttr(Type *type, llvm::AttrBuilder &attrs) {
   type = type->toBasetype();
-  if (type->isintegral() && type->ty != Tvector && type->size() <= 2) {
+  if (type->isintegral() && type->ty != TY::Tvector && type->size() <= 2) {
     attrs.addAttribute(type->isunsigned() ? LLAttribute::ZExt
                                           : LLAttribute::SExt);
   }
@@ -71,85 +72,98 @@ LLType *DtoType(Type *t) {
   IF_LOG Logger::println("Building type: %s", t->toChars());
   LOG_SCOPE;
 
-  assert(t);
   switch (t->ty) {
   // basic types
-  case Tvoid:
-  case Tint8:
-  case Tuns8:
-  case Tint16:
-  case Tuns16:
-  case Tint32:
-  case Tuns32:
-  case Tint64:
-  case Tuns64:
-  case Tint128:
-  case Tuns128:
-  case Tfloat32:
-  case Tfloat64:
-  case Tfloat80:
-  case Timaginary32:
-  case Timaginary64:
-  case Timaginary80:
-  case Tcomplex32:
-  case Tcomplex64:
-  case Tcomplex80:
-  // case Tbit:
-  case Tbool:
-  case Tchar:
-  case Twchar:
-  case Tdchar: {
+  case TY::Tvoid:
+  case TY::Tint8:
+  case TY::Tuns8:
+  case TY::Tint16:
+  case TY::Tuns16:
+  case TY::Tint32:
+  case TY::Tuns32:
+  case TY::Tint64:
+  case TY::Tuns64:
+  case TY::Tint128:
+  case TY::Tuns128:
+  case TY::Tfloat32:
+  case TY::Tfloat64:
+  case TY::Tfloat80:
+  case TY::Timaginary32:
+  case TY::Timaginary64:
+  case TY::Timaginary80:
+  case TY::Tcomplex32:
+  case TY::Tcomplex64:
+  case TY::Tcomplex80:
+  // case TY::Tbit:
+  case TY::Tbool:
+  case TY::Tchar:
+  case TY::Twchar:
+  case TY::Tdchar:
+  case TY::Tnoreturn: {
     return IrTypeBasic::get(t)->getLLType();
   }
 
   // pointers
-  case Tnull:
-  case Tpointer: {
+  case TY::Tnull:
+  case TY::Tpointer: {
     return IrTypePointer::get(t)->getLLType();
   }
 
   // arrays
-  case Tarray: {
+  case TY::Tarray: {
     return IrTypeArray::get(t)->getLLType();
   }
 
-  case Tsarray: {
+  case TY::Tsarray: {
     return IrTypeSArray::get(t)->getLLType();
   }
 
   // aggregates
-  case Tstruct: {
-    TypeStruct *ts = static_cast<TypeStruct *>(t);
-    if (ts->sym->type->ctype) {
-      // This should not happen, but the frontend seems to be buggy. Not
-      // sure if this is the best way to handle the situation, but we
-      // certainly don't want to override ts->sym->type->ctype.
-      IF_LOG Logger::cout()
-          << "Struct with multiple Types detected: " << ts->toChars() << " ("
-          << ts->sym->locToChars() << ")" << std::endl;
-      return ts->sym->type->ctype->getLLType();
+  case TY::Tstruct:
+  case TY::Tclass: {
+    const auto isStruct = t->ty == TY::Tstruct;
+    AggregateDeclaration *ad;
+    if (isStruct) {
+      ad = static_cast<TypeStruct *>(t)->sym;
+    } else {
+      ad = static_cast<TypeClass *>(t)->sym;
     }
-    return IrTypeStruct::get(ts->sym)->getLLType();
-  }
-  case Tclass: {
-    TypeClass *tc = static_cast<TypeClass *>(t);
-    if (tc->sym->type->ctype) {
-      // See Tstruct case.
-      IF_LOG Logger::cout()
-          << "Class with multiple Types detected: " << tc->toChars() << " ("
-          << tc->sym->locToChars() << ")" << std::endl;
-      return tc->sym->type->ctype->getLLType();
+    if (ad->type->ty == TY::Terror) {
+      static LLStructType *opaqueErrorType =
+          LLStructType::create(gIR->context(), Type::terror->toChars());
+      return opaqueErrorType;
     }
-    return IrTypeClass::get(tc->sym)->getLLType();
+    Type *adType = stripModifiers(ad->type);
+    if (adType->ctype) {
+      /* This should not happen, but e.g. can for aggregates whose mangled name
+       * contains a lambda which got promoted from a delegate to a function.
+       * We certainly don't want to override adType->ctype, and not associate
+       * an IrType to multiple Types either (see e.g.
+       * IrTypeStruct::resetDComputeTypes()).
+       * This means there are some aggregate Types which don't have an
+       * associated ctype, so getIrType() should always be fed with its
+       * AggregateDeclaration::type.
+       */
+      IF_LOG {
+        Logger::println("Aggregate with multiple Types detected: %s (%s)",
+                        ad->toPrettyChars(), ad->locToChars());
+        LOG_SCOPE;
+        Logger::println("Existing deco:    %s", adType->deco);
+        Logger::println("Mismatching deco: %s", t->deco);
+      }
+      return adType->ctype->getLLType();
+    }
+    return isStruct ? IrTypeStruct::get(ad->isStructDeclaration())->getLLType()
+                    : IrTypeClass::get(ad->isClassDeclaration())->getLLType();
   }
 
   // functions
-  case Tfunction: {
+  case TY::Tfunction: {
     return IrTypeFunction::get(t)->getLLType();
   }
 
   // delegates
-  case Tdelegate: {
+  case TY::Tdelegate: {
     return IrTypeDelegate::get(t)->getLLType();
   }
 
@@ -157,7 +171,7 @@ LLType *DtoType(Type *t) {
   // enum
 
   // FIXME: maybe just call toBasetype first ?
-  case Tenum: {
+  case TY::Tenum: {
     Type *bt = t->toBasetype();
     assert(bt);
     if (t == bt) {
@@ -170,12 +184,11 @@ LLType *DtoType(Type *t) {
   }
 
   // associative arrays
-  case Taarray:
+  case TY::Taarray:
     return getVoidPtrType();
 
-  case Tvector: {
+  case TY::Tvector:
     return IrTypeVector::get(t)->getLLType();
-  }
 
   default:
     llvm_unreachable("Unknown class of D Type!");
@@ -215,15 +228,24 @@ LLValue *DtoDelegateEquals(TOK op, LLValue *lhs, LLValue *rhs) {
 ////////////////////////////////////////////////////////////////////////////////
 
 LinkageWithCOMDAT DtoLinkage(Dsymbol *sym) {
-  // Function (incl. delegate) literals are emitted into each referencing
-  // compilation unit; use template linkage to prevent conflicts.
-  auto linkage = (sym->isFuncLiteralDeclaration() || DtoIsTemplateInstance(sym))
-                     ? templateLinkage
-                     : LLGlobalValue::ExternalLinkage;
-
-  // If @(ldc.attributes.weak) is applied, override the linkage to WeakAny
+  LLGlobalValue::LinkageTypes linkage = LLGlobalValue::ExternalLinkage;
   if (hasWeakUDA(sym)) {
     linkage = LLGlobalValue::WeakAnyLinkage;
+  } else {
+    // Function (incl. delegate) literals are emitted into each referencing
+    // compilation unit, so use linkonce_odr for all lambdas and all global
+    // variables they define.
+    auto potentialLambda = sym;
+    if (auto vd = sym->isVarDeclaration()) {
+      if (vd->isDataseg())
+        potentialLambda = vd->toParent2();
+    }
+
+    if (potentialLambda->isFuncLiteralDeclaration()) {
+      linkage = LLGlobalValue::LinkOnceODRLinkage;
+    } else if (sym->isInstantiated()) {
+      linkage = templateLinkage;
+    }
   }
 
   return {linkage, needsCOMDAT()};
@@ -253,8 +275,22 @@ void setLinkageAndVisibility(Dsymbol *sym, llvm::GlobalObject *obj) {
 }
 
 void setVisibility(Dsymbol *sym, llvm::GlobalObject *obj) {
-  if (opts::defaultToHiddenVisibility && !sym->isExport())
-    obj->setVisibility(LLGlobalValue::HiddenVisibility);
+  if (global.params.targetTriple->isOSWindows()) {
+    bool isExported = sym->isExport();
+    if (!isExported && global.params.dllexport) {
+      const auto l = obj->getLinkage();
+      isExported = l == LLGlobalValue::ExternalLinkage ||
+                   l == LLGlobalValue::WeakODRLinkage ||
+                   l == LLGlobalValue::WeakAnyLinkage;
+    }
+    obj->setDLLStorageClass(isExported ? LLGlobalValue::DLLExportStorageClass
+                                       : LLGlobalValue::DefaultStorageClass);
+  } else {
+    if (opts::symbolVisibility == opts::SymbolVisibility::hidden &&
+        !sym->isExport()) {
+      obj->setVisibility(LLGlobalValue::HiddenVisibility);
+    }
+  }
 }
 
 ////////////////////////////////////////////////////////////////////////////////
@@ -263,13 +299,11 @@ LLIntegerType *DtoSize_t() {
   // the type of size_t does not change once set
   static LLIntegerType *t = nullptr;
   if (t == nullptr) {
-    auto triple = global.params.targetTriple;
-
-    if (triple->isArch64Bit()) {
+    if (target.ptrsize == 8) {
       t = LLType::getInt64Ty(gIR->context());
-    } else if (triple->isArch32Bit()) {
+    } else if (target.ptrsize == 4) {
       t = LLType::getInt32Ty(gIR->context());
-    } else if (triple->isArch16Bit()) {
+    } else if (target.ptrsize == 2) {
       t = LLType::getInt16Ty(gIR->context());
     } else {
       llvm_unreachable("Unsupported size_t width");
@@ -280,12 +314,16 @@ LLIntegerType *DtoSize_t() {
 
 ////////////////////////////////////////////////////////////////////////////////
 
+LLType *getPointeeType(LLValue *pointer) {
+  return pointer->getType()->getPointerElementType();
+}
+
+////////////////////////////////////////////////////////////////////////////////
+
 namespace {
 llvm::GetElementPtrInst *DtoGEP(LLValue *ptr, llvm::ArrayRef<LLValue *> indices,
                                 const char *name, llvm::BasicBlock *bb) {
-  LLPointerType *p = isaPointer(ptr);
-  assert(p && "GEP expects a pointer type");
-  auto gep = llvm::GetElementPtrInst::Create(p->getElementType(), ptr, indices,
+  auto gep = llvm::GetElementPtrInst::Create(getPointeeType(ptr), ptr, indices,
                                              name, bb ? bb : gIR->scopebb());
   gep->setIsInBounds(true);
   return gep;
@@ -315,11 +353,8 @@ LLValue *DtoGEP(LLValue *ptr, unsigned i0, unsigned i1, const char *name,
 }
 
 LLConstant *DtoGEP(LLConstant *ptr, unsigned i0, unsigned i1) {
-  LLPointerType *p = isaPointer(ptr);
-  (void)p;
-  assert(p && "GEP expects a pointer type");
   LLValue *indices[] = {DtoConstUint(i0), DtoConstUint(i1)};
-  return llvm::ConstantExpr::getGetElementPtr(p->getElementType(), ptr, indices,
+  return llvm::ConstantExpr::getGetElementPtr(getPointeeType(ptr), ptr, indices,
                                               /* InBounds = */ true);
 }
 
@@ -428,12 +463,8 @@ LLConstant *DtoConstFP(Type *t, const real_t value) {
 
 LLConstant *DtoConstCString(const char *str) {
   llvm::StringRef s(str ? str : "");
-
   LLGlobalVariable *gvar = gIR->getCachedStringLiteral(s);
-
-  LLConstant *idxs[] = {DtoConstUint(0), DtoConstUint(0)};
-  return llvm::ConstantExpr::getGetElementPtr(gvar->getInitializer()->getType(),
-                                              gvar, idxs, true);
+  return DtoGEP(gvar, 0u, 0u);
 }
 
 LLConstant *DtoConstString(const char *str) {
