@@ -50,24 +50,23 @@
 #include "gen/runtime.h"
 #include "gen/structs.h"
 #include "gen/tollvm.h"
+#include "ir/irdsymbol.h"
 #include "ir/irtype.h"
 #include <ir/irtypeclass.h>
 #include "ir/irvar.h"
 #include <cassert>
 #include <cstdio>
 
-// defined in dmd/typinf.d:
-void genTypeInfo(Loc loc, Type *torig, Scope *sc);
-bool builtinTypeInfo(Type *t);
+void genTypeInfo(Loc loc, Type *torig, Scope *sc); // in dmd/typinf.d
 
-TypeInfoDeclaration *getOrCreateTypeInfoDeclaration(const Loc &loc, Type *torig,
-                                                    Scope *sc) {
-  IF_LOG Logger::println("Type::getTypeInfo(): %s", torig->toChars());
+TypeInfoDeclaration *getOrCreateTypeInfoDeclaration(const Loc &loc, Type *forType) {
+  IF_LOG Logger::println("getOrCreateTypeInfoDeclaration(): %s",
+                         forType->toChars());
   LOG_SCOPE
 
-  genTypeInfo(loc, torig, sc);
+  genTypeInfo(loc, forType, nullptr);
 
-  return torig->vtinfo;
+  return forType->vtinfo;
 }
 
 /* ========================================================================= */
@@ -84,44 +83,20 @@ void emitTypeInfoMetadata(LLGlobalVariable *typeinfoGlobal, Type *forType) {
   // As those types cannot appear as LLVM values, they are not interesting for
   // the optimizer passes anyway.
   Type *t = forType->toBasetype();
-  if (t->ty < Terror && t->ty != Tvoid && t->ty != Tfunction &&
-      t->ty != Tident) {
+  if (t->ty < TY::Terror && t->ty != TY::Tvoid && t->ty != TY::Tfunction &&
+      t->ty != TY::Tident) {
     const auto metaname = getMetadataName(TD_PREFIX, typeinfoGlobal);
 
-    llvm::NamedMDNode *meta = gIR->module.getNamedMetadata(metaname);
-
-    if (!meta) {
-      // Construct the fields
-      llvm::Metadata *mdVals[TD_NumFields];
-      mdVals[TD_TypeInfo] = llvm::ValueAsMetadata::get(typeinfoGlobal);
-      mdVals[TD_Type] = llvm::ConstantAsMetadata::get(
-          llvm::UndefValue::get(DtoType(forType)));
-
+    if (!gIR->module.getNamedMetadata(metaname)) {
       // Construct the metadata and insert it into the module.
-      llvm::NamedMDNode *node = gIR->module.getOrInsertNamedMetadata(metaname);
-      node->addOperand(llvm::MDNode::get(
-          gIR->context(), llvm::makeArrayRef(mdVals, TD_NumFields)));
+      auto meta = gIR->module.getOrInsertNamedMetadata(metaname);
+      auto val = llvm::UndefValue::get(DtoType(forType));
+      meta->addOperand(llvm::MDNode::get(gIR->context(),
+                                         llvm::ConstantAsMetadata::get(val)));
     }
   }
 }
 
-void DtoResolveTypeInfo(TypeInfoDeclaration *tid) {
-  if (tid->ir->isResolved()) {
-    return;
-  }
-  tid->ir->setResolved();
-
-  // TypeInfo instances (except ClassInfo ones) are always emitted as weak
-  // symbols when they are used. We call semanticTypeInfo() to make sure
-  // that the type (e.g. for structs) is semantic3'd and codegen() does not
-  // skip it on grounds of being speculative, as DtoResolveTypeInfo() means
-  // that we actually need the value somewhere else in codegen.
-  // TODO: DMD does not seem to call semanticTypeInfo() from the glue layer,
-  // so there might be a structural issue somewhere.
-  semanticTypeInfo(nullptr, tid->tinfo);
-  Declaration_codegen(tid);
-}
-
 /* ========================================================================= */
 
 namespace {
@@ -155,7 +130,7 @@ public:
 
     RTTIBuilder b(getEnumTypeInfoType());
 
-    assert(decl->tinfo->ty == Tenum);
+    assert(decl->tinfo->ty == TY::Tenum);
     TypeEnum *tc = static_cast<TypeEnum *>(decl->tinfo);
     EnumDeclaration *sd = tc->sym;
 
@@ -216,7 +191,7 @@ public:
                            decl->toChars());
     LOG_SCOPE;
 
-    assert(decl->tinfo->ty == Tsarray);
+    assert(decl->tinfo->ty == TY::Tsarray);
     TypeSArray *tc = static_cast<TypeSArray *>(decl->tinfo);
 
     RTTIBuilder b(getStaticArrayTypeInfoType());
@@ -239,7 +214,7 @@ public:
         decl->toChars());
     LOG_SCOPE;
 
-    assert(decl->tinfo->ty == Taarray);
+    assert(decl->tinfo->ty == TY::Taarray);
     TypeAArray *tc = static_cast<TypeAArray *>(decl->tinfo);
 
     RTTIBuilder b(getAssociativeArrayTypeInfoType());
@@ -277,7 +252,7 @@ public:
                            decl->toChars());
     LOG_SCOPE;
 
-    assert(decl->tinfo->ty == Tdelegate);
+    assert(decl->tinfo->ty == TY::Tdelegate);
     Type *ret_type = decl->tinfo->nextOf()->nextOf();
 
     RTTIBuilder b(getDelegateTypeInfoType());
@@ -309,14 +284,13 @@ public:
     LOG_SCOPE;
 
     // make sure interface is resolved
-    assert(decl->tinfo->ty == Tclass);
-    TypeClass *tc = static_cast<TypeClass *>(decl->tinfo);
-    DtoResolveClass(tc->sym);
+    auto cd = decl->tinfo->isTypeClass()->sym;
+    DtoResolveClass(cd);
 
     RTTIBuilder b(getInterfaceTypeInfoType());
 
-    // TypeInfo base
-    b.push_classinfo(tc->sym);
+    // TypeInfo_Class info
+    b.push(getIrAggr(cd)->getClassInfoSymbol());
 
     // finish
     b.finalize(gvar);
@@ -330,7 +304,7 @@ public:
     LOG_SCOPE;
 
     // create elements array
-    assert(decl->tinfo->ty == Ttuple);
+    assert(decl->tinfo->ty == TY::Ttuple);
     TypeTuple *tu = static_cast<TypeTuple *>(decl->tinfo);
 
     size_t dim = tu->arguments->length;
@@ -340,7 +314,7 @@ public:
     LLType *tiTy = DtoType(getTypeInfoType());
 
     for (auto arg : *tu->arguments) {
-      arrInits.push_back(DtoTypeInfoOf(arg->type));
+      arrInits.push_back(DtoTypeInfoOf(decl->loc, arg->type));
     }
 
     // build array
@@ -419,7 +393,7 @@ public:
                            decl->toChars());
     LOG_SCOPE;
 
-    assert(decl->tinfo->ty == Tvector);
+    assert(decl->tinfo->ty == TY::Tvector);
     TypeVector *tv = static_cast<TypeVector *>(decl->tinfo);
 
     RTTIBuilder b(getVectorTypeInfoType());
@@ -451,28 +425,34 @@ void buildTypeInfo(TypeInfoDeclaration *decl) {
     Logger::println("typeinfo mangle: %s", mangled);
   }
 
-  // Only declare the symbol if it isn't yet, otherwise the subtype of
-  // built-in TypeInfos (rt.typeinfo.*) may clash with the base type when
-  // compiling the rt.typeinfo.* modules.
-  const auto irMangle = getIRMangledVarName(mangled, LINKd);
+  // Only declare the symbol if it isn't yet, otherwise it may clash with an
+  // existing init symbol of a built-in TypeInfo class (in rt.util.typeinfo)
+  // when compiling the rt.util.typeinfo unittests.
+  const auto irMangle = getIRMangledVarName(mangled, LINK::d);
   LLGlobalVariable *gvar = gIR->module.getGlobalVariable(irMangle);
-  if (!gvar) {
+  const bool isBuiltin = builtinTypeInfo(forType);
+  if (gvar) {
+    assert(isBuiltin && "existing global expected to be the init symbol of a "
+                        "built-in TypeInfo");
+  } else {
     LLType *type = DtoType(decl->type)->getPointerElementType();
     // We need to keep the symbol mutable as the type is not declared as
     // immutable on the D side, and e.g. synchronized() can be used on the
     // implicit monitor.
-    gvar = declareGlobal(decl->loc, gIR->module, type, irMangle, false);
-
-    emitTypeInfoMetadata(gvar, forType);
+    const bool isConstant = false;
+    bool useDLLImport = isBuiltin && global.params.dllimport != DLLImport::none;
+    gvar = declareGlobal(decl->loc, gIR->module, type, irMangle, isConstant,
+                         false, useDLLImport);
   }
 
+  emitTypeInfoMetadata(gvar, forType);
+
   IrGlobal *irg = getIrGlobal(decl, true);
   irg->value = gvar;
 
   // check if the definition can be elided
-  if (gvar->hasInitializer() || !global.params.useTypeInfo ||
-      !Type::dtypeinfo || isSpeculativeType(forType) ||
-      builtinTypeInfo(forType)) {
+  if (isBuiltin || gvar->hasInitializer() || !global.params.useTypeInfo ||
+      !Type::dtypeinfo) {
     return;
   }
   if (auto forClassType = forType->isTypeClass()) {
@@ -493,26 +473,20 @@ class DeclareOrDefineVisitor : public Visitor {
 
   // Define struct TypeInfos as linkonce_odr in each referencing CU.
   void visit(TypeInfoStructDeclaration *decl) override {
-    auto forType = decl->tinfo->isTypeStruct();
-
-    auto irstruct = getIrAggr(forType->sym, true);
-    auto gvar = irstruct->getTypeInfoSymbol();
-
+    auto sd = decl->tinfo->isTypeStruct()->sym;
+    auto irstruct = getIrAggr(sd, true);
     IrGlobal *irg = getIrGlobal(decl, true);
-    irg->value = gvar;
+
+    auto ti = irstruct->getTypeInfoSymbol();
+    irg->value = ti;
 
     // check if the definition can be elided
-    if (gvar->hasInitializer() || irstruct->suppressTypeInfo() ||
-        isSpeculativeType(forType)) {
+    if (ti->hasInitializer() || irstruct->suppressTypeInfo()) {
       return;
     }
 
-    LLConstant *init = irstruct->getTypeInfoInit(); // might define gvar!
-
-    if (!gvar->hasInitializer()) {
-      defineGlobal(gvar, init, irstruct->aggrdecl);
-      gvar->setLinkage(TYPEINFO_LINKAGE_TYPE); // override
-    }
+    irstruct->getTypeInfoSymbol(/*define=*/true);
+    ti->setLinkage(TYPEINFO_LINKAGE_TYPE); // override
   }
 
   // Only declare class TypeInfos. They are defined once in their owning module
@@ -526,17 +500,22 @@ class DeclareOrDefineVisitor : public Visitor {
   }
 
   // Build all other TypeInfos.
-  void visit(TypeInfoDeclaration *decl) override {
-    buildTypeInfo(decl);
-  }
+  void visit(TypeInfoDeclaration *decl) override { buildTypeInfo(decl); }
 };
 } // anonymous namespace
 
-void TypeInfoDeclaration_codegen(TypeInfoDeclaration *decl) {
-  IF_LOG Logger::println("TypeInfoDeclaration_codegen(%s)",
-                         decl->toPrettyChars());
+LLGlobalVariable *DtoResolveTypeInfo(TypeInfoDeclaration *tid) {
+  IF_LOG Logger::println("DtoResolveTypeInfo(%s)", tid->toPrettyChars());
   LOG_SCOPE;
 
-  DeclareOrDefineVisitor v;
-  decl->accept(&v);
+  assert(!gIR->dcomputetarget);
+
+  if (!tid->ir->isResolved()) {
+    DeclareOrDefineVisitor v;
+    tid->accept(&v);
+
+    tid->ir->setResolved();
+  }
+
+  return llvm::cast<LLGlobalVariable>(getIrValue(tid));
 }
