@@ -10,6 +10,7 @@
 #include "ir/irtypeaggr.h"
 
 #include "dmd/aggregate.h"
+#include "dmd/errors.h"
 #include "dmd/init.h"
 #include "gen/irstate.h"
 #include "gen/logger.h"
@@ -149,7 +150,12 @@ void AggrTypeBuilder::addAggregate(
     if (!vd)
       continue;
 
-    assert(vd->offset >= m_offset && "Variable overlaps previous field.");
+    if (vd->offset < m_offset) {
+      // TODO: ImportC
+      vd->error(
+          "overlaps previous field. LDC doesn't support C bit fields yet");
+      fatal();
+    }
 
     // Add an explicit field for any padding so we can zero it, as per TDPL
     // §7.1.1.
@@ -205,17 +211,14 @@ bool IrTypeAggr::isPacked(AggregateDeclaration *ad) {
   // make it packed.
   unsigned aggregateSize = ~0u;
   unsigned aggregateAlignment = 1;
-  if (ad->sizeok == SIZEOKdone) {
+  if (ad->sizeok == Sizeok::done) {
     aggregateSize = ad->structsize;
+    aggregateAlignment = ad->alignsize;
 
-    const auto naturalAlignment = ad->alignsize;
-    auto explicitAlignment = STRUCTALIGN_DEFAULT;
-    if (auto sd = ad->isStructDeclaration())
-      explicitAlignment = sd->alignment;
-
-    aggregateAlignment = explicitAlignment == STRUCTALIGN_DEFAULT
-                             ? naturalAlignment
-                             : explicitAlignment;
+    if (auto sd = ad->isStructDeclaration()) {
+      if (!sd->alignment.isDefault() && !sd->alignment.isPack())
+        aggregateAlignment = sd->alignment.get();
+    }
   }
 
   // Classes apparently aren't padded; their size may not match the alignment.
@@ -226,16 +229,10 @@ bool IrTypeAggr::isPacked(AggregateDeclaration *ad) {
   // For unions, only a subset of the fields are actually used for the IR type -
   // don't care (about a few potentially needlessly packed IR structs).
   for (const auto field : ad->fields) {
-    const auto naturalFieldTypeAlignment = field->type->alignsize();
-    const auto explicitFieldTypeAlignment = field->type->alignment();
-    const auto fieldTypeAlignment =
-        explicitFieldTypeAlignment == STRUCTALIGN_DEFAULT
-            ? naturalFieldTypeAlignment
-            : explicitFieldTypeAlignment;
-
     // The aggregate size, aggregate alignment and the field offset need to be
     // multiples of the field type's alignment, otherwise the aggregate type is
     // unnaturally aligned, and LLVM would insert padding.
+    const unsigned fieldTypeAlignment = DtoAlignment(field->type);
     const auto mask = fieldTypeAlignment - 1;
     if ((aggregateSize | aggregateAlignment | field->offset) & mask)
       return true;
