@@ -7,7 +7,7 @@ $DMD -m${MODEL} -I${TEST_DIR} -od${RESULTS_TEST_DIR} -c ${EXTRA_FILES}/test39.d
 $DMD -m${MODEL} -I${TEST_DIR} -od${RESULTS_TEST_DIR} -c ${TEST_DIR}/imports/test39a.d
 libname=${OUTPUT_BASE}a${LIBEXT}
 
-if [ ${OS} == "win32" -o ${OS} == "win64" ]; then
+if [ ${OS} == "windows" ]; then
     $DMD -m${MODEL} -lib -of${libname} ${OUTPUT_BASE}a${OBJ}
 else
     ar -r ${libname} ${OUTPUT_BASE}a${OBJ}
@@ -17,4 +17,4 @@ $DMD -m${MODEL} -of${OUTPUT_BASE}${EXE} ${OUTPUT_BASE}${OBJ} ${libname}
 
 ${OUTPUT_BASE}${EXE}
 
-rm ${OUTPUT_BASE}{${OBJ},a${OBJ},${EXE}} ${libname}
\ No newline at end of file
+rm_retry ${OUTPUT_BASE}{${OBJ},a${OBJ},${EXE}} ${libname}
\ No newline at end of file
